@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,15 +15,14 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: render_handler_cairo.h,v 1.6 2007/07/01 10:53:48 bjacques Exp $ */
+/* $Id: render_handler_cairo.h,v 1.8 2008/01/21 20:56:05 rsavoye Exp $ */
 
 #ifndef BACKEND_RENDER_HANDLER_CAIRO_H
 #define BACKEND_RENDER_HANDLER_CAIRO_H
 
 #include "tu_config.h"
+#include <cairo/cairo.h>
+#include "render_handler.h"
 
 namespace gnash {
 namespace renderer {
@@ -35,7 +34,7 @@ namespace cairo {
 gnash::render_handler* create_handler();
 
 /// Make sure to call this before starting display
-void set_handle(cairo_t* handle);
+void set_context(render_handler* handler, cairo_t* context);
 
 } // namespace gnash::renderer::cairo
 } // namespace gnash::renderer
