@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,15 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
-/* $Id: aqua_ogl_glue.h,v 1.8 2007/07/23 01:04:18 nihilus Exp $ */
+/* $Id: aqua_ogl_glue.h,v 1.10 2008/01/21 20:55:41 rsavoye Exp $ */
 
 #ifndef AQUA_OGL_GLUE_H
 #define AQUA_OGL_GLUE_H
 
 #include "aqua_glue.h"
+#include <AGL/agl.h>
 
 using namespace std;
 
@@ -38,14 +36,15 @@ class AquaOglGlue : public AquaGlue
 
     bool init(int argc, char ***argv);
     render_handler* createRenderHandler();
-    bool prepDrawingArea(int width, int height);
+    bool prepDrawingArea(int width, int height, AGLDrawable drawable);
     void setInvalidatedRegions(const InvalidatedRanges& /* ranges */);
     void render();
    private:
 #ifdef FIX_I810_LOD_BIAS
     float _tex_lod_bias;
 #endif
+    AGLContext _context;
 };
 }
 
-#endif /* AQUA_OGL_GLUE_H */
\ No newline at end of file
+#endif /* AQUA_OGL_GLUE_H */
