@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "gtk_glue_cairo.h"
 #include "render_handler_cairo.h"
 
@@ -59,7 +56,9 @@ GtkCairoGlue::prepDrawingArea(GtkWidget *drawing_area)
 render_handler*
 GtkCairoGlue::createRenderHandler()
 {
-    return renderer::cairo::create_handler();
+    _renderer = renderer::cairo::create_handler();
+
+    return _renderer;
 }
 
 void
@@ -80,18 +79,15 @@ GtkCairoGlue::configure(GtkWidget *const /*widget*/,
     if (!_drawing_area)  return;
 
     // Create cairo handle for output window
-    if (_cairo_handle)  cairo_destroy(_cairo_handle);
     _cairo_handle = gdk_cairo_create(_drawing_area->window);
-    assert(_cairo_handle);
 
     // Create offscreen image for rendering
-    if (_cairo_offscreen)  cairo_destroy(_cairo_offscreen);
     cairo_surface_t* surface = cairo_image_surface_create(
-	CAIRO_FORMAT_RGB24, event->width, event->height);
+      CAIRO_FORMAT_ARGB32, event->width, event->height);
     _cairo_offscreen = cairo_create(surface);
-    assert(_cairo_offscreen);
     cairo_surface_destroy(surface);
-    renderer::cairo::set_handle(_cairo_offscreen);
+
+    renderer::cairo::set_context(_renderer, _cairo_offscreen);
 }
 
 
