@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,61 +15,51 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef __KDESUP_H__
 #define __KDESUP_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
 #include "gui.h"
 
-#if 0
-#include <qobject.h>
-#include <qgl.h>
-#include <qwidget.h>
-
-#include <qtimer.h>
-#include <qapplication.h>
-#include <qeventloop.h>
-#include <qlabel.h>
-#include <qevent.h>
-#include <qkeycode.h>
-
-#endif
-
 #include <qapplication.h>
 #include <qpopupmenu.h>
 
 #ifdef RENDERER_OPENGL
 # include <qgl.h>
 # include "kde_glue_opengl.h"
-//#elif defined(RENDERER_CAIRO)
+# define WIDGETCLASS QGLWidget
+# define GLUE KdeOpenGLGlue
+#elif defined(RENDERER_CAIRO)
 // #include <cairo.h>
 // #include "kde_glue_cairo.h"
-//# error "Cairo not supported yet for KDE!"
-#else
-# error "KDE gui only supports OpenGL renderer!"
+# error "Cairo not supported yet for KDE!"
+#elif defined(RENDERER_AGG)
+# include "kde_glue_agg.h"
+# define WIDGETCLASS QWidget
+# define GLUE KdeAggGlue
 #endif
 
+
 namespace gnash
 {
 
 
 class KdeGui;
 
-class DSOEXPORT qwidget : public QGLWidget
+class DSOEXPORT qwidget : public WIDGETCLASS
 {
     Q_OBJECT
 public:
     qwidget(KdeGui* godfather);
     void setInterval(unsigned int interval);
 protected:
+#if QT_VERSION > 2312
     void contextMenuEvent(QContextMenuEvent *event);
+#endif
     void mousePressEvent(QMouseEvent *event);
     void mouseReleaseEvent( QMouseEvent * );
     void mouseMoveEvent(QMouseEvent *event);
@@ -77,8 +67,10 @@ protected:
     void keyReleaseEvent(QKeyEvent *event);
     void timerEvent(QTimerEvent *);
     void resizeEvent(QResizeEvent *event);
+    void paintEvent (QPaintEvent *event);
 public slots:
     void menuitem_restart_callback();
+    void menuitem_refresh_callback();
     void menuitem_quit_callback();
     void menuitem_play_callback();
     void menuitem_pause_callback();
@@ -108,12 +100,14 @@ public:
     virtual void setInterval(unsigned int interval);
     virtual void setTimeout(unsigned int timeout);
     virtual void handleKeyEvent(QKeyEvent *event, bool down);
+    virtual void setCursor(gnash_cursor_type newcursor);
+    void setInvalidatedRegions(const InvalidatedRanges& ranges);
+    void resize(int width, int height);
+    void quit();
  private:
-    QApplication*  _qapp;
-    qwidget*       _qwidget;
-    KdeOpenGLGlue  _glue;    
-
-    QTimer        *_timer;
+    std::auto_ptr<QApplication>  _qapp;
+    std::auto_ptr<qwidget>       _qwidget;
+    GLUE                         _glue;    
 
     gnash::key::code qtToGnashKey(QKeyEvent *event);
     int qtToGnashModifier(Qt::ButtonState state);
