@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 #include "sdl_ogl_glue.h"
 #include "tu_opengl_includes.h"
 #include "SDL.h"
@@ -82,7 +79,7 @@ SdlOglGlue::createRenderHandler(int depth)
 }
 
 bool
-SdlOglGlue::prepDrawingArea(int width, int height, uint32_t sdl_flags)
+SdlOglGlue::prepDrawingArea(int width, int height, boost::uint32_t sdl_flags)
 {
     if (_bpp == 16) {
       // 16-bit color, surface creation is likely to succeed.
