@@ -1,6 +1,6 @@
 // FLVParser.h:  Flash Video file format parser, for Gnash.
 //
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-// $Id: FLVParser.h,v 1.18 2007/07/08 20:27:39 martinwguy Exp $
+// $Id: FLVParser.h,v 1.22 2008/01/21 20:55:43 rsavoye Exp $
 
 // Information about the FLV format can be found at http://osflash.org/flv
 
@@ -30,49 +30,16 @@
 
 namespace gnash {
 
-enum videoCodecType
-{
-	VIDEO_CODEC_H263 = 2,	// H263/SVQ3 video codec
-	VIDEO_CODEC_SCREENVIDEO = 3,	// Screenvideo codec
-	VIDEO_CODEC_VP6 = 4,		// On2 VP6 video codec
-	VIDEO_CODEC_VP6A = 5,		// On2 VP6 Alpha video codec
-	VIDEO_CODEC_SCREENVIDEO2 = 6	// Screenvideo2 codec
-};
-
-enum audioCodecType
-{
-	AUDIO_CODEC_RAW = 0,		// unspecified format.  Useful for 8-bit sounds???
-	AUDIO_CODEC_ADPCM = 1,	// gnash doesn't pass this through; it uncompresses and sends FORMAT_NATIVE16
-	AUDIO_CODEC_MP3 = 2,
-	AUDIO_CODEC_UNCOMPRESSED = 3,	// 16 bits/sample, little-endian
-	AUDIO_CODEC_NELLYMOSER_8HZ_MONO = 5,	// According to ffmpeg
-	AUDIO_CODEC_NELLYMOSER = 6	// Mystery proprietary format; see nellymoser.com
-};
-
-enum tagType
-{
-	AUDIO_TAG = 0x08,
-	VIDEO_TAG = 0x09,
-	META_TAG = 0x12
-};
-
-enum videoFrameType
-{
-	KEY_FRAME = 1,
-	INTER_FRAME = 2,
-	DIS_INTER_FRAME = 3
-};
-
 /// \brief
 /// The FLVFrame class contains a video or audio frame, its size, its
 /// timestamp,
 class FLVFrame
 {
 public:
-	uint32_t dataSize;
-	uint8_t* data;
-	uint64_t timestamp;
-	uint8_t tag;
+	boost::uint32_t dataSize;
+	boost::uint8_t* data;
+	boost::uint64_t timestamp;
+	boost::uint8_t tag;
 };
 
 /// \brief
@@ -83,7 +50,7 @@ public:
 class FLVAudioInfo
 {
 public:
-	FLVAudioInfo(uint16_t codeci, uint16_t sampleRatei, uint16_t sampleSizei, bool stereoi, uint64_t durationi)
+	FLVAudioInfo(boost::uint16_t codeci, boost::uint16_t sampleRatei, boost::uint16_t sampleSizei, bool stereoi, boost::uint64_t durationi)
 		: codec(codeci),
 		sampleRate(sampleRatei),
 		sampleSize(sampleSizei),
@@ -92,11 +59,11 @@ public:
 		{
 		}
 
-	uint16_t codec;
-	uint16_t sampleRate;
-	uint16_t sampleSize;
+	boost::uint16_t codec;
+	boost::uint16_t sampleRate;
+	boost::uint16_t sampleSize;
 	bool stereo;
-	uint64_t duration;
+	boost::uint64_t duration;
 };
 
 /// \brief
@@ -107,7 +74,7 @@ public:
 class FLVVideoInfo
 {
 public:
-	FLVVideoInfo(uint16_t codeci, uint16_t widthi, uint16_t heighti, uint16_t frameRatei, uint64_t durationi)
+	FLVVideoInfo(boost::uint16_t codeci, boost::uint16_t widthi, boost::uint16_t heighti, boost::uint16_t frameRatei, boost::uint64_t durationi)
 		: codec(codeci),
 		width(widthi),
 		height(heighti),
@@ -116,28 +83,28 @@ public:
 		{
 		}
 
-	uint16_t codec;
-	uint16_t width;
-	uint16_t height;
-	uint16_t frameRate;
-	uint64_t duration;
+	boost::uint16_t codec;
+	boost::uint16_t width;
+	boost::uint16_t height;
+	boost::uint16_t frameRate;
+	boost::uint64_t duration;
 };
 
 
 class FLVVideoFrame
 {
 public:
-	uint16_t frameType;
-	uint32_t dataSize;
-	uint64_t dataPosition;
+	boost::uint16_t frameType;
+	boost::uint32_t dataSize;
+	boost::uint64_t dataPosition;
 
 	/// in milliseconds 
-	uint32_t timestamp;
+	boost::uint32_t timestamp;
 
 	/// Return true if this video frame is a key frame
 	bool isKeyFrame() const
 	{
-		return frameType == KEY_FRAME;
+		return frameType == 1 /*KEY_FRAME*/;
 	}
 
 };
@@ -145,11 +112,11 @@ public:
 class FLVAudioFrame
 {
 public:
-	uint32_t dataSize;
-	uint64_t dataPosition;
+	boost::uint32_t dataSize;
+	boost::uint64_t dataPosition;
 
 	/// in milliseconds 
-	uint32_t timestamp;
+	boost::uint32_t timestamp;
 
 };
 
@@ -170,6 +137,40 @@ class DSOEXPORT FLVParser
 
 public:
 
+	enum videoCodecType
+	{
+		VIDEO_CODEC_H263 = 2,	// H263/SVQ3 video codec
+		VIDEO_CODEC_SCREENVIDEO = 3,	// Screenvideo codec
+		VIDEO_CODEC_VP6 = 4,		// On2 VP6 video codec
+		VIDEO_CODEC_VP6A = 5,		// On2 VP6 Alpha video codec
+		VIDEO_CODEC_SCREENVIDEO2 = 6	// Screenvideo2 codec
+	};
+
+	enum audioCodecType
+	{
+		AUDIO_CODEC_RAW = 0,		// unspecified format.  Useful for 8-bit sounds???
+		AUDIO_CODEC_ADPCM = 1,	// gnash doesn't pass this through; it uncompresses and sends FORMAT_NATIVE16
+		AUDIO_CODEC_MP3 = 2,
+		AUDIO_CODEC_UNCOMPRESSED = 3,	// 16 bits/sample, little-endian
+		AUDIO_CODEC_NELLYMOSER_8HZ_MONO = 5,	// According to ffmpeg
+		AUDIO_CODEC_NELLYMOSER = 6	// Mystery proprietary format; see nellymoser.com
+	};
+
+	enum tagType
+	{
+		AUDIO_TAG = 0x08,
+		VIDEO_TAG = 0x09,
+		META_TAG = 0x12
+	};
+
+	enum videoFrameType
+	{
+		KEY_FRAME = 1,
+		INTER_FRAME = 2,
+		DIS_INTER_FRAME = 3
+	};
+
+
 	/// \brief
 	/// Create an FLV parser reading input from
 	/// the given LoadThread
@@ -245,7 +246,7 @@ public:
 	/// @param time
 	///	Timestamp, in milliseconds.
 	///
-	bool isTimeLoaded(uint32_t time);
+	bool isTimeLoaded(boost::uint32_t time);
 
 	/// \brief
 	/// Seeks to the closest possible position the given position,
@@ -253,14 +254,14 @@ public:
 	//
 	/// Locks the _mutex
 	///
-	uint32_t seek(uint32_t);
+	boost::uint32_t seek(boost::uint32_t);
 
 	/// Returns the framedelay from the last to the current
 	/// audioframe in milliseconds. This is used for framerate.
 	//
 	/// Locks the _mutex
 	///
-	uint32_t audioFrameDelay();
+	boost::uint32_t audioFrameDelay();
 
 	/// \brief
 	/// Returns the framedelay from the last to the current
@@ -268,13 +269,13 @@ public:
 	//
 	/// Locks the _mutex
 	///
-	uint32_t videoFrameDelay();
+	boost::uint32_t videoFrameDelay();
 
 	/// Returns the framerate of the video
 	//
 	/// Locks the _mutex
 	///
-	uint16_t videoFrameRate();
+	boost::uint16_t videoFrameRate();
 
 	/// Returns the "bufferlength", meaning the differens between the
 	/// current frames timestamp and the timestamp of the last parseable
@@ -282,17 +283,17 @@ public:
 	//
 	/// Locks the _mutex
 	///
-	uint32_t getBufferLength();
+	boost::uint32_t getBufferLength();
 
 private:
 
 	/// seeks to the closest possible position the given position,
 	/// and returns the new position.
-	uint32_t seekAudio(uint32_t time);
+	boost::uint32_t seekAudio(boost::uint32_t time);
 
 	/// seeks to the closest possible position the given position,
 	/// and returns the new position.
-	uint32_t seekVideo(uint32_t time);
+	boost::uint32_t seekVideo(boost::uint32_t time);
 
 
 	/// Parses next frame from the file, returns true is a frame
@@ -303,7 +304,7 @@ private:
 	bool parseHeader();
 
 	// Functions used to extract numbers from the file
-	inline uint32_t getUInt24(uint8_t* in);
+	inline boost::uint32_t getUInt24(boost::uint8_t* in);
 
 	/// The interface to the file, externally owned
 	LoadThread& _lt;
@@ -319,7 +320,7 @@ private:
 	AudioFrames _audioFrames;
 
 	/// The position where the parsing should continue from.
-	uint64_t _lastParsedPosition;
+	boost::uint64_t _lastParsedPosition;
 
 	/// Whether the parsing is complete or not
 	bool _parsingComplete;
