@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,9 +15,6 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 #ifndef _GNASH_GNASHEXCEPTION__H
 #define _GNASH_GNASHEXCEPTION__H 1
 
@@ -116,6 +113,31 @@ public:
 
 };
 
+/// An ActionScript type error 
+//
+/// This exception can be thrown by as_value::to_primitive when an object
+/// can't be converted to a primitive value or by native function when
+/// they are called as method of an unexpected type
+///
+class ActionTypeError: public ActionException
+{
+
+public:
+
+	ActionTypeError(const std::string& s)
+		:
+		ActionException(s)
+	{}
+
+	ActionTypeError()
+		:
+		ActionException("ActionTypeError")
+	{}
+
+	virtual ~ActionTypeError() throw() {}
+
+};
+
 } // namespace gnash
 
 #endif // def _GNASH_GNASHEXCEPTION__H
