@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,15 +15,11 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef __GNASH_URL_H__
 #define __GNASH_URL_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
@@ -63,28 +59,32 @@ public:
 	URL(const std::string& relative_url, const URL& baseurl);
 
 	/// Return the 'protocol' member of this URL, as a string
-	std::string protocol() const { return _proto; }
+	const std::string& protocol() const { return _proto; }
 
 	/// Return the 'hostname' member of this URL, as a string
 	//
 	/// NOTE: return the empty string if protocol() is "file"
 	///
-	std::string hostname() const { return _host; }
+	const std::string& hostname() const { return _host; }
 
 	/// Return the 'path' member of this URL, as a string
-	std::string path() const { return _path; }
+	const std::string& path() const { return _path; }
 
 	/// Return the 'anchor' member of this URL, as a string
 	//
 	/// The anchor is the string after the '#' character
 	///
-	std::string anchor() const { return _anchor; }
+	const std::string& anchor() const { return _anchor; }
 
 	/// Return the 'querystring' member of this URL, as a string
 	//
 	/// The query is the string after the '?' character
 	///
-	std::string querystring() const { return _querystring; }
+	const std::string& querystring() const { return _querystring; }
+	
+	/// Set the 'querystring' member of this URL to a new value
+	//
+	void set_querystring(const std::string& value) { _querystring = value; } 
 
 	/// Return the full absolute URL as a string.
 	//
@@ -127,6 +127,17 @@ public:
 	///
 	static void encode(std::string& str);
 
+	/// \brief
+	/// Encode a string to URL-encoded format
+	/// converting all dodgy characters to %AB hex sequences. This
+	/// merely uses the void encode() function on a new string.
+	///
+	/// @ param str
+	///	The input string
+	/// @ return
+	///	An encoded version of the input string	
+	static std::string encode(const std::string& str);
+
 	/// \brief
 	/// Decode a string from URL-encoded format
 	/// converting all hexadecimal sequences to ASCII characters.
