@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,13 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: container.h,v 1.59 2007/07/01 10:54:07 bjacques Exp $ */
+/* $Id: container.h,v 1.61 2008/01/21 20:55:44 rsavoye Exp $ */
 
 #ifndef __CONTAINER_H__
 #define __CONTAINER_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
@@ -40,7 +40,7 @@
 // corrent paths supplied, this one file barfs with GCC 3.3.3 on
 // NetBSD, so screw it, and just hack it for now. We hope this entire
 // file will be gone soon anyway.
-#ifndef HAVE_WINSOCK_H
+#if !defined(HAVE_WINSOCK_H) || defined(__OS2__)
 #define _LIB_PTHREAD_ 1
 #ifndef _LIB_PTHREAD_TYPES_H
 #  define _LIB_PTHREAD_TYPES_H 1
