@@ -1,6 +1,6 @@
 // extension.cpp:  Read and enable plug-in extensions to Flash, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: extension.cpp,v 1.12 2007/07/01 10:54:08 bjacques Exp $ */
+/* $Id: extension.cpp,v 1.17 2008/02/19 19:20:50 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 // #if defined(_WIN32) || defined(WIN32)
@@ -78,7 +78,7 @@ Extension::Extension()
         _pluginsdir = env;
     }
 
-    log_msg("Plugins path: %s", _pluginsdir);
+    log_debug("Plugins path: %s", _pluginsdir);
     lt_dlsetsearchpath(_pluginsdir);
 }
 
@@ -113,7 +113,6 @@ bool
 Extension::scanAndLoad(as_object &obj)
 {
 //    GNASH_REPORT_FUNCTION;
-//    const char *mod;
     string mod;
     
     if (_modules.size() == 0) {
@@ -123,7 +122,7 @@ Extension::scanAndLoad(as_object &obj)
     vector<string>::iterator it;
     for (it = _modules.begin(); it != _modules.end(); it++) {
         mod = *(it);
-        log_msg(_("Loading module: %s"), mod.c_str());
+        log_security(_("Loading module: %s"), mod.c_str());
         SharedLib sl;
         initModule(mod.c_str(), obj);
     }   
@@ -139,7 +138,7 @@ Extension::initModule(const char *module, as_object &obj)
     SharedLib *sl;
     string symbol;
 
-    log_msg(_("Initializing module: \"%s\""), module);
+    log_security(_("Initializing module: \"%s\""), module);
     
     symbol = module;
     if (_plugins[module] == 0) {
@@ -162,6 +161,34 @@ Extension::initModule(const char *module, as_object &obj)
     return true;
 }
 
+bool
+Extension::initModuleWithFunc(const char *module, const char *func,
+	as_object &obj)
+{
+	SharedLib::initentry *symptr;
+	SharedLib *sl;
+
+	log_security(_("Initializing module: \"%s\""), module);
+
+	if (_plugins[module] == 0) {
+		sl = new SharedLib(module);
+		sl->openLib();
+		_plugins[module] = sl;
+	} else {
+		sl = _plugins[module];
+	}
+
+	symptr = sl->getInitEntry(func);
+
+	if (symptr) {
+		symptr(obj);
+	} else {
+		log_error(_("Couldn't get class_init symbol: \"%s\""), func);
+	}
+
+	return true;
+}
+
 bool
 Extension::scanDir()
 {
@@ -179,7 +206,8 @@ Extension::scanDir(const char *dirlist)
     struct dirent *entry;
     //string::size_type pos;
     char *dirlistcopy;
-    char *dir;
+    char *dir, libsdir;
+    char *suffix = 0;
 
 //    scoped_lock lock(lib_mutex);
 
@@ -189,9 +217,10 @@ Extension::scanDir(const char *dirlist)
     if (dir == NULL) {
         dir = dirlistcopy;
     }
-    
+
+            
     while (dir) {
-        log_msg(_("Scanning directory \"%s\" for plugins"), dir);
+        log_debug(_("Scanning directory \"%s\" for plugins"), dir);
         DIR *library_dir = opendir(dir);
 
         if (library_dir == NULL) {
@@ -199,28 +228,36 @@ Extension::scanDir(const char *dirlist)
             return false;
         }
         
-        // By convention, the first two entries in each directory are for . and
-        // .. (``dot'' and ``dot dot''), so we ignore those.
         entry = readdir(library_dir);
-        entry = readdir(library_dir);
-        
         for (i=0; entry>0; i++) {
             // We only want shared libraries than end with the suffix, otherwise
             // we get all the duplicates.
             entry = readdir(library_dir);
-            if (entry <= NULL) {
-                break;
-            }
-            
-            if (strstr(entry->d_name, ".la") == 0) {
+
+            if (entry <= NULL) { // All done
                 continue;
             }
+
+            if (strncmp(entry->d_name, ".", 1) == 0) {
+                continue;
+            }            
             
-            *(strrchr(entry->d_name, '.')) = 0;
-            log_msg(_("Gnash Plugin name: %s"), entry->d_name);
-            _modules.push_back(entry->d_name);
+            suffix = strrchr(entry->d_name, '.');
+            if (suffix == 0) {
+                continue;
+            }
+
+            log_debug(_("Gnash Plugin name: %s"), entry->d_name);
             
+            if (strcmp(suffix, ".so") == 0) {
+                *suffix = 0;
+                log_debug(_("Gnash Plugin name: %s"), entry->d_name);
+                _modules.push_back(entry->d_name);
+            } else {
+                continue;
+            }
         }
+        
         if (closedir(library_dir) != 0) {
             return false;
         }
