@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -19,7 +19,7 @@
 #define GNASH_LOG_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #ifdef HAVE_WINSOCK2_H
@@ -32,12 +32,12 @@
 #define N_(String) gettext_noop (String)
 
 #include "rc.h" // for IF_VERBOSE_* implementation
-//#include "tu_config.h"
 
 #include <fstream>
 #include <sstream>
 #include <boost/thread/thread.hpp>
 #include <boost/thread/mutex.hpp>
+//#include <boost/format.hpp>
 
 // the default name for the debug log
 #define DEFAULT_LOGFILE "gnash-dbg.log"
@@ -50,14 +50,7 @@ extern std::ostream& stampon(std::ostream& x);
 extern std::ostream& stampoff(std::ostream& x);
 extern std::ostream& timestamp(std::ostream& x);
 extern std::ostream& datetimestamp(std::ostream& x);
-#define TRACELEVEL 2
-
-class DSOLOCAL Verbose {
-    int level;
-public:
-    Verbose(int l) { level = l; }
-    friend std::ostream& operator<<(std::ostream&, Verbose&);
-};
+#define DEBUGLEVEL 2
 
 // This is a basic file logging class
 class DSOEXPORT LogFile {
@@ -65,18 +58,14 @@ public:
 
     static LogFile& getDefaultInstance();
 
-    ~LogFile(void) {
-	if (_state == OPEN) {
-	    closeLog();
-	}
-    }
+    ~LogFile();
 
-    enum file_state {
-	CLOSED,
-	OPEN,
-	INPROGRESS,
-	IDLE
-    } _state;
+    enum fileState {
+        CLOSED,
+        OPEN,
+        INPROGRESS,
+        IDLE
+    };
 
     /// Intended for use by log_*(). Thread-safe (locks _ioMutex)
     //
@@ -86,101 +75,126 @@ public:
     /// @param msg
     ///		The message string ie: "bah" for "ERROR: bah"
     ///
-    void log(const char* label, const char* msg);
+    void log(const std::string& label, const std::string& msg);
 
     /// Intended for use by log_*(). Thread-safe (locks _ioMutex)
     //
     /// @param msg
     ///		The message to print
     ///
-    void log(const char* msg);
+    void log(const std::string& msg);
     
-    file_state GetState (void) { return _state; }
+    /// Remove the log file
+    //
+    /// Does NOT lock _ioMutex (should it?)
+    ///
+    bool removeLog();
 
-    const char *getEntry(void);
-    
-    bool openLog(const char *filespec);
-    bool removeLog(void);
-
-    // locks _ioMutex to prevent race conditions accessing _outstream
-    bool closeLog(void);
+    /// Close the log file
+    //
+    /// Locks _ioMutex to prevent race conditions accessing _outstream
+    ///
+    bool closeLog();
 
     // accessors for the verbose level
-    void setVerbosity (void) {
-	_verbose++;
+    void setVerbosity () {
+        _verbose++;
     }
+
     void setVerbosity (int x) {
-	_verbose = x;
+        _verbose = x;
     }
-    int getVerbosity (void) {
-	return _verbose;
+
+    int getVerbosity () {
+        return _verbose;
     }
     
     void setActionDump (int x) {
-	_actiondump = x;
+        _actiondump = x;
     }
-    int getActionDump (void) {
-	return _actiondump;
+
+    int getActionDump () {
+        return _actiondump;
     }
     
     void setParserDump (int x) {
-	_parserdump = x;
+        _parserdump = x;
     }
-    int getParserDump (void) {
-	return _parserdump;
+
+    int getParserDump () {
+        return _parserdump;
     }
     
     void setStamp (bool b) {
 	_stamp = b;
     }
-    bool getStamp (void) {
-	return _stamp;
+
+    bool getStamp () {
+        return _stamp;
     }
+
     void setWriteDisk (bool b) {
 	_write = b;
     }
-    bool getWriteDisk (void) {
+
+    bool getWriteDisk () {
 	return _write;
     }
     
 private:
+    
+    /// Open the specified file to write logs on disk
+    //
+    /// Locks _ioMutex to prevent race conditions accessing _outstream
+    ///
+    /// @return true on success, false on failure
+    ///
+    bool openLog(const std::string& filespec);
+
+    /// \brief
+    /// Open the RcInitFile-specified log file if log write
+    /// is requested. 
+    //
+    /// This method is called before any attempt to write is made.
+    /// It will return true if the file was opened, false if wasn't
+    /// (either not requested or error).
+    ///
+    /// On error, will print a message on stderr
+    ///
+    bool openLogIfNeeded();
 
     // Use getDefaultInstance for getting the singleton
-    LogFile (void);
-
-    // Use getDefaultInstance for getting the singleton
-    LogFile (const char *);
+    LogFile ();
 
+    /// Mutex for locking I/O during logfile access.
     boost::mutex _ioMutex;
 
-    static std::ofstream _console;
+    /// Stream to write to stdout.
     std::ofstream	 _outstream;
+
+    /// How much output is required: 2 or more gives debug output.
     static int		 _verbose;
+
+    /// Whether to dump all SWF actions
     static bool		 _actiondump;
+
+    /// Whether to dump parser output
     static bool		 _parserdump;
+
+    /// The state of the log file.
+    fileState _state;
+
     bool		 _stamp;
+
+    /// Whether to write the log file to disk.
     bool		 _write;
-    bool		 _trace;
+
     std::string		 _filespec;
-    std::string		 _logentry;
+
+    /// For the ostream << operator
     friend std::ostream & operator << (std::ostream &os, LogFile& e);
 
-    LogFile& operator << (char x);
-    LogFile& operator << (int x);
-    LogFile& operator << (long x);
-    LogFile& operator << (unsigned int x);
-    LogFile& operator << (unsigned long x);
-    // These both resolve to an unsigned int.
-    // LogFile& operator << (size_t x);
-    // LogFile& operator << (time_t x);
-    LogFile& operator << (float x);
-    LogFile& operator << (double &x);
-    LogFile& operator << (bool x);
-    LogFile& operator << (void *);
-    LogFile& operator << (const char *);
-    LogFile& operator << (unsigned char const *);
     LogFile& operator << (const std::string &s);
-//     LogFile& operator << (const xmlChar *c);
     LogFile& operator << (std::ostream & (&)(std::ostream &));
 
     /// Print anything that can be printed on a stringstream
@@ -208,14 +222,6 @@ DSOEXPORT unsigned char *hexify(unsigned char *p, const unsigned char *s, int le
 #define GNUC_LOG_ATTRS
 #endif
 
-/// Log a generic message
-//
-/// This is usually used for debugging, so most
-/// such calls should be in comments, unless you're actively debugging
-/// that piece of code right now.
-///
-DSOEXPORT void log_msg(const char* fmt, ...) GNUC_LOG_ATTRS;
-
 /// Log a runtime error
 //
 /// Runtime errors, such as un-openable files, un-allocatable memory,
@@ -224,6 +230,7 @@ DSOEXPORT void log_msg(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// or log_swferror for that.
 ///
 DSOEXPORT void log_error(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logError(const boost::format& fmt);
 
 /// Log a message about unimplemented features.
 //
@@ -232,6 +239,7 @@ DSOEXPORT void log_error(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// implement those features of Flash.
 ///
 DSOEXPORT void log_unimpl(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logUnimpl(const boost::format& fmt);
 
 /// Use only for explicit user traces
 //
@@ -239,12 +247,14 @@ DSOEXPORT void log_unimpl(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// ASHandlers.cpp for ActionTrace
 ///
 DSOEXPORT void log_trace(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logTrace(const boost::format& fmt);
 
 /// Log debug info
 //
 /// Used for function entry/exit tracing.
 ///
 DSOEXPORT void log_debug(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logDebug(const boost::format& fmt);
 
 /// Log action execution info
 //
@@ -254,6 +264,7 @@ DSOEXPORT void log_debug(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// at runtime.
 ///
 DSOEXPORT void log_action(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logAction(const boost::format& fmt);
 
 /// Log parsing information
 //
@@ -263,9 +274,11 @@ DSOEXPORT void log_action(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// at runtime.
 ///
 DSOEXPORT void log_parse(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logParse(const boost::format& fmt);
 
 /// Log security information
 DSOEXPORT void log_security(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logSecurity(const boost::format& fmt);
 
 /// Log a malformed SWF error
 //
@@ -278,6 +291,7 @@ DSOEXPORT void log_security(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// at runtime.
 ///
 DSOEXPORT void log_swferror(const char* fmt, ...) GNUC_LOG_ATTRS;
+//DSOEXPORT void logSWFError(const boost::format& fmt);
 
 /// Log an ActionScript error
 //
@@ -290,7 +304,7 @@ DSOEXPORT void log_swferror(const char* fmt, ...) GNUC_LOG_ATTRS;
 /// at runtime.
 ///
 DSOEXPORT void log_aserror(const char* fmt, ...) GNUC_LOG_ATTRS;
-
+//DSOEXPORT void logASError(const boost::format& fmt);
 
 
 // Define to 0 to completely remove parse debugging at compile-time
@@ -361,12 +375,26 @@ public:
     }
 
     ~__Host_Function_Report__(void) {
-	if (LogFile::getDefaultInstance().getVerbosity() >= TRACELEVEL+1) {
+	if (LogFile::getDefaultInstance().getVerbosity() >= DEBUGLEVEL+1) {
 	    log_debug("returning");
 	}
     }
 };
 
+#ifndef HAVE_FUNCTION
+	#ifndef HAVE_func
+		#define dummystr(x) # x
+		#define dummyestr(x) dummystr(x)
+		#define __FUNCTION__ __FILE__":"dummyestr(__LINE__)
+	#else
+		#define __FUNCTION__ __func__	
+	#endif
+#endif
+
+#ifndef HAVE_PRETTY_FUNCTION
+	#define __PRETTY_FUNCTION__ __FUNCTION__
+#endif
+
 #if defined(__cplusplus) && defined(__GNUC__)
 #define GNASH_REPORT_FUNCTION   \
 	gnash::__Host_Function_Report__ __host_function_report__( __PRETTY_FUNCTION__)
