@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -19,14 +19,14 @@
 #define __NETWORK_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_config.h"
 
 #include <string>
 
-#ifndef HAVE_WINSOCK_H
+#if !defined(HAVE_WINSOCK_H) || defined(__OS2__)
 # include <netinet/in.h>
 # include <arpa/inet.h>
 #else
@@ -46,7 +46,12 @@ const short RTMP = 1935;
 const short RTMPT = 80;
 const short RTMPTS = 443;
 
-#ifdef HAVE_WINSOCK_H
+#ifdef __OS2__
+	typedef int    socklen_t;
+	#define SHUT_RDWR 0x2
+#endif
+
+#if defined(HAVE_WINSOCK_H) && !defined(__OS2__)
 	typedef long   in_addr_t;
 #	define inet_lnaof(x) inet_addr(inet_ntoa(x))
 	typedef int    socklen_t;
@@ -65,7 +70,10 @@ public:
     // Accept a client connection for the current server.
     bool newConnection(void);
     bool newConnection(bool block);
-    
+
+    // Connect to a named pipe
+    bool connectSocket(const char *sock);
+
     // Create a client connection to a tcp/ip server
     bool createClient(void);
     bool createClient(short port);
