@@ -1,6 +1,6 @@
 // sharedlib.cpp:  Shared Library support, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: sharedlib.cpp,v 1.17 2007/07/01 10:54:10 bjacques Exp $ */
+/* $Id: sharedlib.cpp,v 1.20 2008/02/19 19:20:50 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "log.h"
@@ -106,7 +106,7 @@ SharedLib::SharedLib(const char *filespec)
     if (errors) {
         log_error (_("Couldn't initialize ltdl: %s"), lt_dlerror());
 //     } else {
-//         log_msg ("Initialized ltdl");
+//         log_debug ("Initialized ltdl");
     }
     char *pluginsdir;
     char *env = getenv ("GNASH_PLUGINS");
@@ -166,7 +166,7 @@ SharedLib::openLib (const char *filespec)
 //     cerr << "Searching in " << lt_dlgetsearchpath()
 //          << "for database drivers" << endl;
 
-//    log_msg ("Trying to open shared library \"%s\"", filespec);
+//    log_debug ("Trying to open shared library \"%s\"", filespec);
     _dlhandle = lt_dlopenext (filespec);
     
     if (_dlhandle == NULL) {
@@ -177,7 +177,7 @@ SharedLib::openLib (const char *filespec)
     // Make this module unloadable
     lt_dlmakeresident(_dlhandle);
     
-    log_msg (_("Opened dynamic library \"%s\""), filespec);
+    log_debug (_("Opened dynamic library \"%s\""), filespec);
 
     _filespec = filespec;
     
@@ -197,6 +197,7 @@ SharedLib::moduleName()
 SharedLib::entrypoint *
 SharedLib::getDllSymbol (std::string &symbol)
 {
+    GNASH_REPORT_FUNCTION;
     return getDllSymbol(symbol.c_str());
 }
 
@@ -214,7 +215,7 @@ SharedLib::getInitEntry (const char *symbol)
         log_error (_("Couldn't find symbol: %s"), symbol);
         return NULL;
     } else {
-        log_msg (_("Found symbol %s @ %p"), symbol, (void *)run);
+        log_debug (_("Found symbol %s @ %p"), symbol, (void *)run);
     }
     
     return (initentry *)run;
@@ -223,7 +224,7 @@ SharedLib::getInitEntry (const char *symbol)
 SharedLib::entrypoint *
 SharedLib::getDllSymbol(const char *symbol)
 {
-//    GNASH_REPORT_FUNCTION;
+    GNASH_REPORT_FUNCTION;
     
     lt_ptr run = NULL;
     
@@ -239,7 +240,7 @@ SharedLib::getDllSymbol(const char *symbol)
         log_error (_("Couldn't find symbol: %s"), symbol);
         return NULL;
     } else {
-        log_msg (_("Found symbol %s @ %p"), symbol, (void *)run);
+        log_debug (_("Found symbol %s @ %p"), symbol, (void *)run);
     }
     
     return (entrypoint *)run;
