@@ -1,5 +1,5 @@
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,16 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-//
-//
-
 // Smart (ref-counting) pointer classes.  Uses "intrusive" approach:
 // the types pointed to must have add_ref() and drop_ref() methods.
 // Typically this is done by inheriting from a ref_counted class,
 // although the nice thing about templates is that no particular
 // ref-counted class is mandated.
 
-/* $Id: smart_ptr.h,v 1.22 2007/07/01 10:54:10 bjacques Exp $ */
+/* $Id: smart_ptr.h,v 1.25 2008/01/21 20:55:45 rsavoye Exp $ */
 
 #ifndef SMART_PTR_H
 #define SMART_PTR_H
@@ -76,6 +73,12 @@ intrusive_ptr_release(ref_counted* o)
 inline void intrusive_ptr_add_ref(GcResource* ) { }
 inline void intrusive_ptr_release(GcResource* ) { }
 
+#ifdef GNASH_USE_GC
+class as_object;
+inline void intrusive_ptr_add_ref(as_object* ) { }
+inline void intrusive_ptr_release(as_object* ) { }
+#endif
+
 // The below thing won't work. We'll need a real templated class..
 //template <typename C> typedef C* gc_ptr;
 
