@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
-/* $Id: DynamicShape.h,v 1.5 2007/07/01 10:54:18 bjacques Exp $ */
+/* $Id: DynamicShape.h,v 1.10 2008/01/21 20:55:46 rsavoye Exp $ */
 
 #ifndef GNASH_DYNAMIC_SHAPE_H
 #define GNASH_DYNAMIC_SHAPE_H
@@ -55,11 +55,17 @@ public:
 	/// Start drawing with a solid fill
 	void beginFill(const rgba& color);
 
+	/// Start drawing with a linear gradient fill
+	void beginLinearGradientFill(const std::vector<gradient_record>& grad, const matrix& mat);
+
+	/// Start drawing with a radial gradient fill
+	void beginRadialGradientFill(const std::vector<gradient_record>& grad, const matrix& mat);
+
 	/// Close an existing filled path, if any.
 	void endFill();
 
 	/// Set current line style and start a new path.
-	void lineStyle(uint16_t thickness, const rgba& color);
+	void lineStyle(boost::uint16_t thickness, const rgba& color);
 
 	/// Reset line style to no style and start a new path.
 	void resetLineStyle();
@@ -86,6 +92,29 @@ public:
 	///
 	size_t add_line_style(const line_style& stl);
 
+	// Override from shape_character_def to call ::finalize
+	// NOTE: this is not correct in that a call to hitTest should
+	//       not force closing the path being drawn.
+	//       Instead, the closeup should be "temporary" and int
+	//       the point_test_local itself (but only for dynamic drawing).
+	//       We need to add a testcase for this as we currently have none.
+	//       The testcase would look like this:
+	//
+	//       moveTo(0, 0); lineTo(10, 0); lineTo(10, 10); // an L shape so far
+	//       hitTest(8, 2, true); !hitTest(2, 8, true); // imaginarly forming a closed triangle as hitTest is concerned
+	//       lineTo(0, 10); lineTo(0, 0); // explicitly closed as a square now
+	//       hitTest(8, 2, true); hitTest(2, 8, true); // effectively forming a closed square
+	//
+	//       In the test above, permanently closing on hit-test (what this implementation does)
+	//       would result in a triangle and a stroke, which should fail the last hitTest(2,8).
+	//
+	//
+	bool point_test_local(float x, float y)
+	{
+		finalize();
+		return shape_character_def::point_test_local(x, y);
+	}
+
 	/// Add a path, updating _currpath and recomputing bounds
 	//
 	/// TODO: make private? Only current user is BitmapMovieDefinition.
@@ -102,13 +131,16 @@ public:
 
 private:
 
-	// Initialize a new path, used when changing
-	// style or moving the pen
+	/// Initialize a new path
 	//
-	// The newly added path will use current values
-	// for origin, fill and line styles.
-	//
-	void startNewPath();
+	/// Used when changing style or moving the pen
+	///
+	/// The newly added path will use current values
+	/// for origin, fill and line styles.
+	///
+	/// If newShape is true the new shape will start a new subshape.
+	///
+	void startNewPath(bool newShape);
 
 	path* _currpath;
 
