@@ -1,6 +1,6 @@
 // Global.cpp:  Global ActionScript class setup, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: Global.cpp,v 1.66 2007/07/24 14:19:31 strk Exp $ */
+/* $Id: Global.cpp,v 1.90 2008/02/19 19:20:54 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "as_object.h"
@@ -28,6 +28,7 @@
 #include "as_value.h"
 #include "as_function.h" // for function_class_init
 #include "array.h"
+#include "AsBroadcaster.h"
 #include "Boolean.h"
 #include "Camera.h"
 #include "Color.h"
@@ -64,6 +65,8 @@
 #include "builtin_function.h"
 #include "edit_text_character.h"
 #include "rc.h"
+#include "ClassHierarchy.h"
+#include "namedStrings.h"
 
 #include "fn_call.h"
 #include "sprite_instance.h"
@@ -72,7 +75,6 @@
 #include "xmlsocket.h"
 
 #include <limits> // for numeric_limits<double>::quiet_NaN
-#include <boost/scoped_array.hpp>
 
 // Common code to warn and return if a required single arg is not present
 // and to warn if there are extra args.
@@ -114,7 +116,7 @@ as_global_isnan(const fn_call& fn)
 {
     ASSERT_FN_ARGS_IS_1
 
-    return as_value( static_cast<bool>(isnan(fn.arg(0).to_number(&fn.env())) ));
+    return as_value( static_cast<bool>(isnan(fn.arg(0).to_number()) ));
 }
 
 static as_value
@@ -122,7 +124,7 @@ as_global_isfinite(const fn_call& fn)
 {
     ASSERT_FN_ARGS_IS_1
 
-    return as_value( isfinite(fn.arg(0).to_number(&fn.env())) );
+    return as_value( (bool)isfinite(fn.arg(0).to_number()) );
 }
 
 /// \brief Encode a string to URL-encoded format
@@ -186,7 +188,6 @@ as_global_parsefloat(const fn_call& fn)
 static as_value
 as_global_parseint(const fn_call& fn)
 {
-    as_environment* env = &fn.env();
     // assert(fn.nargs == 2 || fn.nargs == 1);
     if (fn.nargs < 1) {
 	IF_VERBOSE_ASCODING_ERRORS(
@@ -199,92 +200,106 @@ as_global_parseint(const fn_call& fn)
             log_aserror(_("%s has more than two arguments"), __FUNCTION__);
     )
 
-    // Make sure our argument is the correct type
-    if (fn.nargs > 1)
-	fn.arg(1).convert_to_number(env);
-
-    // Set up some variables
-    const string digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
-    boost::scoped_array<char> input_buffer ( new char[fn.arg(0).to_string().size()+1] );
-    char *input = input_buffer.get();
-    strcpy(input,fn.arg(0).to_string().c_str());
-    int base;
-    bool bNegative;
+    const std::string& expr = fn.arg(0).to_string();
+    
+    int base = 10;
+    const std::string digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+    bool bNegative = false;
+  
+    std::string::const_iterator it = expr.begin();
 
     // Skip leading whitespace
-    while ((input[0] == ' ') || (input[0] == 0x9))
-	input++;
+    while(*it == ' ' || *it == '\n' || *it == '\t' || *it == '\r')
+    {
+        ++it;
+    }
 
-    if (input[0] == '-')
-	{
-	    bNegative = true;
-	    input++;
-	}
-    else
-	bNegative = false;
-
-    // Convert the string to uppercase
-    for (int i=strlen(input)-1; i >= 0; i--)
-	input[i] = toupper(input[i]);
+    // Is the first non-whitespace character a minus?
+    if (*it == '-')
+    {
+        bNegative = true;
+	++it;
+    }
 
     // if we were sent a second argument, that's our base
     if (fn.nargs > 1)
-	{
-	    // to_number returns a double. atoi() would be better
-	    base = (int)(fn.arg(1).to_number(env));
-	}
-    // if the string starts with "0x" then a hex digit
-    else if (strlen(input) > 2 && input[0] == '0' && input[1] == 'X'
-	     && (isdigit(input[2]) || (input[2] >= 'A' && input[2] <= 'F')))
-	{
-	    base = 16;	// the base is 16
-	    input = input + 2; // skip the leading "0x"
-	}
-    // if the string starts with "0" then an octal digit
-    else if (strlen(input) > 1 && input[0] == '0' &&
-	     (input[1] >= '0' && input[1] <= '7'))
-	{
-	    base = 8;
-	    input++; // skip the leading '0'
-	}
-    else
-	// default base is 10
-	base = 10;
-
-    if (base < 2 || base > 36)
-	{
+    {
+	// to_number returns a double. atoi() would be better
+	base = (int)(fn.arg(1).to_number());
+	
+	// Bases from 2 to 36 are valid, otherwise return NaN
+        if (base < 2 || base > 36)
+        {
 	    as_value rv;
 	    rv.set_nan();
 	    return rv;
-	}
+        }	
 
-    int numdigits = 0;
+    }
 
-    // Start at the beginning, see how many valid digits we have
-    // in the base we're dealing with
-    while (numdigits < int(strlen(input)) 
-	   && int(digits.find(input[numdigits])) < base
-	   && digits.find(input[numdigits]) != std::string::npos)
-	numdigits++;
+    // If the string starts with '0x':
+    else if (expr.end() - it >= 2 &&
+    		(*it == '0' && toupper(*(it + 1)) == 'X' ))
+    {
+        // the base is 16
+	base = 16;
+        // Move to the digit after the 'x'
+	it += 2; 
+    }
 
-    // If we didn't get any digits, we should return NaN
-    if (numdigits == 0)
-	{
+    // Octal if the string starts with "0" then an octal digit, but
+    // *only* if there is no whitespace before it; in that case decimal.
+    else if (it - expr.begin() == (bNegative ? 1 : 0))
+    {
+        if (expr.end() - it >= 2 && 
+    		*it == '0' && isdigit(*(it + 1)))
+        {
+            // And if there are any chars other than 0-7, it's *still* a
+            // base 10 number...
+            // At least we know where we are in the string, so can use
+            // string methods.
+	    if (expr.find_first_not_of("01234567", (bNegative ? 1 : 0)) !=
+	    	std::string::npos)
+	    {
+	        base = 10;
+	    }
+	    else base = 8;
+
+	    // Point the iterator to the first digit after the '0'.
+	    ++it;
+
+        }
+    }
+
+    // Check to see if the first digit is valid, otherwise 
+    // return NaN.
+    int digit = digits.find(toupper(*it));
+
+    if (digit >= base)
+    {
 	    as_value rv;
 	    rv.set_nan();
 	    return rv;
-	}
+    }
 
-    int result = 0;
-    for (int i=0;i<numdigits;i++)
-	{
-	    result = result * base + digits.find(input[i]);
-	}
+    // The first digit was valid, so continue from the present position
+    // until we reach the end of the string or an invalid character,
+    // adding valid characters to our result.
+    // Which characters are invalid depends on the base. 
+    int result = digit;
+    ++it;
+    
+    while (it != expr.end() && (digit = digits.find(toupper(*it))) < base
+    		&& digit >= 0)
+    {
+	    result = result * base + digit;
+	    ++it;
+    }
 
     if (bNegative)
 	result = -result;
     
-    // Now return the parsed string
+    // Now return the parsed string as an integer.
     return as_value(result);
 }
 
@@ -292,11 +307,9 @@ as_global_parseint(const fn_call& fn)
 static as_value
 as_global_assetpropflags(const fn_call& fn)
 {
-    as_environment* env = &fn.env();
-    int version = env->get_version();
-    
+    int version = VM::get().getSWFVersion();
 
-    //log_msg(_("ASSetPropFlags called with %d args"), fn.nargs);
+    //log_debug(_("ASSetPropFlags called with %d args"), fn.nargs);
 
     // Check the arguments
     // assert(fn.nargs == 3 || fn.nargs == 4);
@@ -335,7 +348,7 @@ as_global_assetpropflags(const fn_call& fn)
     // are used to determine whether the list of child names should be hidden,
     // un-hidden, protected from over-write, un-protected from over-write,
     // protected from deletion and un-protected from deletion
-    int set_true = int(fn.arg(2).to_number(env)) & as_prop_flags::as_prop_flags_mask;
+    int set_true = int(fn.arg(2).to_number()) & as_prop_flags::as_prop_flags_mask;
 
     // Is another integer bitmask that works like set_true,
     // except it sets the attributes to false. The
@@ -344,7 +357,7 @@ as_global_assetpropflags(const fn_call& fn)
     // ASSetPropFlags was exposed in Flash 5, however the fourth argument 'set_false'
     // was not required as it always defaulted to the value '~0'. 
     int set_false = (fn.nargs == 3 ? 
-		     (version == 5 ? ~0 : 0) : int(fn.arg(3).to_number(env)))
+		     (version == 5 ? ~0 : 0) : int(fn.arg(3).to_number()))
 	& as_prop_flags::as_prop_flags_mask;
 
 	obj->setPropFlags(props, set_false, set_true);
@@ -352,7 +365,91 @@ as_global_assetpropflags(const fn_call& fn)
     return as_value();
 }
 
-Global::Global(VM& vm)
+// ASNative function
+// See: http://osflash.org/flashcoders/undocumented/asnative?s=asnative
+static as_value
+as_global_asnative(const fn_call& fn)
+{
+	as_value ret;
+
+	if (fn.nargs < 2)
+	{
+		IF_VERBOSE_ASCODING_ERRORS(	
+		log_aserror(_("ASNative(%s): needs at least two arguments"), fn.dump_args().c_str());
+		)
+		return ret;
+	}
+
+	int sx = fn.arg(0).to_int();
+	int sy = fn.arg(1).to_int();
+
+	if ( sx < 0 )
+	{
+		IF_VERBOSE_ASCODING_ERRORS(	
+		log_aserror(_("ASNative(%s): first arg must be >= 0"), fn.dump_args().c_str());
+		)
+		return ret;
+	}
+	if ( sy < 0 )
+	{
+		IF_VERBOSE_ASCODING_ERRORS(	
+		log_aserror(_("ASNative(%s): second arg must be >= 0"), fn.dump_args().c_str());
+		)
+		return ret;
+	}
+
+	unsigned x = (unsigned)sx;
+	unsigned y = (unsigned)sy;
+
+	VM& vm = VM::get();
+	as_function* fun = vm.getNative(x, y);
+	if ( ! fun ) return ret;
+	ret.set_as_function(fun);
+	return ret;
+		
+}
+
+// ASSetNative function
+// TODO: find dox 
+static as_value
+as_global_assetnative(const fn_call& /*fn*/)
+{
+	log_unimpl("ASSetNative");
+	return as_value();
+}
+
+// ASSetNativeAccessor function
+// TODO: find dox 
+static as_value
+as_global_assetnativeaccessor(const fn_call& /*fn*/)
+{
+	log_unimpl("ASSetNativeAccessor");
+	return as_value();
+}
+
+// ASconstructor function
+// TODO: find dox 
+static as_value
+as_global_asconstructor(const fn_call& /*fn*/)
+{
+	log_unimpl("ASconstructor");
+	return as_value();
+}
+
+// updateAfterEvent function
+static as_value
+as_global_updateAfterEvent(const fn_call& /*fn*/)
+{
+	static bool warned=false;
+	if ( ! warned )
+	{
+		log_unimpl("updateAfterEvent()");
+		warned=true;
+	}
+	return as_value();
+}
+
+Global::Global(VM& vm, ClassHierarchy *ch)
 	:
 	as_object()
 {
@@ -372,59 +469,63 @@ Global::Global(VM& vm)
 
 	// ASSetPropFlags
 	init_member("ASSetPropFlags", new builtin_function(as_global_assetpropflags));
+	init_member("ASnative", new builtin_function(as_global_asnative));
+	init_member("ASSetNative", new builtin_function(as_global_assetnative));
+	init_member("ASSetNativeAccessor", new builtin_function(as_global_assetnativeaccessor));
+	init_member("ASconstructor", new builtin_function(as_global_asconstructor));
+	init_member("updateAfterEvent", new builtin_function(as_global_updateAfterEvent));
 
 	// Defined in timers.h
 	init_member("setInterval", new builtin_function(timer_setinterval));
 	init_member("clearInterval", new builtin_function(timer_clearinterval));
+	init_member("setTimeout", new builtin_function(timer_settimeout));
+	init_member("clearTimeout", new builtin_function(timer_clearinterval));
 
-	// System and Function were added in Player Version 6, but
-	// seem to be available even if SWF target version is
-	// inferior
-	system_class_init(*this); // System and System.capabilities
-	function_class_init(*this);
+	ch->setGlobal(this);
 
-	// I find Stage object used in SWF5 movies, don't
-	// see a good reason not to enable this always
-	stage_class_init(*this);
+// If extensions aren't used, then no extensions will be loaded.
+#ifdef USE_EXTENSIONS
+	ch->setExtension(&et);
+#endif
 
+	ch->massDeclare(vm.getSWFVersion());
+
+	if (vm.getSWFVersion() >= 5)
+	{
+		object_class_init(*this);
+		ch->getGlobalNs()->stubPrototype(NSV::CLASS_OBJECT);
+		ch->getGlobalNs()->getClass(NSV::CLASS_OBJECT)->setDeclared();
+		array_class_init(*this);
+		ch->getGlobalNs()->stubPrototype(NSV::CLASS_ARRAY);
+		ch->getGlobalNs()->getClass(NSV::CLASS_ARRAY)->setDeclared();
+		string_class_init(*this);
+		ch->getGlobalNs()->stubPrototype(NSV::CLASS_STRING);
+		ch->getGlobalNs()->getClass(NSV::CLASS_STRING)->setDeclared();
+	}
+	if (vm.getSWFVersion() >= 6)
+	{
+		function_class_init(*this);
+		ch->getGlobalNs()->stubPrototype(NSV::CLASS_FUNCTION);
+		ch->getGlobalNs()->getClass(NSV::CLASS_FUNCTION)->setDeclared();
+	}
+	
 	if ( vm.getSWFVersion() < 3 ) goto extscan;
 	//-----------------------
 	// SWF3
 	//-----------------------
 
-	movieclip_class_init(*this);
-	textfield_class_init(*this);
-
 	if ( vm.getSWFVersion() < 4 ) goto extscan;
 	//-----------------------
 	// SWF4
 	//-----------------------
 
 	init_member("trace", new builtin_function(as_global_trace));
-	// The methods of the Math class are available
-	// as global methods in SWF4 and as members of
-	// the Math class from SWF5 up
-	math_class_init(*this);
 
 	if ( vm.getSWFVersion() < 5 ) goto extscan;
 	//-----------------------
 	// SWF5
 	//-----------------------
 
-	boolean_class_init(*this);
-	color_class_init(*this);
-	selection_class_init(*this); // Selection
-	sound_class_init(*this);
-    xmlsocket_class_init(*this);
-	date_class_init(*this);
-	xml_class_init(*this);
-	xmlnode_class_init(*this);
-	mouse_class_init(*this);
-	object_class_init(*this);
-	number_class_init(*this); 
-	string_class_init(*this); 
-	array_class_init(*this);
-	key_class_init(*this); // Key
 	init_member("escape", new builtin_function(as_global_escape));
 	init_member("unescape", new builtin_function(as_global_unescape));
 	init_member("parseFloat", new builtin_function(as_global_parsefloat));
@@ -438,36 +539,20 @@ Global::Global(VM& vm)
 	init_member("NaN", as_value(NAN));
 	init_member("Infinity", as_value(INFINITY));
 
+	color_class_init(*this);
+
 	if ( vm.getSWFVersion() < 6 ) goto extscan;
 	//-----------------------
 	// SWF6
 	//-----------------------
-
-	// See: http://sephiroth.it/reference.php?id=717&cat=1
-	textsnapshot_class_init(*this);
 	init_member("LocalConnection", new builtin_function(localconnection_new));
 	init_member("TextFormat", new builtin_function(textformat_new));
-	video_class_init(*this); // Video
-	camera_class_init(*this); // Camera
-	microphone_class_init(*this); // Microphone
-	sharedobject_class_init(*this);
-	loadvars_class_init(*this);
-	customactions_class_init(*this);
 
 	if ( vm.getSWFVersion() < 7 ) goto extscan;
 	//-----------------------
 	// SWF7
 	//-----------------------
 
-	netconnection_class_init(*this);
-	netstream_class_init(*this);
-	contextmenu_class_init(*this);
-	moviecliploader_class_init(*this);
-	// sephiroth.it refers this to be introduced in SWF7
-	// but empirical checks seem to confirm it's also avaiable in SWF6
-	// if player is >= 7
-	error_class_init(*this);
-
 	if ( vm.getSWFVersion() < 8 ) goto extscan;
 	//-----------------------
 	// SWF8
