@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 namespace gnash {
 	class VM;
 	class fn_call;
+	class ClassHierarchy;
 }
 
 namespace gnash {
@@ -38,7 +39,7 @@ namespace gnash {
 class Global: public as_object
 {
 public:
-	Global(VM& vm);
+	Global(VM& vm, ClassHierarchy *ch);
 	~Global() {}
 private:
 
