@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,59 +15,45 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
 #ifndef __LOCALCONNECTION_H__
 #define __LOCALCONNECTION_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <string>
 #include <map>
+#include <boost/cstdint.hpp> // for boost::?int??_t
 
 #include "as_object.h" // for inheritance
 #include "fn_call.h"
-
-#ifdef NETWORK_CONN
-#include "network.h"
-#else
-#include "shm.h"
-#endif
+#include "lcshm.h"
 
 namespace gnash {
   
-#ifdef NETWORK_CONN
-class LocalConnection {
-#else
-class LocalConnection : public Shm {
-#endif
-
+class LocalConnection : public as_object, LcShm {
 public:
     LocalConnection();
     ~LocalConnection();
     void close(void);
-    bool connect(const char *name);
-    std::string domain(void);
+    bool connect();
+    bool connect(const std::string& name);
+    std::string domain(int version);
     void send();
+    std::string &getName() { return _name; };
+    bool connected() { return _connected; };
+    
 // FIXME: these should be callbacks
 //     bool        _allowDomain;
 //     bool        _allowInsecureDomain;
 //     bool        _onStatus;
 private:
+    bool _connected;
     std::string _name;
     std::map<const char *, short> _allocated;
 };
 
-class localconnection_as_object : public as_object
-{
-public:
-    LocalConnection obj;
-};
-
 as_value localconnection_new(const fn_call& fn);
 as_value localconnection_close(const fn_call& fn);
 as_value localconnection_connect(const fn_call& fn);
