@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 
 // This program is distributed in the hope that it will be useful,
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: Math.cpp,v 1.22 2007/05/28 15:41:07 ann Exp $ */
+/* $Id: Math.cpp,v 1.27 2008/01/21 20:55:56 rsavoye Exp $ */
 
 //
 // This file implements methods of the ActionScript Math class.
@@ -30,15 +30,18 @@
 
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 #include <cmath>
 #include <string>
-#include "tu_random.h"
+#include <boost/random.hpp>
+
+#include "VM.h" // get random generator
 #include "fn_call.h"
 #include "GMath.h"
 #include "log.h"
 #include "builtin_function.h" 
+#include "Object.h" // for getObjectInterface
 
 using namespace std;
 
@@ -142,13 +145,22 @@ MATH_WRAP_FUNC2_EXP(min, (arg0 < arg1 ? arg0 : arg1))
 MATH_WRAP_FUNC2_EXP(pow, (pow(arg0, arg1)))
 
 // A couple of oddballs.
-as_value	math_random(const fn_call& /* fn */)
+as_value
+math_random(const fn_call& /* fn */)
 {
-    // Random number between 0 and 1.
-    return as_value(tu_random::next_random() / double(uint32_t(0x0FFFFFFFF)));
+
+	VM::RNG& rnd = VM::get().randomNumberGenerator();
+
+	// Produces double ( 0 <= n < 1)
+	boost::uniform_real<> uni_dist(0, 1);
+	boost::variate_generator<VM::RNG&, boost::uniform_real<> > uni(rnd, uni_dist);
+
+	return as_value(uni());
+
 }
 
-as_value	math_round(const fn_call& fn)
+as_value
+math_round(const fn_call& fn)
 {
 	// round argument to nearest int. 0.5 goes to 1 and -0.5 goes to 0
 	double result;
@@ -164,7 +176,7 @@ as_value	math_round(const fn_call& fn)
 
 math_as_object::math_as_object()
 	:
-	as_object()
+	as_object(getObjectInterface())
 {
 
 	// TODO: rely on ineritance, use init_property ?
