@@ -1,10 +1,10 @@
 // NetConnection.cpp:  Open local connections for FLV files or URLs.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: NetConnection.cpp,v 1.48 2007/06/01 23:36:48 bjacques Exp $ */
+/* $Id: NetConnection.cpp,v 1.57.2.1 2008/02/22 14:16:39 strk Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <iostream>
@@ -31,13 +31,12 @@
 #include "GnashException.h"
 #include "builtin_function.h"
 #include "movie_root.h"
+#include "Object.h" // for getObjectInterface
 
 #include "StreamProvider.h"
 #include "URLAccessManager.h"
 #include "URL.h"
 
-using namespace std;
-
 namespace gnash {
 
 static as_value netconnection_new(const fn_call& fn);
@@ -57,61 +56,101 @@ NetConnection::NetConnection()
 	attachProperties();
 }
 
+
 NetConnection::~NetConnection()
 {
 }
 
+
 /*public*/
 bool NetConnection::openConnection(const std::string& url)
 {
+  // if already running there is no need to setup things again
+  if ( _loader.get() ) {
+    log_debug("NetConnection::openConnection() called when already connected to a stream. Checking if the existing connection can be used.");
+    std::string newurl;
+    if (_prefixUrl.size() > 0) {
+      newurl += _prefixUrl + "/" + url;
+    } else {
+      newurl += url;
+    }
+    if (newurl.compare(_completeUrl) == 0) {
+      return true;
+    } else { 
+      return false;
+    }
+  }
 
-	// if already running there is no need to setup things again
-	if (_loader.get())
-	{
-		log_debug("NetConnection::openConnection() called when already connected to a stream. Checking if the existing connection can be used.");
-		std::string newurl;
-		if (_prefixUrl.size() > 0) {
-			newurl += _prefixUrl + "/" + url;
-		} else {
-			newurl += url;
-		}
-		if (newurl.compare(_completeUrl) == 0) return true;
-		else return false;
-	}
+  if ( _prefixUrl.size() > 0 ) {
+    _completeUrl += _prefixUrl + "/" + url;
+  } else {
+    _completeUrl += url;
+  }
 
+  URL uri( _completeUrl, get_base_url() );
+
+  std::string uriStr( uri.str() );
+  assert( uriStr.find( "://" ) != std::string::npos );
+
+  // Check if we're allowed to open url
+  if ( ! URLAccessManager::allow( uri ) ) {
+    log_security( _("Gnash is not allowed to open this url: %s"), uriStr.c_str() );
+    return false;
+  }
+
+  log_security( _("Connecting to movie: %s"), uriStr.c_str() );
+
+  _loader.reset( new LoadThread() );
+
+  if ( ! _loader->setStream( std::auto_ptr<tu_file>(StreamProvider::getDefaultInstance().getStream( uri ) ) ) ) {
+    log_error( _("Gnash could not open this url: %s"), uriStr.c_str() );
+    _loader.reset();
+
+    return false;
+  }
+
+  log_debug( _("Connection established to movie: %s"), uriStr.c_str() );
+
+  return true;
+}
+
+
+/*public*/
+bool
+NetConnection::eof()
+{
+	if (!_loader.get()) return true; // @@ correct ?
+	return _loader->eof();
+}
+
+
+/*public*/
+std::string NetConnection::validateURL(const std::string& url)
+{
+	std::string completeUrl;
 	if (_prefixUrl.size() > 0) {
-		_completeUrl += _prefixUrl + "/" + url;
+		completeUrl += _prefixUrl + "/" + url;
 	} else {
-		_completeUrl += url;
+		completeUrl += url;
 	}
 
-	URL uri(_completeUrl, get_base_url());
+	URL uri(completeUrl, get_base_url());
 
 	std::string uriStr(uri.str());
-	assert(uriStr.find("://")!=string::npos);
+	assert(uriStr.find("://") != std::string::npos);
 
 	// Check if we're allowed to open url
 	if (!URLAccessManager::allow(uri)) {
 		log_security(_("Gnash is not allowed to open this url: %s"), uriStr.c_str());
-		return false;
+		return "";
 	}
 
-	log_msg(_("Connecting to movie: %s"), uriStr.c_str());
+	log_debug(_("Connection to movie: %s"), uriStr.c_str());
 
-	_loader.reset( new LoadThread() );
-	
-	if (!_loader->setStream(std::auto_ptr<tu_file>(StreamProvider::getDefaultInstance().getStream(uri)))) {
-		log_error(_("Gnash could not open this url: %s"), uriStr.c_str());
-		_loader.reset();
-		return false;
-	}
-
-	log_msg(_("Connection etablished to movie: %s"), uriStr.c_str());
-
-	return true;
+	return uriStr;
 }
 
-/*public*/
+/*private*/
 void
 NetConnection::addToURL(const std::string& url)
 {
@@ -125,30 +164,31 @@ NetConnection::addToURL(const std::string& url)
 	_prefixUrl += url;
 }
 
-/*public*/
-bool
-NetConnection::eof()
-{
-	if (!_loader.get()) return true; // @@ correct ?
-	return _loader->eof();
-}
 
 /*public*/
 size_t
-NetConnection::read(void *dst, size_t bytes)
+NetConnection::read( void *dst, size_t bytes )
 {
-	if (!_loader.get()) return 0; // @@ correct ?
-	return _loader->read(dst, bytes);
+  if ( ! _loader.get() ) {
+    return 0;
+  }
+
+  return _loader->read( dst, bytes );
 }
 
+
 /*public*/
 bool
-NetConnection::seek(size_t pos)
+NetConnection::seek( size_t pos )
 {
-	if (!_loader.get()) return false; // @@ correct ?
-	return _loader->seek(pos);
+  if ( ! _loader.get() ) {
+    return false;
+  }
+
+  return _loader->seek( pos );
 }
 
+
 /*public*/
 size_t
 NetConnection::tell()
@@ -157,6 +197,7 @@ NetConnection::tell()
 	return _loader->tell();
 }
 
+
 /*public*/
 long
 NetConnection::getBytesLoaded()
@@ -174,27 +215,32 @@ NetConnection::getBytesTotal()
 	return _loader->getBytesTotal();
 }
 
+
 /*public*/
 bool
 NetConnection::loadCompleted()
 {
-	if ( !_loader.get() ) return false; // @@ correct ?
-	return _loader->completed();
+  if ( ! _loader.get() ) {
+    return false;
+  }
+
+  return _loader->completed();
 }
 
+
 std::auto_ptr<FLVParser>
 NetConnection::getConnectedParser() const
 {
-	std::auto_ptr<FLVParser> ret;
+  std::auto_ptr<FLVParser> ret;
 
-	if ( _loader.get() )
-	{
-		ret.reset(new FLVParser(*_loader));
-	}
+  if ( _loader.get() ) {
+    ret.reset( new FLVParser(*_loader) );
+  }
 
-	return ret;
+  return ret;
 }
 
+
 /// \brief callback to instantiate a new NetConnection object.
 /// \param fn the parameters from the Flash movie
 /// \return nothing from the function call.
@@ -212,6 +258,13 @@ netconnection_new(const fn_call& /* fn */)
 as_value
 NetConnection::connect_method(const fn_call& fn)
 {
+	// NOTE:
+	//
+	// NetConnection::connect() is *documented*, I repeat, *documented*, to require the
+	// "url" argument to be NULL in AS <= 2. This is *legal* and *required*. Anything
+	// other than NULL is undocumented behaviour, and I would like to know if there
+	// are any movies out there relying on it. --bjacques.
+
 	GNASH_REPORT_FUNCTION;
 
 	boost::intrusive_ptr<NetConnection> ptr = ensureType<NetConnection>(fn.this_ptr); 
@@ -227,17 +280,27 @@ NetConnection::connect_method(const fn_call& fn)
 	as_value& url_val = fn.arg(0);
 
 	// Check first arg for validity 
-	if ( url_val.is_null() || url_val.is_undefined() )
+	if ( url_val.is_null())
 	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		std::stringstream ss; fn.dump_args(ss);
-		log_aserror(_("NetConnection.connect(%s): invalid first arg"), ss.str().c_str());
-		);
+		// Null URL was passed. This is expected. Of course, it also makes this
+		// function (and, this class) rather useless. We return true, even though
+		// returning true has no meaning.
+		
+		return as_value(true);
+	}
+
+	// The remainder of this function is undocumented.
+	
+	if (url_val.is_undefined()) {
+                IF_VERBOSE_ASCODING_ERRORS(
+                log_aserror(_("NetConnection.connect(): first argument shouldn't be undefined"));
+                );
 		return as_value(false);
 	}
 
+
 	/// .. TODO: checkme ... addToURL ?? shoudnl't we attempt a connection ??
-	ptr->addToURL(url_val.to_string(&fn.env()));
+	ptr->addToURL(url_val.to_string());
 
 	if ( fn.nargs > 1 )
 	{
@@ -289,11 +352,13 @@ NetConnection::isConnected_getset(const fn_call& fn)
 	if ( fn.nargs == 0 ) // getter
 	{
 		log_unimpl("NetConnection.isConnected get");
-		return as_value();
+	  return as_value();
 	}
 	else // setter
 	{
-		log_unimpl("NetConnection.isConnected set");
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror("Tried to set read-only property NetConnection.isConnected");
+		);
 		return as_value();
 	}
 }
@@ -347,7 +412,7 @@ NetConnection::getNetConnectionInterface()
 	static boost::intrusive_ptr<as_object> o;
 	if ( o == NULL )
 	{
-		o = new as_object();
+		o = new as_object(getObjectInterface());
 		NetConnection::attachNetConnectionInterface(*o);
 	}
 
