@@ -1,9 +1,9 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -15,13 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: NetConnection.h,v 1.34 2007/06/01 23:36:48 bjacques Exp $ */
+/* $Id: NetConnection.h,v 1.38.2.1 2008/02/22 14:16:39 strk Exp $ */
 
 #ifndef __NETCONNECTION_H__
 #define __NETCONNECTION_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "tu_file.h"
@@ -58,6 +58,28 @@ public:
 	NetConnection();
 	~NetConnection();
 
+	/// Open a connection to stream FLV files.
+	//
+	/// If already connected an error is raised and false
+	/// is returned. Otherwise, a connection is attempted
+	/// using a separate thread that starts loading data
+	/// caching it.
+	///
+	/// @param url
+	///	An url portion to append to the base url (???)
+	///
+	/// @return true on success, false on error.
+	///
+	/// @note Older Flash movies can only take a NULL value as
+	/// the parameter, which therefor only connects to the localhost using
+	/// RTMP. Newer Flash movies have a parameter to connect which is a
+	/// URL string like rtmp://foobar.com/videos/bar.flv
+	///
+	std::string validateURL(const std::string& url);
+
+	/// Register the "NetConnection" constructor to the given global object
+	static void registerConstructor(as_object& global);
+
 	/// Open a connection to stream FLV files.
 	//
 	/// If already connected an error is raised and false
@@ -135,9 +157,6 @@ public:
 	///
 	bool loadCompleted();
 
-	/// Register the "NetConnection" constructor to the given global object
-	static void registerConstructor(as_object& global);
-
 private:
 
 	/// Extend the URL to be used for playing
@@ -149,9 +168,6 @@ private:
 	/// the complete url of the file
 	std::string _completeUrl;
 
-	/// the as_object which owns the connection
-	boost::intrusive_ptr<as_object> _owner;
-
 	/// The file/stream loader thread and interface
 	std::auto_ptr<LoadThread> _loader;
 
