@@ -1,10 +1,10 @@
 // NetStream.cpp:  ActionScript class for streaming audio/video, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: NetStream.cpp,v 1.63 2007/06/03 18:17:20 strk Exp $ */
+/* $Id: NetStream.cpp,v 1.84.2.1 2008/02/21 19:01:42 bjacques Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "NetStream.h"
@@ -35,9 +35,11 @@
 #include "builtin_function.h"
 #include "GnashException.h"
 #include "NetConnection.h"
-#include "action.h" // for call_method
 #include "render.h"	// for gnash::render::videoFrameFormat()
-
+#include "Object.h" // for getObjectInterface
+#include "VM.h"
+#include <boost/algorithm/string/case_conv.hpp> // for PROPNAME
+#include "namedStrings.h"
 #include "movie_root.h"
 
 // Define the following macro to have status notification handling debugged
@@ -130,10 +132,11 @@ static as_value netstream_pause(const fn_call& fn)
 	boost::intrusive_ptr<NetStream> ns = ensureType<NetStream>(fn.this_ptr);
 	
 	// mode: -1 ==> toogle, 0==> pause, 1==> play
-	int mode = -1;
+	NetStream::PauseMode mode = NetStream::pauseModeToggle;
 	if (fn.nargs > 0)
 	{
-		mode = fn.arg(0).to_bool() ? 0 : 1;
+		mode = fn.arg(0).to_bool() ? NetStream::pauseModePause :
+		                             NetStream::pauseModeUnPause;
 	}
 	ns->pause(mode);	// toggle mode
 	return as_value();
@@ -151,7 +154,15 @@ static as_value netstream_play(const fn_call& fn)
 		return as_value();
 	}
 
-	ns->play(fn.arg(0).to_string(&fn.env()));
+	if ( ! ns->isConnected() )
+	{
+		IF_VERBOSE_ASCODING_ERRORS(
+		log_aserror(_("NetStream.play(%s): stream is not connected"), fn.arg(0).to_debug_string().c_str());
+		);
+		return as_value();
+	}
+
+	ns->play(fn.arg(0).to_string());
 
 	return as_value();
 }
@@ -159,12 +170,12 @@ static as_value netstream_play(const fn_call& fn)
 static as_value netstream_seek(const fn_call& fn) {
 	boost::intrusive_ptr<NetStream> ns = ensureType<NetStream>(fn.this_ptr);
 
-	double time = 0;
+	boost::uint32_t time = 0;
 	if (fn.nargs > 0)
 	{
-		time = fn.arg(0).to_number(&fn.env());
+		time = fn.arg(0).to_number<boost::uint32_t>();
 	}
-	ns->seek(static_cast<uint32_t>(time*1000.0));
+	ns->seek(time);
 
 	return as_value();
 }
@@ -181,9 +192,9 @@ static as_value netstream_setbuffertime(const fn_call& fn)
 	double time = 0;
 	if (fn.nargs > 0)
 	{
-		time = fn.arg(0).to_number(&fn.env());
+		time = fn.arg(0).to_number();
 	}
-	ns->setBufferTime(uint32_t(time*1000));
+	ns->setBufferTime(boost::uint32_t(time*1000));
 
 	return as_value();
 }
@@ -290,6 +301,10 @@ netstream_bytesloaded(const fn_call& fn)
 {
 	boost::intrusive_ptr<NetStream> ns = ensureType<NetStream>(fn.this_ptr);
 
+	if ( ! ns->isConnected() )
+	{
+		return as_value();
+	}
 	long ret = ns->bytesLoaded();
 	return as_value(ret);
 }
@@ -300,6 +315,10 @@ netstream_bytestotal(const fn_call& fn)
 {
 	boost::intrusive_ptr<NetStream> ns = ensureType<NetStream>(fn.this_ptr);
 
+	if ( ! ns->isConnected() )
+	{
+		return as_value();
+	}
 	long ret = ns->bytesTotal();
 	return as_value(ret);
 }
@@ -310,22 +329,23 @@ netstream_currentFPS(const fn_call& fn)
 {
 	boost::intrusive_ptr<NetStream> ns = ensureType<NetStream>(fn.this_ptr);
 
-	bool warned = false;
-	if ( ! warned ) {
-		log_unimpl("NetStream.currentFPS getter/setter");
-		warned = true;
-	}
-	if ( fn.nargs == 0 ) // getter
+	if ( ! ns->isConnected() )
 	{
 		return as_value();
 	}
-	else // setter
-	{
-		return as_value();
+
+	double fps = ns->getCurrentFPS();
+
+#if 0
+	if (fps <= 0) {
+		return as_value(); // undef
 	}
+#endif
+
+	return as_value(fps);
 }
 
-// Both a getter and a (do-nothing) setter for bufferLength
+// read-only property bufferLength: amount of time buffered before playback
 static as_value
 netstream_bufferLength(const fn_call& fn)
 {
@@ -402,7 +422,7 @@ attachNetStreamInterface(as_object& o)
 	o.init_readonly_property("bytesTotal", *gettersetter);
 
 	gettersetter = new builtin_function(&netstream_currentFPS, NULL);
-	o.init_readonly_property("currentFPS", *gettersetter);
+	o.init_readonly_property("currentFps", *gettersetter);
 
 	gettersetter = new builtin_function(&netstream_bufferLength, NULL);
 	o.init_readonly_property("bufferLength", *gettersetter);
@@ -422,7 +442,7 @@ getNetStreamInterface()
 	static boost::intrusive_ptr<as_object> o;
 	if ( o == NULL )
 	{
-		o = new as_object();
+		o = new as_object(getObjectInterface());
 		attachNetStreamInterface(*o);
 	}
 
@@ -450,16 +470,26 @@ void netstream_class_init(as_object& global)
 
 }
 
+
+void
+NetStream::processNotify(std::string& funcname, boost::intrusive_ptr<as_object>& info_obj)
+{
+	// TODO: check for System.onStatus too ! use a private getStatusHandler() method for this.
+
+#ifdef GNASH_DEBUG_METADATA
+  log_debug(" Invoking onMetaData");
+#endif
+
+        string_table::key func = getVM().getStringTable().find(PROPNAME(funcname));
+
+	callMethod(func, as_value(info_obj.get()));
+}
+
+
 void
 NetStream::processStatusNotifications()
 {
 	// TODO: check for System.onStatus too ! use a private getStatusHandler() method for this.
-	as_value status;
-	if ( ! get_member("onStatus", &status) || ! status.is_function())
-	{
-		clearStatusQueue();
-		return;
-	}
 
 	StatusCode code;
 	while (1)
@@ -474,11 +504,8 @@ NetStream::processStatusNotifications()
 		// TODO: optimize by reusing the same as_object ?
 		boost::intrusive_ptr<as_object> o = getStatusObject(code);
 
-		m_env->push_val(as_value(o.get()));
-		call_method(status, m_env, this, 1, m_env->get_top_index() );
+		callMethod(NSV::PROP_ON_STATUS, as_value(o.get()));
 	}
-
-
 }
 
 void
@@ -495,26 +522,12 @@ NetStream::setStatus(StatusCode status)
 }
 
 void
-NetStream::setBufferTime(uint32_t time)
+NetStream::setBufferTime(boost::uint32_t time)
 {
 	// The argument is in milliseconds,
 	m_bufferTime = time;
 }
 
-long
-NetStream::bytesLoaded()
-{
-	if (_netCon == NULL) return 0;
-	return _netCon->getBytesLoaded();
-}
-
-long
-NetStream::bytesTotal()
-{
-	if (_netCon == NULL) return 0;
-	return _netCon->getBytesTotal();
-}
-
 long
 NetStream::bufferLength()
 {
@@ -541,20 +554,10 @@ NetStream::get_video()
 {
 	boost::mutex::scoped_lock lock(image_mutex);
 
+	if (!m_imageframe) return std::auto_ptr<image::image_base>(0);
+
 	// TODO: inspect if we could return m_imageframe directly...
-
-	std::auto_ptr<image::image_base> ret_image;
-
-	if (!m_imageframe) return ret_image;
-
-	if (m_videoFrameFormat == render::YUV) {
-		ret_image.reset(new image::yuv(m_imageframe->m_width, m_imageframe->m_height));
-	} else if (m_videoFrameFormat == render::RGB) {
-		ret_image.reset(new image::rgb(m_imageframe->m_width, m_imageframe->m_height));
-	} 
-
-	ret_image->update(m_imageframe->m_data);
-	return ret_image;
+	return m_imageframe->clone();	
 }
 
 std::pair<const char*, const char*>
@@ -598,7 +601,7 @@ NetStream::getStatusObject(StatusCode code)
 	// code, level
 	std::pair<const char*, const char*> info = getStatusCodeInfo(code);
 
-	boost::intrusive_ptr<as_object> o = new as_object();
+	boost::intrusive_ptr<as_object> o = new as_object(getObjectInterface());
 	o->init_member("code",  info.first,  1);
 	o->init_member("level", info.second, as_prop_flags::dontDelete|as_prop_flags::dontEnum);
 
@@ -628,5 +631,20 @@ NetStream::clearStatusQueue()
 	_statusQueue.clear();
 }
 
+#ifdef GNASH_USE_GC
+void
+NetStream::markReachableResources() const
+{
+
+	if ( _netCon ) _netCon->setReachable();
+
+	if ( m_statusHandler ) m_statusHandler->setReachable();
+
+	if ( m_env ) m_env->markReachableResources();
+
+	// Invoke generic as_object marker
+	markAsObjectReachable();
+}
+#endif // GNASH_USE_GC
 
 } // end of gnash namespace
