@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -15,17 +15,13 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-//
-
-/*  $Id: NetStream.h,v 1.48 2007/08/03 20:54:47 strk Exp $ */
+/*  $Id: NetStream.h,v 1.58.2.1 2008/02/21 19:01:42 bjacques Exp $ */
 
 #ifndef __NETSTREAM_H__
 #define __NETSTREAM_H__
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #ifndef __STDC_CONSTANT_MACROS
@@ -36,6 +32,7 @@
 #include "video_stream_instance.h"
 #include "NetConnection.h"
 #include "FLVParser.h"
+#include "as_function.h" // for visibility of destructor by intrusive_ptr
 
 #include <deque>
 
@@ -86,6 +83,7 @@ protected:
 		/// NetStream.Seek.InvalidTime (level: error)
 		invalidTime
 	};
+	
 
 	boost::intrusive_ptr<NetConnection> _netCon;
 
@@ -120,6 +118,9 @@ protected:
 	/// possibly trying to obtain the lock again (::play, ::pause, ...)
 	///
 	void processStatusNotifications();
+	
+	
+	void processNotify(std::string& funcname, boost::intrusive_ptr<as_object>& metadata_obj);
 
 	/// The actionscript enviroment for the AS callbacks
 	//
@@ -131,7 +132,7 @@ protected:
 	as_environment* m_env;
 
 	// The size of the buffer in milliseconds
-	uint32_t m_bufferTime;
+	boost::uint32_t m_bufferTime;
 
 	// The video outputformat
 	int m_videoFrameFormat;
@@ -186,10 +187,17 @@ protected:
 
 public:
 
+	enum PauseMode {
+	  pauseModeToggle = -1,
+	  pauseModePause = 0,
+	  pauseModeUnPause = 1	
+	};
+
 	NetStream();
 
+#if !defined(sgi) || defined(__GNUC__)
 	virtual ~NetStream(){}
-
+#endif
 	/// Closes the video session and frees all ressources used for decoding
 	/// except the FLV-parser (this might not be correct).
 	virtual void close(){}
@@ -197,12 +205,8 @@ public:
 	/// Pauses/starts the playback of the media played by the current instance
 	//
 	/// @param mode
-	///	Defines what mode to put the instance in. 
-	/// -1 : toogle mode
-	/// 0 : switch to pause
-	/// 1 : switch to play
-	///
-	virtual void pause(int /*mode*/){}
+	///	Defines what mode to put the instance in.
+	virtual void pause(PauseMode /*mode*/){}
 
 	/// Starts the playback of the media
 	//
@@ -214,20 +218,24 @@ public:
 	/// Seek in the media played by the current instance
 	//
 	/// @param position
-	///	Defines in milliseconds where to seek to
+	///	Defines in seconds where to seek to
 	///
-	virtual void seek(uint32_t /*pos*/){}
+	virtual void seek(boost::uint32_t /*pos*/){}
 
 	/// Tells where the playhead currently is
 	//
 	/// @return The time in milliseconds of the current playhead position
 	///
-	virtual int32_t time() { return 0; }
+	virtual boost::int32_t time() { return 0; }
 
 	/// Called at the SWF framerate. Used to process queued status messages
 	/// and (re)start after a buffering pause. In NetStreamFfmpeg it is also
 	/// used to find the next video frame to be shown, though this might change.
 	virtual void advance(){}
+	
+	/// Returns the current framerate in frames per second.
+	virtual double getCurrentFPS() { return 0; }
+	
 
 	/// Sets the NetConnection needed to access external files
 	//
@@ -238,6 +246,9 @@ public:
 		_netCon = nc;
 	}
 
+	/// Return true if the NetStream has an associated NetConnection
+	bool isConnected() const { return _netCon != 0; }
+
 	/// Sets the AS Enviroment needed for eventhandlers
 	//
 	/// @param enviroment
@@ -254,25 +265,22 @@ public:
 	/// @param time
 	/// The time in milliseconds that should be buffered.
 	///
-	void setBufferTime(uint32_t time);
+	void setBufferTime(boost::uint32_t time);
 
 	/// Returns what the buffer time has been set to. (100 miliseconds is default)
 	//
 	/// @return The size of the buffer in milliseconds.
 	///
-	uint32_t bufferTime() { return m_bufferTime; }
+	boost::uint32_t bufferTime() { return m_bufferTime; }
 
-	/// Returns the number of bytes loaded of the media file
-	//
-	/// @return the number of bytes loaded of the media file
-	///
-	long bytesLoaded();
+	/// Returns the number of bytes of the media file that have been buffered.
+	virtual long bytesLoaded() { return 0; }
 
 	/// Returns the total number of bytes (size) of the media file
 	//
 	/// @return the total number of bytes (size) of the media file
 	///
-	long bytesTotal();
+	virtual long bytesTotal() { return 0;}
 
 	/// Returns the number of millisecond of the media file that is buffered and 
 	/// yet to be played
@@ -292,6 +300,8 @@ public:
 	/// @return a image containing the video frame, a NULL auto_ptr if none were ready
 	///
 	std::auto_ptr<image::image_base> get_video();
+	
+
 
 private:
 
