@@ -1,6 +1,6 @@
 // Number.cpp:  ActionScript Number class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: Number.cpp,v 1.32 2007/08/01 15:56:54 strk Exp $ */
+/* $Id: Number.cpp,v 1.38 2008/02/19 12:06:07 strk Exp $ */
 
 #include "log.h"
 #include "tu_config.h"
@@ -28,6 +28,7 @@
 #include "as_value.h" // for doubleToString
 #include "builtin_function.h" // need builtin_function
 #include "VM.h" // for registering static GcResources (constructor and prototype)
+#include "Object.h" // for getObjectInterface
 
 #include <sstream>
 #include <cmath>
@@ -49,30 +50,12 @@
 #undef FP_NAN
 #define FP_NAN           16 
 
-using namespace std;
-
 namespace gnash {
 
 // Forward declarations
 //static void number_val_to_str(double val, char *str);
 //static as_value number_to_string(const fn_call& fn);
-
-static void
-attachNumberInterface(as_object& o)
-{
-	// FIXME: add Number interface here:
-	// Number.MAX_VALUE
-	// Number.MIN_VALUE
-	// Number.NaN
-	// Number.NEGATIVE_INFINITY
-	// Number.POSITIVE_INFINITY
-
-	// Number.toString()
-	o.init_member("toString", new builtin_function(as_object::tostring_method));
-
-	// Number.valueOf()
-	o.init_member("valueOf", new builtin_function(as_object::valueof_method));
-}
+static void attachNumberInterface(as_object& o);
 
 static as_object*
 getNumberInterface()
@@ -80,7 +63,7 @@ getNumberInterface()
 	static boost::intrusive_ptr<as_object> o=NULL;
 	if ( o == NULL )
 	{
-		o = new as_object();
+		o = new as_object(getObjectInterface());
 		VM::get().addStatic(o.get());
 
 		attachNumberInterface(*o);
@@ -124,8 +107,27 @@ public:
 		return _val;
 	}
 
+	static as_value toString_method(const fn_call& fn);
 };
 
+static void
+attachNumberInterface(as_object& o)
+{
+	// FIXME: add Number interface here:
+	// Number.MAX_VALUE
+	// Number.MIN_VALUE
+	// Number.NaN
+	// Number.NEGATIVE_INFINITY
+	// Number.POSITIVE_INFINITY
+
+	// Number.toString()
+	o.init_member("toString", new builtin_function(number_as_object::toString_method));
+
+	// Number.valueOf()
+	o.init_member("valueOf", new builtin_function(as_object::valueof_method));
+}
+
+
 static as_value
 number_ctor(const fn_call& fn)
 {
@@ -135,6 +137,11 @@ number_ctor(const fn_call& fn)
 		val = fn.arg(0).to_number();
 	}
 
+	if ( ! fn.isInstantiation() )
+	{
+		return as_value(val);
+	}
+
 	number_as_object* obj = new number_as_object(val);
 	
 	return as_value(obj); // will keep alive
@@ -178,5 +185,30 @@ init_number_instance(double val)
 	env.push(val);
 	return cl->constructInstance(env, 1, 0);
 }
+
+as_value
+number_as_object::toString_method(const fn_call& fn)
+{
+	boost::intrusive_ptr<as_object> obj = fn.this_ptr;
+
+	double val = obj->get_numeric_value();
+	unsigned radix=10;
+
+	if ( fn.nargs ) 
+	{
+		int userRadix = fn.arg(0).to_int();
+		if ( userRadix >= 2 && userRadix <= 36 ) radix=userRadix;
+		else
+		{
+			IF_VERBOSE_ASCODING_ERRORS(
+			log_aserror(_("Number.toString(%s): "
+				"radix must be in the 2..36 range (%d is invalid)"),
+				fn.arg(0).to_debug_string().c_str(), userRadix)
+			)
+		}
+
+	}
+	return as_value::doubleToString(val, radix); 
+}
   
 } // namespace gnash
