@@ -1,6 +1,6 @@
 // SoundFfmpeg.cpp:  Gnash sound output via FFMPEG library.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "SoundFfmpeg.h"
@@ -36,7 +36,7 @@ namespace gnash {
 
 // ffmpeg callback function
 int 
-SoundFfmpeg::readPacket(void* opaque, uint8_t* buf, int buf_size)
+SoundFfmpeg::readPacket(void* opaque, boost::uint8_t* buf, int buf_size)
 {
 
 	SoundFfmpeg* so = static_cast<SoundFfmpeg*>(opaque);
@@ -80,8 +80,9 @@ SoundFfmpeg::seekMedia(void *opaque, offset_t offset, int whence){
 
 
 void
-SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
+SoundFfmpeg::setupDecoder()
 {
+	SoundFfmpeg* so = this;
 
 	boost::intrusive_ptr<NetConnection> nc = so->connection;
 	assert(nc);
@@ -91,7 +92,9 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	if ( !nc->openConnection(so->externalURL) ) {
 		log_error(_("%s could not open audio url: %s"), 
 				__FUNCTION__, so->externalURL.c_str());
+#ifdef LOADS_IN_SEPARATE_THREAD
 		delete so->lock;
+#endif
 		return;
 	}
 
@@ -108,14 +111,16 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	// Probe the file to detect the format
 	AVProbeData probe_data, *pd = &probe_data;
 	pd->filename = "";
-	pd->buf = new uint8_t[2048];
+	pd->buf = new boost::uint8_t[2048];
 	pd->buf_size = 2048;
 
 	if (readPacket(so, pd->buf, pd->buf_size) < 1){
  		log_error(_("%s: could not read from audio url: %s"), 
 				__FUNCTION__, so->externalURL.c_str());
  		delete[] pd->buf;
+#ifdef LOADS_IN_SEPARATE_THREAD
  		delete so->lock;
+#endif
  		return;
 	}
 
@@ -126,7 +131,7 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 
 	// Setup the filereader/seeker mechanism. 7th argument (NULL) is the writer function,
 	// which isn't needed.
-	init_put_byte(&so->ByteIOCxt, new uint8_t[500000], 500000, 0, so, SoundFfmpeg::readPacket, NULL, SoundFfmpeg::seekMedia);
+	init_put_byte(&so->ByteIOCxt, new boost::uint8_t[500000], 500000, 0, so, SoundFfmpeg::readPacket, NULL, SoundFfmpeg::seekMedia);
 	so->ByteIOCxt.is_streamed = 1;
 
 	so->formatCtx = av_alloc_format_context();
@@ -134,7 +139,9 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	// Open the stream. the 4th argument is the filename, which we ignore.
 	if(av_open_input_stream(&so->formatCtx, &so->ByteIOCxt, "", inputFmt, NULL) < 0){
 		log_error(_("Couldn't open file '%s' for decoding"), so->externalURL.c_str());
+#ifdef LOADS_IN_SEPARATE_THREAD
 		delete so->lock;
+#endif
 		return;
 	}
 
@@ -144,7 +151,9 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	if (ret < 0)
 	{
 		log_error(_("Couldn't find stream information from '%s', error code: %d"), so->externalURL.c_str(), ret);
+#ifdef LOADS_IN_SEPARATE_THREAD
 		delete so->lock;
+#endif
 		return;
 	}
 
@@ -188,7 +197,9 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	{
 		log_error(_("No available audio decoder %d to process file: '%s'"),
 		   so->audioCodecCtx->codec_id, so->externalURL.c_str());
+#ifdef LOADS_IN_SEPARATE_THREAD
 		delete so->lock;
+#endif
 		return;
 	}
     
@@ -197,17 +208,21 @@ SoundFfmpeg::setupDecoder(SoundFfmpeg* so)
 	{
 		log_error(_("Could not open audio codec %d for %s"),
 		   so->audioCodecCtx->codec_id, so->externalURL.c_str());
+#ifdef LOADS_IN_SEPARATE_THREAD
 		delete so->lock;
+#endif
 		return;
 	}
 
 	// By deleting this lock we allow start() to start playback
+#ifdef LOADS_IN_SEPARATE_THREAD
 	delete so->lock;
+#endif
 	return;
 }
 
 // audio callback is running in sound handler thread
-bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
+bool SoundFfmpeg::getAudio(void* owner, boost::uint8_t* stream, int len)
 {
 	SoundFfmpeg* so = static_cast<SoundFfmpeg*>(owner);
 
@@ -225,7 +240,7 @@ bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
 				delete[] so->leftOverData;
 				so->leftOverSize = 0;
 			} else {
-				uint8_t* buf = new uint8_t[rest];
+				boost::uint8_t* buf = new boost::uint8_t[rest];
 				memcpy(stream, so->leftOverData+len, rest);
 				delete[] so->leftOverData;
 				so->leftOverData = buf;
@@ -243,7 +258,7 @@ bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
 	AVPacket packet;
 	int rc;
 	bool loop = true;
-	uint8_t* ptr = new uint8_t[AVCODEC_MAX_AUDIO_FRAME_SIZE];
+	boost::uint8_t* ptr = new boost::uint8_t[AVCODEC_MAX_AUDIO_FRAME_SIZE];
 	bool ret = true;
 	while (loop) {
 		// Parse file
@@ -252,23 +267,24 @@ bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
 		{
 			if (packet.stream_index == so->audioIndex)
 			{
-				sound_handler* s = get_sound_handler();
+				media::sound_handler* s = get_sound_handler();
 				if (s)
 				{
 					// Decode audio
 					int frame_size;
 #ifdef FFMPEG_AUDIO2
 					frame_size = (AVCODEC_MAX_AUDIO_FRAME_SIZE * 3) / 2;
-					if (avcodec_decode_audio2(so->audioCodecCtx, (int16_t*) ptr, &frame_size, packet.data, packet.size) >= 0)
+					if (avcodec_decode_audio2(so->audioCodecCtx, (boost::int16_t*) ptr, &frame_size, packet.data, packet.size) >= 0)
 #else
-					if (avcodec_decode_audio(so->audioCodecCtx, (int16_t*) ptr, &frame_size, packet.data, packet.size) >= 0)
+					if (avcodec_decode_audio(so->audioCodecCtx, (boost::int16_t*) ptr, &frame_size, packet.data, packet.size) >= 0)
 #endif
 					{
 
 						bool stereo = so->audioCodecCtx->channels > 1 ? true : false;
 						int samples = stereo ? frame_size >> 2 : frame_size >> 1;
 						int newDataSize = 0;
-						int16_t* output_data = NULL;
+						boost::int16_t* output_data = NULL;
+						bool output_data_allocated = false;
 
 						// Resample if needed
 						if (so->audioCodecCtx->channels != 2 || so->audioCodecCtx->sample_rate != 44100) {
@@ -278,11 +294,12 @@ bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
 								so->resampleCtx = audio_resample_init (2, so->audioCodecCtx->channels, 44100, so->audioCodecCtx->sample_rate);
 							}
 							// The size of this is a guess, we don't know yet... Lets hope it's big enough
-							output_data = new int16_t[AVCODEC_MAX_AUDIO_FRAME_SIZE];
-							samples = audio_resample (so->resampleCtx, output_data, (int16_t*)ptr, samples);
+							output_data = new boost::int16_t[AVCODEC_MAX_AUDIO_FRAME_SIZE];
+							output_data_allocated = true;
+							samples = audio_resample (so->resampleCtx, output_data, (boost::int16_t*)ptr, samples);
 							newDataSize =  samples * 2 * 2; // 2 for stereo and 2 for samplesize = 2 bytes
 						} else {
-							output_data = (int16_t*)ptr;
+							output_data = (boost::int16_t*)ptr;
 							newDataSize = samples * 2 * 2;
 						}
 
@@ -290,20 +307,20 @@ bool SoundFfmpeg::getAudio(void* owner, uint8_t* stream, int len)
 						// If the decoded data isn't enough to fill the buffer, we put the decoded
 						// data into the buffer, and continues decoding.
 						if (newDataSize <= len-pos) {
-							memcpy(stream+pos, (uint8_t*)output_data, newDataSize);
+							memcpy(stream+pos, (boost::uint8_t*)output_data, newDataSize);
 							pos += newDataSize;
 						} else {
 						// If we can fill the buffer, and still have "leftovers", we save them
 						// and use them later.
 							int rest = len-pos;
 							so->leftOverSize = newDataSize - rest;
-							memcpy(stream+pos, (uint8_t*)output_data, rest);
-							so->leftOverData = new uint8_t[so->leftOverSize];
-							memcpy(so->leftOverData, ((uint8_t*)output_data)+rest, so->leftOverSize);
+							memcpy(stream+pos, (boost::uint8_t*)output_data, rest);
+							so->leftOverData = new boost::uint8_t[so->leftOverSize];
+							memcpy(so->leftOverData, ((boost::uint8_t*)output_data)+rest, so->leftOverSize);
 							loop = false;
 							pos += rest;
 						}
-						delete[] output_data;
+						if ( output_data_allocated ) delete[] output_data;
 					}
 				}
 			}
@@ -348,7 +365,7 @@ SoundFfmpeg::~SoundFfmpeg() {
 		}
 
 		if (isAttached) {
-			sound_handler* s = get_sound_handler();
+			media::sound_handler* s = get_sound_handler();
 			if (s) {
 				s->detach_aux_streamer(this);
 			}
@@ -357,7 +374,7 @@ SoundFfmpeg::~SoundFfmpeg() {
 }
 
 void
-SoundFfmpeg::loadSound(std::string file, bool streaming)
+SoundFfmpeg::loadSound(const std::string& file, bool streaming)
 {
 	leftOverData = NULL;
 	leftOverSize = 0;
@@ -375,17 +392,23 @@ SoundFfmpeg::loadSound(std::string file, bool streaming)
 	externalSound = true;
 	isStreaming = streaming;
 
+#ifdef LOADS_IN_SEPARATE_THREAD
 	lock = new boost::mutex::scoped_lock(setupMutex);
 
 	// To avoid blocking while connecting, we use a thread.
-	setupThread = new boost::thread(boost::bind(SoundFfmpeg::setupDecoder, this));
+	setupThread = new boost::thread(boost::bind(&SoundFfmpeg::setupDecoder, this));
+#else
+	setupDecoder();
+#endif
 
 }
 
 void
 SoundFfmpeg::start(int offset, int loops)
 {
+#ifdef LOADS_IN_SEPARATE_THREAD
 	boost::mutex::scoped_lock lock(setupMutex);
+#endif
 
 	if (externalSound) {
 		if (offset > 0) {
@@ -406,7 +429,7 @@ SoundFfmpeg::start(int offset, int loops)
 	}
 
 	// Start sound
-	sound_handler* s = get_sound_handler();
+	media::sound_handler* s = get_sound_handler();
 	if (s) {
 		if (externalSound) {
 			if (audioIndex >= 0)
@@ -424,7 +447,7 @@ void
 SoundFfmpeg::stop(int si)
 {
 	// stop the sound
-	sound_handler* s = get_sound_handler();
+	media::sound_handler* s = get_sound_handler();
 	if (s != NULL)
 	{
 	    if (si < 0) {
@@ -445,7 +468,7 @@ SoundFfmpeg::getDuration()
 
 	// If this is a event sound get the info from the soundhandler
 	if (!externalSound) {
-		sound_handler* s = get_sound_handler();
+		media::sound_handler* s = get_sound_handler();
 		if (s) {		
 	    	return (s->get_duration(soundId));
 	    } else {
@@ -466,7 +489,7 @@ SoundFfmpeg::getPosition()
 {
 	// If this is a event sound get the info from the soundhandler
 	if (!externalSound) {
-		sound_handler* s = get_sound_handler();
+		media::sound_handler* s = get_sound_handler();
 		if (s) {
 			return s->get_position(soundId);
 	    } else {
@@ -475,10 +498,13 @@ SoundFfmpeg::getPosition()
 	}
 
 	// Return the position in the file in milliseconds
-	if (formatCtx && audioIndex) {
+	if (formatCtx && audioIndex >= 0)
+	{
 		double time = (double)formatCtx->streams[audioIndex]->time_base.num / formatCtx->streams[audioIndex]->time_base.den * (double)formatCtx->streams[audioIndex]->cur_dts;
 		return static_cast<unsigned int>(time * 1000);
-	} else {
+	}
+	else
+	{
 		return 0;
 	}
 }
