@@ -1,6 +1,6 @@
 // Stage.cpp:  All the world is one, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "Stage.h"
@@ -28,106 +28,105 @@
 #include "smart_ptr.h" // for boost intrusive_ptr
 #include "builtin_function.h" // need builtin_function
 #include "VM.h"
+#include "Object.h" // for getObjectInterface()
+#include "AsBroadcaster.h" // for initializing self as a broadcaster
+#include "namedStrings.h"
 
 #include <string>
-#include <boost/algorithm/string/case_conv.hpp> // for using PROPNAME
 
 namespace gnash {
 
-as_value stage_addlistener(const fn_call& fn);
-as_value stage_removelistener(const fn_call& fn);
 as_value stage_scalemode_getset(const fn_call& fn);
+as_value stage_align_getset(const fn_call& fn);
+as_value stage_showMenu_getset(const fn_call& fn);
 as_value stage_width_getset(const fn_call& fn);
 as_value stage_height_getset(const fn_call& fn);
 
 static void
 attachStageInterface(as_object& o)
 {
-	if ( VM::get().getSWFVersion() < 6 ) return;
+	VM& vm = o.getVM();
 
-	o.init_member("addListener", new builtin_function(stage_addlistener));
-	o.init_member("removeListener", new builtin_function(stage_removelistener));
+	if ( vm.getSWFVersion() < 6 ) return;
 
-	boost::intrusive_ptr<builtin_function> getset(new builtin_function(stage_scalemode_getset));
-	o.init_property("scaleMode", *getset, *getset);
+	// Stage.scaleMode getter-setter
+	vm.registerNative(stage_scalemode_getset, 666, 1);
+	vm.registerNative(stage_scalemode_getset, 666, 2);
+	o.init_property("scaleMode", *(vm.getNative(666,1)), *(vm.getNative(666,2)));
 
-	getset = new builtin_function(stage_width_getset);
-	o.init_property("width", *getset, *getset);
+	// Stage.align getter-setter
+	vm.registerNative(stage_align_getset, 666, 3);
+	vm.registerNative(stage_align_getset, 666, 4);
+	o.init_property("width", *(vm.getNative(666,3)), *(vm.getNative(666,4)));
+
+	// Stage.width getter-setter
+	vm.registerNative(stage_width_getset, 666, 5);
+	vm.registerNative(stage_width_getset, 666, 6);
+	o.init_property("width", *(vm.getNative(666,5)), *(vm.getNative(666,6)));
+
+	// Stage.height getter-setter
+	vm.registerNative(stage_height_getset, 666, 7);
+	vm.registerNative(stage_height_getset, 666, 8);
+	o.init_property("height", *(vm.getNative(666,7)), *(vm.getNative(666,8)));
+
+	// Stage.showMenu getter-setter
+	vm.registerNative(stage_showMenu_getset, 666, 9);
+	vm.registerNative(stage_showMenu_getset, 666, 10);
+	o.init_property("showMenu", *(vm.getNative(666,9)), *(vm.getNative(666,10)));
 
-	getset = new builtin_function(stage_height_getset);
-	o.init_property("height", *getset, *getset);
 }
 
 Stage::Stage()
 	:
+	as_object(getObjectInterface()),
 	_scaleMode(showAll)
 {
 	attachStageInterface(*this);
+
+	int swfversion = _vm.getSWFVersion();
+	if ( swfversion > 5 )
+	{
+		AsBroadcaster::initialize(*this);
+	}
 }
 
 void
-Stage::onResize(as_environment* env)
+Stage::onResize()
 {
 	as_value v;
-	if ( get_member(PROPNAME("scaleMode"), &v) && v.to_string(env) == "noScale" )
+	if (get_member(NSV::PROP_SCALE_MODE, &v) && v.to_string() == "noScale" )
 	{
-		notifyResize(env);
+		notifyResize();
 	}
 }
 
 void
-Stage::notifyResize(as_environment* env)
+Stage::notifyResize()
 {
-	for (ListenersList::iterator it=_listeners.begin(),
-			itEnd=_listeners.end();
-			it != itEnd; ++it)
-	{
-		notifyResize(*it, env);
-	}
-}
-
-/// Notify an object about an resize event
-void
-Stage::notifyResize(boost::intrusive_ptr<as_object> obj, as_environment* env)
-{
-	std::string eventname = PROPNAME("onResize");
-
-	as_value method;
-	if ( ! obj->get_member(eventname, &method) ) {
-		// nothing to do
-		return;
-	}
-
-	boost::intrusive_ptr<as_function> func = method.to_as_function();
-	if ( ! func ) return; // method is not a function
-
-	func->call(fn_call(obj.get(), env, 0, 0));
+	log_debug("notifying Stage listeners about a resize");
+	callMethod(NSV::PROP_BROADCAST_MESSAGE, "onResize");
 }
 
 unsigned
 Stage::getWidth() const
 {
+  //if (_scaleMode==showAll)      <-- the only mode implemented in Gnash 
+	return (unsigned int)VM::get().getRoot().get_movie_definition()->get_width_pixels();
+	/*
+	else
 	return VM::get().getRoot().getWidth();
+	*/	
 }
 
 unsigned
 Stage::getHeight() const
 {
+  //if (_scaleMode==showAll)      <-- the only mode implemented in Gnash 
+	return (unsigned int)VM::get().getRoot().get_movie_definition()->get_height_pixels();
+	/*
+	else
 	return VM::get().getRoot().getHeight();
-}
-
-void
-Stage::addListener(boost::intrusive_ptr<as_object> obj)
-{
-	//log_msg("Adding listener %p to Stage %p", obj.get(), this);
-	_listeners.push_back(obj);
-}
-
-void
-Stage::removeListener(boost::intrusive_ptr<as_object> obj)
-{
-	//log_msg("Removing listener %p from Stage %p", obj.get(), this);
-	_listeners.remove(obj);
+	*/	
 }
 
 const char*
@@ -145,70 +144,21 @@ Stage::getScaleModeString()
 void
 Stage::setScaleMode(ScaleMode mode)
 {
+	if ( _scaleMode == mode ) return; // nothing to do
+
 	_scaleMode = mode;
+	//log_debug("Scale mode set to %s", getScaleModeString());
 
-	//log_msg("Scale mode set to %s", getScaleModeString());
 	if ( _scaleMode == noScale )
 	{
-		//log_msg("Setting rescaling allowance to false");
+		//log_debug("Setting rescaling allowance to false");
 		VM::get().getRoot().allowRescaling(false);
 	}
-}
-
-as_value stage_addlistener(const fn_call& fn)
-{
-	boost::intrusive_ptr<Stage> stage = ensureType<Stage>(fn.this_ptr);
-
-	if ( fn.nargs < 1 )
+	else
 	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("Stage.addListener() needs one argument"));
-		);
-		return as_value();
+		//log_debug("Setting rescaling allowance to true");
+		VM::get().getRoot().allowRescaling(true);
 	}
-
-	boost::intrusive_ptr<as_object> obj = fn.arg(0).to_object();
-	if ( ! obj )
-	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		std::stringstream ss;
-		fn.dump_args(ss);
-		log_aserror(_("Invalid call to Stage.addListener(%s) : first arg doesn't cast to an object"),
-			ss.str().c_str());
-		);
-		return as_value();
-	}
-
-	stage->addListener(obj);
-	return as_value();
-}
-
-as_value stage_removelistener(const fn_call& fn)
-{
-	boost::intrusive_ptr<Stage> stage = ensureType<Stage>(fn.this_ptr);
-
-	if ( fn.nargs < 1 )
-	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		log_aserror(_("Stage.removeListener() needs one argument"));
-		);
-		return as_value();
-	}
-
-	boost::intrusive_ptr<as_object> obj = fn.arg(0).to_object();
-	if ( ! obj )
-	{
-		IF_VERBOSE_ASCODING_ERRORS(
-		std::stringstream ss;
-		fn.dump_args(ss);
-		log_aserror(_("Invalid call to Stage.removeListener(%s) : first arg doesn't cast to an object"),
-			ss.str().c_str());
-		);
-		return as_value();
-	}
-
-	stage->removeListener(obj);
-	return as_value();
 }
 
 as_value stage_scalemode_getset(const fn_call& fn)
@@ -223,7 +173,7 @@ as_value stage_scalemode_getset(const fn_call& fn)
 	{
 		Stage::ScaleMode mode = Stage::showAll;
 
-		const std::string& str = fn.arg(0).to_string(&(fn.env()));
+		const std::string& str = fn.arg(0).to_string();
 		if ( str == "noScale" ) mode = Stage::noScale;
 		else if ( str == "exactFill" ) mode = Stage::exactFill;
 		else if ( str == "noBorder" ) mode = Stage::noBorder;
@@ -269,6 +219,56 @@ stage_height_getset(const fn_call& fn)
 	}
 }
 
+as_value
+stage_align_getset(const fn_call& fn)
+{
+	boost::intrusive_ptr<Stage> stage = ensureType<Stage>(fn.this_ptr);
+
+	if ( fn.nargs == 0 ) // getter
+	{
+		static bool warned=false;
+		if ( ! warned ) {
+			log_unimpl("Stage.align getter");
+			warned=true;
+		}
+		return as_value();
+	}
+	else // setter
+	{
+		static bool warned=false;
+		if ( ! warned ) {
+			log_unimpl("Stage.align setter");
+			warned=true;
+		}
+		return as_value();
+	}
+}
+
+as_value
+stage_showMenu_getset(const fn_call& fn)
+{
+	boost::intrusive_ptr<Stage> stage = ensureType<Stage>(fn.this_ptr);
+
+	if ( fn.nargs == 0 ) // getter
+	{
+		static bool warned=false;
+		if ( ! warned ) {
+			log_unimpl("Stage.showMenu getter");
+			warned=true;
+		}
+		return as_value();
+	}
+	else // setter
+	{
+		static bool warned=false;
+		if ( ! warned ) {
+			log_unimpl("Stage.showMenu setter");
+			warned=true;
+		}
+		return as_value();
+	}
+}
+
 // extern (used by Global.cpp)
 void stage_class_init(as_object& global)
 {
@@ -276,18 +276,4 @@ void stage_class_init(as_object& global)
 	global.init_member("Stage", obj.get());
 }
 
-#ifdef GNASH_USE_GC
-void
-Stage::markReachableResources() const
-{
-        for (ListenersList::const_iterator i=_listeners.begin(), e=_listeners.end(); i!=e; ++i)
-        {
-                (*i)->setReachable();
-        }
-
-	// Invoke generic as_object marker
-	markAsObjectReachable();
-}
-#endif // def GNASH_USE_GC
-
 } // end of gnash namespace
