@@ -1,6 +1,6 @@
 // xmlnode.cpp:  ActionScript "XMLNode" class, for Gnash.
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,10 +16,10 @@
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-/* $Id: xmlnode.cpp,v 1.38 2007/08/04 04:19:29 strk Exp $ */
+/* $Id: xmlnode.cpp,v 1.46 2008/02/19 19:20:55 bwy Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "log.h"
@@ -29,6 +29,11 @@
 #include "array.h" // for childNodes
 #include "xmlnode.h"
 #include "log.h"
+#include "Object.h" // for getObjectInterface
+#include "VM.h" // for getting the string_table..
+#include "string_table.h" // ..for using the string_table
+
+#include <boost/algorithm/string/case_conv.hpp>
 
 #include <string>
 #include <sstream>
@@ -42,9 +47,6 @@
 #include <libxml/tree.h>
 #include <libxml/xmlreader.h>
 
-
-using namespace std;
-
 //#define DEBUG_MEMORY_ALLOCATION 1
 
 namespace gnash {
@@ -68,7 +70,7 @@ static as_value xmlnode_childNodes(const fn_call& fn);
 static as_value xmlnode_parentNode(const fn_call& fn);
 as_object* getXMLNodeInterface();
 
-static LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
+//static LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 
 
 //std::vector<as_object *> _xmlobjs;    // FIXME: hack alert
@@ -79,9 +81,9 @@ XMLNode::XMLNode()
     _parent(0),
     _type(tElement)
 {
-    //log_msg("%s: %p", __PRETTY_FUNCTION__, this);
+    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("\tCreating XMLNode data at %p"), this);
+    log_debug(_("\tCreating XMLNode data at %p"), this);
 #endif
 }
 
@@ -91,9 +93,9 @@ XMLNode::XMLNode(as_object* overridden_interface)
     _parent(0),
     _type(tElement)
 {
-    //log_msg("%s: %p", __PRETTY_FUNCTION__, this);
+    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("\tCreating XMLNode data at %p"), this);
+    log_debug(_("\tCreating XMLNode data at %p"), this);
 #endif
 }
 
@@ -119,9 +121,9 @@ XMLNode::XMLNode(const XMLNode& tpl, bool deep)
 
 XMLNode::~XMLNode()
 {
-    //log_msg("%s: %p", __PRETTY_FUNCTION__, this);
+    //log_debug("%s: %p", __PRETTY_FUNCTION__, this);
 #ifdef DEBUG_MEMORY_ALLOCATION
-    log_msg(_("\tDeleting XMLNode data %s at %p"), this->_name.c_str(), this);
+    log_debug(_("\tDeleting XMLNode data %s with value %s at %p"), this->_name.c_str(), this->_value.c_str(), this);
 #endif
 }
 
@@ -149,7 +151,7 @@ XMLNode::lastChild()
 	//GNASH_REPORT_FUNCTION;
 	if ( _children.empty() )
 	{
-			log_msg(_("XMLNode %p has no children"), (void*)this);
+			log_debug(_("XMLNode %p has no children"), (void*)this);
 			return NULL;
 	}
 	return _children.back();
@@ -175,7 +177,7 @@ boost::intrusive_ptr<XMLNode>
 XMLNode::cloneNode(bool deep)
 {
     //GNASH_REPORT_FUNCTION;
-    //log_msg(_("%s: deep is %d"), __PRETTY_FUNCTION__, deep);
+    //log_debug(_("%s: deep is %d"), __PRETTY_FUNCTION__, deep);
 
     boost::intrusive_ptr<XMLNode> newnode = new XMLNode(*this, deep);
 
@@ -237,7 +239,7 @@ XMLNode::previousSibling()
     {
         if (itx->get() == this)
         {
-            // log_msg("Found the previous XMLNode child !!!! %s <%p>", (*itx)->nodeName(), (void*)*itx);
+            // log_debug("Found the previous XMLNode child !!!! %s <%p>", (*itx)->nodeName(), (void*)*itx);
 		    return previous_node;
 		}
 		previous_node = itx->get();
@@ -253,12 +255,12 @@ XMLNode::nextSibling()
 
     if ( ! _parent)
     {
-            //log_msg("Node %p has no parent, returning NULL", this);
+            //log_debug("Node %p has no parent, returning NULL", this);
             return NULL;
     }
     if (_parent->_children.size() <= 1)
     {
-            //log_msg("Node %p parent has only this node, returning NULL", this);
+            //log_debug("Node %p parent has only this node, returning NULL", this);
             return NULL;
     }
 
@@ -268,7 +270,7 @@ XMLNode::nextSibling()
     {
         if (itx->get() == this)
         {
-            //log_msg("Found the next XMLNode child !!!! %s <%p>", (*itx)->nodeName().c_str(), (void*)itx->get());
+            //log_debug("Found the next XMLNode child !!!! %s <%p>", (*itx)->nodeName().c_str(), (void*)itx->get());
 		    return previous_node;
 		}
 		previous_node = itx->get();
@@ -294,10 +296,10 @@ XMLNode::stringify(const XMLNode& xml, std::ostream& xmlout)
     NodeType type = xml.nodeType();
 
 
-//    log_msg("%s: processing for object %s <%p>", __PRETTY_FUNCTION__, nodename, xml);
+//    log_debug("%s: processing for object %s <%p>", __PRETTY_FUNCTION__, nodename, xml);
 
 #ifdef GNASH_DEBUG
-    log_msg(_("Stringifying node %p with name %s, value %s, %u attributes and %u children"),
+    log_debug(_("Stringifying node %p with name %s, value %s, %u attributes and %u children"),
                     (void*)&xml, nodename, nodevalue, xml._attributes.size(), xml._children.size());
 #endif
 
@@ -339,7 +341,7 @@ XMLNode::stringify(const XMLNode& xml, std::ostream& xmlout)
     ChildList::const_iterator itx;
     for (itx = xml._children.begin(); itx != xml._children.end(); itx++)
     {
-//      log_msg(_("Found One XMLNode child.  %s <%p>"), (*itx)->nodeName().c_str(), (void*)*itx);
+//      log_debug(_("Found One XMLNode child.  %s <%p>"), (*itx)->nodeName().c_str(), (void*)*itx);
 //      cerr << "<" << (*it)->nodeName() << ">" << endl;
         (*itx)->toString(xmlout);
     }
@@ -422,7 +424,7 @@ getXMLNodeInterface()
 {
     static boost::intrusive_ptr<as_object> o;
     if ( o == NULL ) {
-        o = new as_object();
+        o = new as_object(getObjectInterface());
         attachXMLNodeInterface(*o);
     }
     return o.get();
@@ -436,10 +438,10 @@ xmlnode_new(const fn_call& fn)
     XMLNode *xml_obj = new XMLNode;
     if ( fn.nargs > 0 )
     {
-        xml_obj->nodeTypeSet(XMLNode::NodeType(int(fn.arg(0).to_number(&(fn.env())))));
+        xml_obj->nodeTypeSet(XMLNode::NodeType(int(fn.arg(0).to_number())));
         if (fn.nargs > 1)
         {
-            xml_obj->nodeValueSet(fn.arg(1).to_string(&(fn.env())));
+            xml_obj->nodeValueSet(fn.arg(1).to_string());
         }
     }
     
@@ -479,7 +481,7 @@ static as_value
 xmlnode_clonenode(const fn_call& fn)
 {
     //GNASH_REPORT_FUNCTION;
-//    log_msg("%s: %d args", __PRETTY_FUNCTION__, fn.nargs);
+//    log_debug("%s: %d args", __PRETTY_FUNCTION__, fn.nargs);
     boost::intrusive_ptr<XMLNode> ptr = ensureType<XMLNode>(fn.this_ptr);
 
     bool deep = false;
@@ -555,7 +557,7 @@ xmlnode_tostring(const fn_call& fn)
     
     std::stringstream ss;
     ptr->toString(ss);
-    //log_msg("Stringstream: %s", ss.str().c_str());
+    //log_debug("Stringstream: %s", ss.str().c_str());
 
     return as_value(ss.str());
 }
@@ -578,16 +580,16 @@ xmlnode_nodevalue(const fn_call& fn)
     as_value rv;
     rv.set_null();
     
-    //log_msg("xmlnode_nodevalue called with %d args against 'this' = %p", fn.nargs, ptr);
+    //log_debug("xmlnode_nodevalue called with %d args against 'this' = %p", fn.nargs, ptr);
     if ( fn.nargs == 0 )
     {
-	    //log_msg("  nodeValue() returns '%s'", ptr->nodeValue().c_str());
+	    //log_debug("  nodeValue() returns '%s'", ptr->nodeValue().c_str());
         const std::string& val = ptr->nodeValue();
         if ( ! val.empty() ) rv = val;
     }
     else
     {
-        ptr->nodeValueSet(fn.arg(0).to_string(&(fn.env())));
+        ptr->nodeValueSet(fn.arg(0).to_string());
     }
     return rv;
 }
@@ -607,7 +609,7 @@ xmlnode_nodename(const fn_call& fn)
     }
     else
     {
-	ptr->nodeNameSet(fn.arg(0).to_string(&(fn.env())));
+	ptr->nodeNameSet(fn.arg(0).to_string());
     }
     return rv;
 }
@@ -627,19 +629,27 @@ xmlnode_nodetype(const fn_call& fn)
 static as_value
 xmlnode_attributes(const fn_call& fn)
 {
-//    GNASH_REPORT_FUNCTION;
+    //GNASH_REPORT_FUNCTION;
     
     boost::intrusive_ptr<XMLNode> ptr = ensureType<XMLNode>(fn.this_ptr);
 
-    XMLNode::AttribList& attrs = ptr->attributes();
-    boost::intrusive_ptr<as_object> ret = new as_object();
-    for (XMLNode::AttribList::const_iterator it=attrs.begin(),
-         itEnd=attrs.end(); it != itEnd; ++it) {
+    VM& vm = ptr->getVM();
+    string_table& st = vm.getStringTable();
 
-         const XMLAttr& at = *it;
-         const std::string& name = at.name();
-         const std::string& val = at.value();
-         ret->init_member(name, val);
+    XMLNode::AttribList& attrs = ptr->attributes();
+    //log_debug("Node %p has %d attributes", (void*)ptr.get(), attrs.size());
+
+    boost::intrusive_ptr<as_object> ret = new as_object(); // attributes are not Object types (getObjectInterface());
+    for (XMLNode::AttribList::const_iterator it=attrs.begin(),
+        itEnd=attrs.end(); it != itEnd; ++it)
+    {
+
+        const XMLAttr& at = *it;
+        const std::string& name = at.name();
+        const std::string& val = at.value();
+        //log_debug("%s: %s", name.c_str(), val.c_str());
+        // These must be enumerable !
+        ret->set_member(st.find(name), val);
     }
 
     return as_value(ret); 
