@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,20 +17,21 @@
 
 // Based on work of Thatcher Ulrich <tu@tulrich.com> 2003
 
-/* $Id: fill_style.h,v 1.8 2007/08/07 20:53:10 strk Exp $ */
+/* $Id: fill_style.h,v 1.16 2008/01/21 20:55:50 rsavoye Exp $ */
 
 #ifndef GNASH_FILL_STYLE_H
 #define GNASH_FILL_STYLE_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
-//#include "impl.h"
 #include "types.h"
 #include "matrix.h"
 #include "bitmap_character_def.h"
 
+#include <vector> // for composition
+
 namespace gnash {
 
 class stream;
@@ -38,11 +39,22 @@ class stream;
 class gradient_record
 {
 public:
-	gradient_record();
+	gradient_record()
+		:
+		m_ratio(0),
+		m_color()
+	{}
+
+	gradient_record(boost::uint8_t ratio, const rgba& color)
+		:
+		m_ratio(ratio),
+		m_color(color)
+	{}
+
 	void	read(stream* in, int tag_type);
 	
 	//data:
-	uint8_t	m_ratio;
+	boost::uint8_t	m_ratio;
 	rgba	m_color;
 };
 
@@ -60,12 +72,57 @@ public:
 	/// use by bitmap shape character.
 	///
 	/// TODO: use a subclass for this
+	/// TODO: provide a setBitmap, for consisteny with other setType() methods
 	///
-	fill_style(bitmap_character_def* bitmap);
+	/// @param bitmap
+	///	The bitmap character definition to use with this bitmap fill.
+	///
+	/// @param mat
+	///	The matrix to apply to the bitmap.
+	///
+	fill_style(bitmap_character_def* bitmap, const matrix& mat);
 
 	void setSolid(const rgba& color);
 
-	virtual ~fill_style();
+	/// Turn this fill style into a linear gradient
+	//
+	/// @param gradients
+	///	Gradient records.
+	///
+	/// @param mat
+	///	Gradient matrix.
+	///
+	///
+	void setLinearGradient(const std::vector<gradient_record>& gradients, 
+			const matrix& mat);
+
+	/// Turn this fill style into a radial gradient
+	//
+	/// @param gradients
+	///	Gradient records.
+	///
+	/// @param mat
+	///	Gradient matrix.
+	///
+	///
+	void setRadialGradient(const std::vector<gradient_record>& gradients,
+			const matrix& mat);
+
+	/// Turn this fill style into a focal gradient
+	//
+	/// @param gradients
+	///	Gradient records.
+	///
+	/// @param mat
+	///	Gradient matrix.
+	///
+	/// @param fpoint
+	///	Focal point.
+	///
+	void setRadialGradient(const std::vector<gradient_record>& gradients,
+			const matrix& mat, float fpoint);
+
+	~fill_style() {}
 	
 	/// Read the fill style from a stream
 	//
@@ -74,7 +131,12 @@ public:
 	/// Throw a ParserException if there's no enough bytes in the
 	/// currently opened tag for reading. See stream::ensureBytes()
 	///
-	void	read(stream* in, int tag_type, movie_definition* m);
+	void	read(stream* in, int tag_type, movie_definition* m,
+		fill_style *pOther = NULL);
+
+	/// Read the fill style from a stream, morph version.
+	void read_morph(stream* in, int tag_type, movie_definition* m,
+		fill_style *pOther);
 
 	/// \brief
 	/// Make a bitmap_info* corresponding to our gradient.
@@ -119,7 +181,13 @@ public:
 	
 	/// Returns the color stop value at a specified index
 	const gradient_record& get_color_stop(int index) const;
-	
+
+	/// Get and set the focal point for gradient focal fills.
+	/// This should be from -1.0 to 1.0, representing the left
+	/// and right edges of the rectangle.
+	float get_focal_point() const { return m_focal_point; }
+	void set_focal_point(float f) { m_focal_point = f; }
+
 #ifdef GNASH_USE_GC
 	/// Mark reachable resources (for the GC)
 	//
@@ -138,7 +206,7 @@ private:
 	/// @param ratio
 	///	Ratio is in the range [0, 255].
 	///
-	rgba sample_gradient(uint8_t ratio) const;
+	rgba sample_gradient(boost::uint8_t ratio) const;
 
 	friend class morph2_character_def;
 	friend class triangulating_render_handler;
@@ -147,6 +215,7 @@ private:
 	int	m_type;
 	rgba	m_color;
 	matrix	m_gradient_matrix;
+    float m_focal_point; // For focal fill gradients.
 	std::vector<gradient_record>	m_gradients;
 	boost::intrusive_ptr<gnash::bitmap_info>	m_gradient_bitmap_info;
 	boost::intrusive_ptr<bitmap_character_def>	m_bitmap_character;
