@@ -1,22 +1,20 @@
 // 
-//   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
+// the Free Software Foundation; either version 3 of the License, or
 // (at your option) any later version.
 // 
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU General Public License for more details.
+// 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-// 
-//
-
 // Stateful live Movie instance 
 
 
@@ -24,12 +22,15 @@
 #define GNASH_MOVIE_INSTANCE_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <vector>
+#include <set>
 
 #include "sprite_instance.h" // for inheritance
+#include "smart_ptr.h" // for composition
+#include "movie_definition.h" // for dtor visibility by smart ptr
 
 // Forward declarations
 namespace gnash {
@@ -47,16 +48,40 @@ class movie_instance : public sprite_instance
 
 public:
 
-	movie_instance(movie_def_impl* def,
-		movie_root* r, character* parent);
+	// We take a generic movie_definition to allow
+	// for subclasses for other then SWF movies
+	movie_instance(movie_definition* def, character* parent);
 
 	virtual ~movie_instance() {}
 
-	virtual void advance(float delta_time);
+	virtual void advance();
+
+	/// Handle a top-level movie on stage placement.
+	//
+	/// This method will just ensure first frame is loaded
+	/// and then call sprite_instance::stagePlacementCallback.
+	///
+	/// It's intended to be called by movie_root::setLevel().
+	///
+	void stagePlacementCallback();
+
+	/// Set a character in the dictionary as initialized, returning
+	/// true if not already initialized.
+	bool setCharacterInitialized(int cid)
+	{
+		return _initializedCharacters.insert(cid).second;
+	}
 
 private:
 
-	movie_def_impl* _def;
+	/// A map to track execution of init actions
+	//
+	/// Elements of this set are ids of characters
+	/// in our definition's CharacterDictionary.
+	///
+	std::set<int> _initializedCharacters;
+
+	boost::intrusive_ptr<movie_definition> _def;
 };
 
 
