@@ -1,6 +1,6 @@
-// BitmapMovieDefinition.cpp:  , for Gnash.
+// BitmapMovieDefinition.cpp:  Bitmap movie definition, for Gnash.
 // 
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 
@@ -50,7 +50,10 @@ BitmapMovieDefinition::getShapeDef()
 
 	// Add the bitmap fill style (fill style 0)
 
-	fill_style bmFill(_bitmap.get());
+	matrix mat;
+	//mat.concatenate_scale(1.0/20.0);
+	mat.set_scale(1.0/20.0, 1.0/20.0); // bitmap fills get matrix reversed
+	fill_style bmFill(_bitmap.get(), mat);
 	size_t fillLeft = _shapedef->add_fill_style(bmFill);
 
 	// Define a rectangle filled with the bitmap style
@@ -58,10 +61,10 @@ BitmapMovieDefinition::getShapeDef()
 	// We use one twip for each pixel in the image
 	// The character will be scaled * 20
 	// when placed in BitmapMovieInstance's DisplayList
-	float w = _framesize.width()/20;
-	float h = _framesize.height()/20;
+	float w = _framesize.width(); // /20;
+	float h = _framesize.height(); // /20;
 
-	log_msg(_("Creating a shape_definition wrapping a %g x %g bitmap"), w, h);
+	log_parse(_("Creating a shape_definition wrapping a %g x %g bitmap"), w, h);
 
 	path bmPath(w, h, fillLeft, 0, 0);
 	bmPath.drawLineTo(w, 0);
@@ -82,12 +85,12 @@ BitmapMovieDefinition::BitmapMovieDefinition(
 	:
 	_version(6),
 	// image::rgb size is in pixels
-	_framesize(0, 0, image->m_width*20, image->m_height*20),
+	_framesize(0, 0, image->width()*20, image->height()*20),
 	_framecount(1),
-	_playlist(_framecount),
 	_framerate(12),
 	_url(url),
-	_image(image)
+	_image(image),
+	_bytesTotal(_image->size())
 {
 	// Do not create shape_character_def now (why?)
 }
