@@ -1,6 +1,6 @@
 // edit_text_character_def.cpp:  Text tags, for Gnash.
 //
-//   Copyright (C) 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: edit_text_character_def.cpp,v 1.12 2007/07/11 16:52:47 strk Exp $ */
+/* $Id: edit_text_character_def.cpp,v 1.17 2008/02/11 16:32:52 bwy Exp $ */
 
 // Based on the public domain text.cpp of Thatcher Ulrich <tu@tulrich.com> 2003
 
@@ -36,34 +36,37 @@ class movie_definition;
 
 void
 edit_text_character_def::read(stream* in, int tag_type,
-		movie_definition* m)
+		movie_definition* /*m*/)
 {
-	assert(m != NULL);
+	//assert(m != NULL);
 	assert(tag_type == SWF::DEFINEEDITTEXT); // 37
 
 	m_rect.read(in);
 
 	in->align();
-	bool	has_text = in->read_uint(1) ? true : false;
-	m_word_wrap = in->read_uint(1) ? true : false;
-	m_multiline = in->read_uint(1) ? true : false;
-	m_password = in->read_uint(1) ? true : false;
-	m_readonly = in->read_uint(1) ? true : false;
-	bool	has_color = in->read_uint(1) ? true : false;
-	bool	has_max_length = in->read_uint(1) ? true : false;
-	bool	has_font = in->read_uint(1) ? true : false;
+	in->ensureBytes(1);
+	bool	has_text = in->read_bit();
+	m_word_wrap = in->read_bit();
+	m_multiline = in->read_bit();
+	m_password = in->read_bit();
+	m_readonly = in->read_bit(); 
+	bool	has_color = in->read_bit(); 
+	bool	has_max_length = in->read_bit(); 
+	bool	has_font = in->read_bit(); 
 
-	in->read_uint(1);	// reserved
-	m_auto_size = in->read_uint(1) ? true : false;
-	bool	has_layout = in->read_uint(1) ? true : false;
-	m_no_select = in->read_uint(1) ? true : false;
-	m_border = in->read_uint(1) ? true : false;
-	in->read_uint(1);	// reserved
-	m_html = in->read_uint(1) ? true : false;
-	m_use_outlines = in->read_uint(1) ? true : false;
+	in->ensureBytes(1);
+	in->read_bit();	// reserved
+	m_auto_size = in->read_bit(); 
+	bool	has_layout = in->read_bit(); 
+	m_no_select = in->read_bit(); 
+	m_border = in->read_bit(); 
+	in->read_bit(); // reserved
+	m_html = in->read_bit(); 
+	m_use_outlines = in->read_bit(); 
 
 	if (has_font)
 	{
+		in->ensureBytes(4);
 		m_font_id = in->read_u16();
 		m_text_height = in->read_u16();
 	}
@@ -75,16 +78,16 @@ edit_text_character_def::read(stream* in, int tag_type,
 
 	if (has_max_length)
 	{
+		in->ensureBytes(2);
 		m_max_length = in->read_u16();
 	}
 
 	if (has_layout)
 	{
-		m_alignment = (alignment) in->read_u8();
-		//m_left_margin = (float) in->read_u16();
-		m_left_margin = in->read_u16();
-		//m_right_margin = (float) in->read_u16();
-		m_right_margin = in->read_u16();
+		in->ensureBytes(9); //1 + 2 + 2 + 2 + 2
+		m_alignment = static_cast<alignment>(in->read_u8());
+		m_left_margin = in->read_u16(); // used to be cast to float
+		m_right_margin = in->read_u16(); // used to be cast to float
 		m_indent = in->read_s16();
 		m_leading = in->read_s16();
 	}
@@ -119,7 +122,7 @@ edit_text_character_def::get_font()
 		if (m_font == NULL)
 		{
 			// this is fine, the textfield would use a default device font
-			log_debug(_("text style with undefined font; font_id = %d; using a default font"), m_font_id);
+			//log_debug(_("text style with undefined font; font_id = %d; using a default font"), m_font_id);
 			m_font = fontlib::get_default_font().get();
 		}
 	}
