@@ -1,6 +1,6 @@
 // movie_def_impl.cpp:  Load in Movie definitions, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -18,12 +18,12 @@
 //
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "movie_def_impl.h"
 #include "movie_definition.h" // for inheritance
-#include "sprite_instance.h" // for create_instance()
+#include "sprite_instance.h" // for ??
 #include "tu_file.h"
 #include "zlib_adapter.h"
 #include "stream.h"
@@ -38,10 +38,11 @@
 #include "movie_root.h"
 #include "VM.h" // for assertions
 #include "GnashException.h" // for parser exception
-#include "execute_tag.h"
+#include "ControlTag.h"
 #include "sound_definition.h" // for sound_sample
 #include <boost/bind.hpp>
 
+#include <iomanip>
 #include <memory>
 #include <string>
 #include <unistd.h>
@@ -123,7 +124,7 @@ MovieLoader::start()
 {
 #ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
 	// don't start MovieLoader thread !
-	assert(0);
+	abort();
 #endif
 	// We have two sanity checks, started() and isSelfThread() which rely
 	// on boost::thread() returning before they are executed. Therefore,
@@ -139,46 +140,38 @@ MovieLoader::start()
 //
 // some utility stuff
 //
-// FIXME, this needs work now that log_msg always ends with a newline.
 
-void	dump_tag_bytes(stream* in)
-    // Log the contents of the current tag, in hex.
+/// Log the contents of the current tag, in hex to the output strream
+static void	dump_tag_bytes(stream* in, std::ostream& os)
 {
     static const int	ROW_BYTES = 16;
     char	row_buf[ROW_BYTES];
     int	row_count = 0;
 
-    while(in->get_position() < in->get_tag_end_position())
-        {
-            int	c = in->read_u8();
-            log_msg("%02X", c);
+    row_buf[ROW_BYTES-1] = '\0';
 
-            if (c < 32) c = '.';
-            if (c > 127) c = '.';
+    os << std::endl;
+    while(in->get_position() < in->get_tag_end_position())
+    {
+            int	c = in->read_u8();
+            os << std::hex << std::setw(2) << std::setfill('0') << c << " ";
+
+            if (c < 32 || c > 127 ) c = '.';
             row_buf[row_count] = c;
 
             row_count++;
             if (row_count >= ROW_BYTES)
-                {
-                    log_msg("    ");
-                    for (int i = 0; i < ROW_BYTES; i++)
-                        {
-                            log_msg("%c", row_buf[i]);
-                        }
-
-                    log_msg("\n");
+            {
+                    os << row_buf << std::endl;
                     row_count = 0;
-                }
-            else
-                {
-                    log_msg(" ");
-                }
-        }
-
-    if (row_count > 0)
-        {
-            log_msg("\n");
-        }
+            }
+   }
+   if ( row_count )
+   {
+            row_buf[row_count] = '\0';
+            while (row_count++ < ROW_BYTES ) os << "   ";
+            os << row_buf << std::endl;
+   }
 }
 
 
@@ -199,13 +192,10 @@ register_progress_callback(progress_callback progress_handle)
 // movie_def_impl
 //
 
-movie_def_impl::movie_def_impl(create_bitmaps_flag cbf,
-		create_font_shapes_flag cfs)
+movie_def_impl::movie_def_impl()
 	:
 	// FIXME: use a class-static TagLoadersTable for movie_def_impl
 	_tag_loaders(SWF::TagLoadersTable::getInstance()),
-	m_create_bitmaps(cbf),
-	m_create_font_shapes(cfs),
 	m_frame_rate(30.0f),
 	m_frame_count(0u),
 	m_version(0),
@@ -226,23 +216,16 @@ movie_def_impl::~movie_def_impl()
 	// Request cancelation of the loading thread
 	_loadingCanceled = true;
 
-    // Release our playlist data.
-    {for (size_t i = m_playlist.size() - 1; i != static_cast<size_t>(-1); i--) // Optimized
-        {
-            for (size_t j = m_playlist[i].size()-1; j != static_cast<size_t>(-1); j--)
-                {
-                    delete m_playlist[i][j];
-                }
-        }}
+	// Release frame tags
+	for (PlayListMap::iterator i=m_playlist.begin(), e=m_playlist.end(); i!=e; ++i)
+	{
+		PlayList& pl = i->second;
 
-    // Release init action data.
-    {for (size_t i = m_init_action_list.size() - 1; i != static_cast<size_t>(-1); i--) //Optimized
-        {
-            for (size_t j = m_init_action_list[i].size()-1; j != static_cast<size_t>(-1); j--)
-                {
-                    delete m_init_action_list[i][j];
+		for (PlayList::iterator j=pl.begin(), je=pl.end(); j!=je; ++j)
+		{
+                    delete *j;
                 }
-        }}
+        }
 
 	// It's supposed to be cleaned up in read()
 	// TODO: join with loader thread instead ?
@@ -328,6 +311,7 @@ void movie_def_impl::resolve_import(const std::string& source_url, movie_definit
 void movie_def_impl::add_character(int character_id, character_def* c)
 {
 	assert(c);
+	boost::mutex::scoped_lock lock(_dictionaryMutex);
 	_dictionary.add_character(character_id, c);
 }
 
@@ -343,6 +327,8 @@ movie_def_impl::get_character_def(int character_id)
         }
 #endif // not NDEBUG
 
+	boost::mutex::scoped_lock lock(_dictionaryMutex);
+
 	boost::intrusive_ptr<character_def> ch = _dictionary.get_character(character_id);
 #ifndef GNASH_USE_GC
 	assert(ch == NULL || ch->get_ref_count() > 1);
@@ -353,7 +339,7 @@ movie_def_impl::get_character_def(int character_id)
 void movie_def_impl::add_font(int font_id, font* f)
 {
     assert(f);
-    m_fonts.insert(make_pair(font_id, boost::intrusive_ptr<font>(f)));
+    m_fonts.insert(std::make_pair(font_id, boost::intrusive_ptr<font>(f)));
 }
 
 font* movie_def_impl::get_font(int font_id) const
@@ -386,9 +372,9 @@ movie_def_impl::add_bitmap_character_def(int character_id,
 		bitmap_character_def* ch)
 {
     assert(ch);
-    //log_msg(_("Add bitmap character %d"), character_id);
+    //log_debug(_("Add bitmap character %d"), character_id);
     //m_bitmap_characters.add(character_id, ch);
-    m_bitmap_characters.insert(make_pair(character_id, boost::intrusive_ptr<bitmap_character_def>(ch)));
+    m_bitmap_characters.insert(std::make_pair(character_id, boost::intrusive_ptr<bitmap_character_def>(ch)));
 
 	// we can *NOT* generate bitmap_info until
 	// a renderer is present
@@ -415,7 +401,7 @@ void movie_def_impl::add_sound_sample(int character_id, sound_sample* sam)
     log_parse(_("Add sound sample %d assigning id %d"),
 		character_id, sam->m_sound_handler_id);
     );
-    m_sound_samples.insert(make_pair(character_id,
+    m_sound_samples.insert(std::make_pair(character_id,
 			    boost::intrusive_ptr<sound_sample>(sam)));
 }
 
@@ -432,8 +418,8 @@ movie_def_impl::readHeader(std::auto_ptr<tu_file> in, const std::string& url)
 	if ( url == "" ) _url = "<anonymous>";
 	else _url = url;
 
-	uint32_t file_start_pos = _in->get_position();
-	uint32_t header = _in->read_le32();
+	boost::uint32_t file_start_pos = _in->get_position();
+	boost::uint32_t header = _in->read_le32();
 	m_file_length = _in->read_le32();
 	_swf_end_pos = file_start_pos + m_file_length;
 
@@ -489,7 +475,9 @@ movie_def_impl::readHeader(std::auto_ptr<tu_file> in, const std::string& url)
 		log_swferror("non-finite movie bounds");
 		);
 	}
-	m_frame_rate = _str->read_u16() / 256.0f;
+	m_frame_rate = _str->read_u16();
+	if ( ! m_frame_rate ) m_frame_rate = 65535;
+	else m_frame_rate /= 256.0f;
 	m_frame_count = _str->read_u16();
 
 	/* Markus: Probably this is better anyways */
@@ -497,11 +485,6 @@ movie_def_impl::readHeader(std::auto_ptr<tu_file> in, const std::string& url)
 	// TODO: This seems dangerous, check closely
 	if(m_frame_count == 0) m_frame_count++;
 
-	// Allocate 1 more then the expected slots
-	// for actions, to make handling malformed SWF easier.
-	m_playlist.resize(m_frame_count+1);
-	m_init_action_list.resize(m_frame_count+1);
-
 	IF_VERBOSE_PARSE(
 		m_frame_size.print();
 		log_parse(_("frame rate = %f, frames = " SIZET_FMT),
@@ -567,21 +550,21 @@ movie_def_impl::read(std::auto_ptr<tu_file> in, const std::string& url)
 bool
 movie_def_impl::ensure_frame_loaded(size_t framenum)
 {
+	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
+
 #ifndef LOAD_MOVIES_IN_A_SEPARATE_THREAD
 	return ( framenum <= _frames_loaded );
 #endif
 
-	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
-
 	if ( framenum <= _frames_loaded ) return true;
 
 	_waiting_for_frame = framenum;
-        //log_msg(_("Waiting for frame %u to be loaded"), framenum);
+        //log_debug(_("Waiting for frame %u to be loaded"), framenum);
 
 	// TODO: return false on timeout
 	_frame_reached_condition.wait(lock);
 
-        //log_msg(_("Condition reached (_frames_loaded=%u)"), _frames_loaded);
+        //log_debug(_("Condition reached (_frames_loaded=%u)"), _frames_loaded);
 
 	return ( framenum <= _frames_loaded );
 }
@@ -623,129 +606,10 @@ void movie_def_impl::get_owned_fonts(std::vector<font*>* fonts)
 }
 
 
-/* movie_def_impl */
-void movie_def_impl::generate_font_bitmaps()
-    // Generate bitmaps for our fonts, if necessary.
-{
-    // Collect list of fonts.
-    std::vector<font*>	fonts;
-    get_owned_fonts(&fonts);
-    fontlib::generate_font_bitmaps(fonts, this);
-}
-
-void
-movie_def_impl::output_cached_data(tu_file* out, const cache_options& options)
-{
-    // Write a little header.
-    char	header[5];
-    strcpy(header, "gscX");
-    header[3] = CACHE_FILE_VERSION;
-    compiler_assert(CACHE_FILE_VERSION < 256);
-
-    out->write_bytes(header, 4);
-
-    // Write font data.
-    std::vector<font*>	fonts;
-    get_owned_fonts(&fonts);
-    fontlib::output_cached_data(out, fonts, this, options);
-
-	// Write character data.
-	{
-
-	for ( CharacterDictionary::iterator
-		it = _dictionary.begin(), itEnd = _dictionary.end();
-		it != itEnd;
-		++it )
-	{
-		out->write_le16(it->first);
-		it->second->output_cached_data(out, options);
-	}
-
-#if 0
-	for (hash<int, boost::intrusive_ptr<character_def> >::iterator it = m_characters.begin();
-          it != m_characters.end();
-          ++it)
-        {
-            out->write_le16(it->first);
-            it->second->output_cached_data(out, options);
-        }
-#endif
-	}
-
-    out->write_le16((int16_t) -1);	// end of characters marker
-}
-
-
-void
-movie_def_impl::input_cached_data(tu_file* in)
-{
-    // Read the header & check version.
-    unsigned char	header[4];
-    in->read_bytes(header, 4);
-    if (header[0] != 'g' || header[1] != 's' || header[2] != 'c')
-        {
-            log_error(_("cache file does not have the correct format; skipping"));
-            return;
-        }
-    else if (header[3] != CACHE_FILE_VERSION)
-        {
-            log_error(
-                _("cached data is version %d, but we require version %d; skipping"),
-                int(header[3]), CACHE_FILE_VERSION);
-            return;
-        }
-
-    // Read the cached font data.
-    std::vector<font*>	fonts;
-    get_owned_fonts(&fonts);
-    fontlib::input_cached_data(in, fonts, this);
-
-    // Read the cached character data.
-    for (;;)
-        {
-            if (in->get_error() != TU_FILE_NO_ERROR)
-                {
-                    log_error(_("error reading cache file (characters); skipping"));
-                    return;
-                }
-            if (in->get_eof())
-                {
-                    log_error(_("unexpected eof reading cache file (characters); skipping"));
-                    return;
-                }
-
-            int16_t	id = in->read_le16();
-            if (id == (int16_t) -1) { break; }	// done
-
-            boost::intrusive_ptr<character_def> ch = _dictionary.get_character(id);
-            //m_characters.get(id, &ch);
-            if (ch != NULL)
-                {
-                    ch->input_cached_data(in);
-                }
-            else
-                {
-                    log_error(_("sync error in cache file (reading characters).  "
-                              "Skipping rest of cache data."));
-                    return;
-                }
-        }
-}
-
-sprite_instance*
-movie_def_impl::create_instance()
-{
-	return create_movie_instance();
-}
-
 movie_instance*
-movie_def_impl::create_movie_instance()
+movie_def_impl::create_movie_instance(character* parent)
 {
-
-	// @@ Shouldn't we return a movie_instance instead ?
-	// @@ and leave movie_root creation to the caller ..
-
-	return new movie_instance(this, NULL);
+	return new movie_instance(this, parent);
 }
 
 
@@ -759,7 +623,7 @@ CharacterDictionary::dump_chars() const
 	for ( const_iterator it=begin(), endIt=end();
 		it != endIt; ++it )
 	{
-		log_msg(_("Character %d @ %p"), it->first, static_cast<void*>(it->second.get()));
+		log_debug(_("Character %d @ %p"), it->first, static_cast<void*>(it->second.get()));
 		//character_def* cdef = it->second;
 	}
 }
@@ -782,7 +646,7 @@ CharacterDictionary::get_character(int id)
 void
 CharacterDictionary::add_character(int id, boost::intrusive_ptr<character_def> c)
 {
-	//log_msg(_("CharacterDictionary: add char %d"), id);
+	//log_debug(_("CharacterDictionary: add char %d"), id);
 	_map[id] = c;
 	//dump_chars();
 }
@@ -805,11 +669,11 @@ movie_def_impl::load_next_frame_chunk()
 	nextframe += FRAMELOAD_CHUNK; // load in chunks of 10 frames
 	if ( nextframe > framecount ) nextframe = framecount;
 #endif
-	//log_msg(_("Framecount: %u, Lastloaded: %u"), framecount, lastloaded);
+	//log_debug(_("Framecount: %u, Lastloaded: %u"), framecount, lastloaded);
 	if ( nextframe <= framecount )
 	{
 #ifdef DEBUG_FRAMES_LOAD // debugging
-		log_msg(_("Ensure load of frame %u/%u (last loaded is: %u)"),
+		log_debug(_("Ensure load of frame %u/%u (last loaded is: %u)"),
 			nextframe, framecount, lastloaded);
 #endif
 		if ( ! ensure_frame_loaded(nextframe) )
@@ -817,13 +681,13 @@ movie_def_impl::load_next_frame_chunk()
 			log_error(_("Could not advance to frame " SIZET_FMT),
 				nextframe);
 			// these kind of errors should be handled by callers
-			assert(0);
+			abort();
 		}
 	}
 #ifdef DEBUG_FRAMES_LOAD
 	else
 	{
-		log_msg(_("No more frames to load. Framecount: %u, Lastloaded: %u, next to load: %u"), framecount, lastloaded, nextframe);
+		log_debug(_("No more frames to load. Framecount: %u, Lastloaded: %u, next to load: %u"), framecount, lastloaded, nextframe);
 	}
 #endif
 }
@@ -846,7 +710,7 @@ movie_def_impl::read_all_swf()
 	try {
 
 	//size_t it=0;
-	while ( (uint32_t) str.get_position() < _swf_end_pos )
+	while ( (boost::uint32_t) str.get_position() < _swf_end_pos )
 	{
 		if ( _loadingCanceled )
 		{
@@ -854,7 +718,7 @@ movie_def_impl::read_all_swf()
 			return;
 		}
 
-		//log_msg(_("Loading thread iteration %u"), it++);
+		//log_debug(_("Loading thread iteration %u"), it++);
 
 		SWF::tag_type tag_type = str.open_tag();
 
@@ -862,7 +726,7 @@ parse_tag:
 
 		if (s_progress_function != NULL)
                 {
-			s_progress_function((uint32_t)str.get_position(),
+			s_progress_function((boost::uint32_t)str.get_position(),
 				_swf_end_pos);
                 }
 
@@ -888,11 +752,11 @@ parse_tag:
 			// show frame tag -- advance to the next frame.
 
 			IF_VERBOSE_PARSE(
-				log_parse("  show_frame\n");
+				log_parse("  show_frame");
 			);
 
-			incrementLoadedFrames();
-			if ( _frames_loaded == m_frame_count )
+			size_t floaded = incrementLoadedFrames();
+			if ( floaded == m_frame_count )
 			{
 				str.close_tag();
 				tag_type = str.open_tag();
@@ -921,7 +785,9 @@ parse_tag:
 			log_error(_("*** no tag loader for type %d (movie)"),
 				tag_type);
 			IF_VERBOSE_PARSE(
-				dump_tag_bytes(&str);
+				std::stringstream ss;
+				dump_tag_bytes(&str, ss);
+				log_error("tag dump follows: %s", ss.str().c_str());
 			);
 		}
 
@@ -940,24 +806,23 @@ parse_tag:
 		log_error(_("Parsing exception: %s"), e.what());
 	}
 
-	if ( ! m_playlist[_frames_loaded].empty() || ! m_init_action_list[_frames_loaded].empty() )
+	size_t floaded = get_loading_frame();
+	if ( ! m_playlist[floaded].empty() )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
-		log_swferror(_(SIZET_FMT " action blocks and " SIZET_FMT " init action blocks are NOT followed by"
+		log_swferror(_(SIZET_FMT " control tags are NOT followed by"
 			" a SHOWFRAME tag"),
-			m_playlist[_frames_loaded].size(),
-			m_init_action_list[_frames_loaded].size());
+			m_playlist[floaded].size());
 		);
 	}
 
-	if ( m_frame_count > _frames_loaded )
+	if ( m_frame_count > floaded )
 	{
 		IF_VERBOSE_MALFORMED_SWF(
 		log_swferror(_(SIZET_FMT " frames advertised in header, but only " SIZET_FMT " SHOWFRAME tags "
-			"found in stream. Updating total frames count"), m_frame_count, _frames_loaded);
+			"found in stream. Updating total frames count"), m_frame_count, floaded);
 		);
-		boost::mutex::scoped_lock lock(_frames_loaded_mutex);
-		m_frame_count = _frames_loaded;
+		m_frame_count = floaded;
 		// Notify any thread waiting on frame reached condition
 		_frame_reached_condition.notify_all();
 	}
@@ -970,7 +835,7 @@ movie_def_impl::get_loading_frame() const
 	return _frames_loaded;
 }
 
-void
+size_t
 movie_def_impl::incrementLoadedFrames()
 {
 	boost::mutex::scoped_lock lock(_frames_loaded_mutex);
@@ -991,12 +856,11 @@ movie_def_impl::incrementLoadedFrames()
 				get_url().c_str(), _frames_loaded,
 				m_frame_count);
 		);
-		m_playlist.resize(_frames_loaded+1);
-		m_init_action_list.resize(_frames_loaded+1);
+		//m_playlist.resize(_frames_loaded+1);
 	}
 
 #ifdef DEBUG_FRAMES_LOAD
-	log_msg(_("Loaded frame %u/%u"),
+	log_debug(_("Loaded frame %u/%u"),
 		_frames_loaded, m_frame_count);
 #endif
 
@@ -1008,15 +872,18 @@ movie_def_impl::incrementLoadedFrames()
 		// See: http://boost.org/doc/html/condition.html
 		_frame_reached_condition.notify_all();
 	}
+
+	return _frames_loaded;
 }
 
 void
 movie_def_impl::export_resource(const std::string& symbol, resource* res)
 {
-	// FIXME: m_exports access should be protected by a mutex
+	// _exportedResources access should be protected by a mutex
+	boost::mutex::scoped_lock lock(_exportedResourcesMutex);
 
 	// SWF sometimes exports the same thing more than once!
-	m_exports[symbol] = res;
+	_exportedResources[symbol] = res;
 }
 
 
@@ -1026,7 +893,7 @@ movie_def_impl::get_exported_resource(const std::string& symbol)
 	boost::intrusive_ptr<resource> res;
 
 #ifdef DEBUG_EXPORTS
-	log_msg(_("get_exported_resource called, frame count=%u"), m_frame_count);
+	log_debug(_("get_exported_resource called, frame count=%u"), m_frame_count);
 #endif
 
 	// Don't call get_exported_resource() from this movie loader
@@ -1054,17 +921,22 @@ movie_def_impl::get_exported_resource(const std::string& symbol)
 	size_t loading_frame = (size_t)-1; // used to keep track of advancements
 	for (;;)
 	{
-		// FIXME: make m_exports access thread-safe
-        ExportMap::iterator it = m_exports.find(symbol);
-        if ( it != m_exports.end() ) return it->second;
+		// _exportedResources access is thread-safe
+		{
+			boost::mutex::scoped_lock lock(_exportedResourcesMutex);
+			ExportMap::iterator it = _exportedResources.find(symbol);
+			if ( it != _exportedResources.end() ) return it->second;
+		}
 
 		size_t new_loading_frame = get_loading_frame();
 
 		if ( new_loading_frame != loading_frame )
 		{
-			log_msg(_("frame load advancement (from "
+#ifdef DEBUG_EXPORTS
+			log_debug(_("looking for exported resource: frame load advancement (from "
 				SIZET_FMT " to " SIZET_FMT ")"),
 				loading_frame, new_loading_frame);
+#endif
 			loading_frame = new_loading_frame;
 			timeout = def_timeout;
 		}
@@ -1086,22 +958,32 @@ movie_def_impl::get_exported_resource(const std::string& symbol)
 				return res;
 			}
 
-			log_error(_("no frame progress at iteration %lu"), timeout);
+#ifdef DEBUG_EXPORTS
+			log_debug(_("No frame progress at iteration %lu of get_exported_resource(%s)"),
+				timeout, symbol.c_str());
+#endif
 
 			continue; // not worth checking
 		}
 
 		if ( loading_frame >= m_frame_count )
 		{
-			log_error(_("At end of stream, still no '%s' symbol found "
-				"in m_exports (" SIZET_FMT " entries in it, "
-				"follow)"), symbol.c_str(), m_exports.size());
+#ifdef DEBUG_EXPORTS
+			boost::mutex::scoped_lock lock(_exportedResourcesMutex);
+			log_debug(_("At end of stream, still no '%s' symbol found "
+				"in _exportedResources (" SIZET_FMT " entries in it, "
+				"follow)"), symbol.c_str(), _exportedResources.size());
+			for (ExportMap::const_iterator it=_exportedResources.begin(); it!=_exportedResources.end(); ++it)
+			{
+				log_debug(" symbol %s (%s)", it->first.c_str(), typeName(*(it->second)).c_str());
+			}
+#endif
 			return res;
 		}
 
 #ifdef DEBUG_EXPORTS
-		log_msg(_("We haven't finished loading (loading frame %u), "
-			"and m_exports.get returned no entries, "
+		log_debug(_("We haven't finished loading (loading frame %u), "
+			"and _exportedResources.get returned no entries, "
 			"sleeping a bit and trying again"),
 			get_loading_frame());
 #endif
@@ -1115,16 +997,18 @@ movie_def_impl::get_exported_resource(const std::string& symbol)
 void
 movie_def_impl::add_frame_name(const std::string& n)
 {
-	//log_msg(_("labelframe: frame %d, name %s"), _frames_loaded, name);
-	assert(_frames_loaded < m_frame_count);
-    m_named_frames[n] = _frames_loaded;
+	boost::mutex::scoped_lock lock1(_namedFramesMutex);
+	boost::mutex::scoped_lock lock2(_frames_loaded_mutex);
+	//log_debug(_("labelframe: frame %d, name %s"), _frames_loaded, n.c_str());
+	_namedFrames[n] = _frames_loaded;
 }
 
 bool
 movie_def_impl::get_labeled_frame(const std::string& label, size_t& frame_number)
 {
-    NamedFrameMap::iterator it = m_named_frames.find(label);
-    if ( it == m_named_frames.end() ) return false;
+    boost::mutex::scoped_lock lock(_namedFramesMutex);
+    NamedFrameMap::iterator it = _namedFrames.find(label);
+    if ( it == _namedFrames.end() ) return false;
     frame_number = it->second;
     return true;
 }
@@ -1153,9 +1037,13 @@ movie_def_impl::markReachableResources() const
 		i->second->setReachable();
 	}
 
-	for (ExportMap::const_iterator i=m_exports.begin(), e=m_exports.end(); i!=e; ++i)
+	// TODO: turn this into a markExportedResources()
 	{
-		i->second->setReachable();
+		boost::mutex::scoped_lock lock(_exportedResourcesMutex);
+		for (ExportMap::const_iterator i=_exportedResources.begin(), e=_exportedResources.end(); i!=e; ++i)
+		{
+			i->second->setReachable();
+		}
 	}
 
 	for (ImportVect::const_iterator i=m_import_source_movies.begin(), e=m_import_source_movies.end(); i!=e; ++i)
@@ -1163,6 +1051,7 @@ movie_def_impl::markReachableResources() const
 		(*i)->setReachable();
 	}
 
+	boost::mutex::scoped_lock lock(_dictionaryMutex);
 	_dictionary.markReachableResources();
 
 }
