@@ -1,6 +1,6 @@
 // sprite_definition.h:  Holds immutable data for a sprite, for Gnash.
 //
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #define GNASH_SPRITE_DEFINITION_H
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include <vector>
@@ -86,9 +86,9 @@ public:
 	//
 	// TODO: implement in base class ?
 	//
-	void addTimelineDepth(int depth, int ratio)
+	void addTimelineDepth(int depth)
 	{
-		_timeline.addDepth(depth, ratio);
+		_timeline.addDepth(depth);
 	}
 
 	// See dox in base class
@@ -104,7 +104,7 @@ public:
 	//
 	// TODO: implement in base class ?
 	//
-	void getTimelineDepths(size_t frameno, std::map<int, int>& depths)
+	void getTimelineDepths(size_t frameno, std::vector<int>& depths)
 	{
 		_timeline.getFrameDepths(frameno, depths);
 	}
@@ -122,8 +122,10 @@ private:
 	/// (the definition read from SWF stream)
 	movie_definition* m_movie_def;
 
+	typedef std::map<size_t, PlayList> PlayListMap;
+
 	/// movie control events for each frame.
-	std::vector<PlayList> m_playlist;
+	PlayListMap m_playlist;
 
 	// stores 0-based frame #'s
 	typedef std::map<std::string, size_t> NamedFrameMap;
@@ -131,6 +133,7 @@ private:
 
 	size_t m_frame_count;
 
+	// Number of frames completely parsed 
 	size_t m_loading_frame;
 
 	// overloads from movie_definition
@@ -167,7 +170,7 @@ private:
 
 	const rect& get_frame_size() const
 	{
-		assert(0);
+		abort();
 		static const rect unused;
 		return unused;
 	}
@@ -238,13 +241,11 @@ private:
 	}
 
 
-	/// Overridden just for complaining  about malformed SWF
-	virtual void export_resource(const std::string& /*symbol*/,
-			resource* /*res*/)
+	/// Delegate call to associated root movie
+	virtual void export_resource(const std::string& sym,
+			resource* res)
 	{
-		IF_VERBOSE_MALFORMED_SWF (
-		log_swferror(_("Can't export from sprite"));
-		);
+		m_movie_def->export_resource(sym, res);
 	}
 
 	/// Delegate call to associated root movie
@@ -262,24 +263,10 @@ private:
 	    return m_movie_def->get_character_def(id);
 	}
 
-	/// Overridden just for complaining  about malformed SWF
-	//
-	/// Calls to this function should only be made when
-	/// an invalid SWF is being read, as it would mean
-	/// that a Definition tag is been found as part of
-	/// a Sprite definition
-	///
-	virtual void add_character(int /*id*/, character_def* /*ch*/)
+	/// Delegate call to associated root movie
+	virtual void add_character(int id, character_def* ch)
 	{
-		IF_VERBOSE_MALFORMED_SWF (
-		log_swferror(_("add_character tag appears in sprite tags"));
-		);
-	}
-
-
-	virtual sprite_instance* create_instance()
-	{
-	    return NULL;
+		m_movie_def->add_character(id, ch);
 	}
 
 	// Create a (mutable) instance of our definition.  The
@@ -292,21 +279,11 @@ private:
 
 
 	// See dox in movie_definition.h
-	virtual void	add_execute_tag(execute_tag* c)
+	virtual void	addControlTag(ControlTag* c)
 	{
 		m_playlist[m_loading_frame].push_back(c);
 	}
 
-	/// Overridden just for complaining  about malformed SWF
-	//
-	/// Sprite def's should not have do_init_action tags in them!  (@@ correct?)
-	virtual void	add_init_action(execute_tag* /*c*/)
-	{
-		IF_VERBOSE_MALFORMED_SWF (
-		log_swferror(_("sprite_definition::add_init_action called!  Ignored"));
-		);
-	}
-
 	// See dox in movie_definition.h
 	virtual void	add_frame_name(const std::string& name);
 
@@ -314,16 +291,16 @@ private:
 	bool get_labeled_frame(const std::string& label, size_t& frame_number);
 
 	/// frame_number is 0-based
-	const PlayList& get_playlist(size_t frame_number) const
+	const PlayList* getPlaylist(size_t frame_number) const
 	{
-		return m_playlist[frame_number];
-	}
+		// Don't access playlist of a frame which has not been
+		// completely parsed yet.
+		assert(frame_number < m_loading_frame);
 
-	// Sprites do not have init actions in their
-	// playlists!  Only the root movie
-	// (movie_def_impl) does (@@ correct?)
-	// .. no need to override as returning NULL is the default behaviour ..
-	//virtual const PlayList* get_init_actions(size_t /*frame_number*/)
+		PlayListMap::const_iterator it = m_playlist.find(frame_number);
+		if ( it == m_playlist.end() ) return NULL;
+		else return &(it->second);
+	}
 
 	virtual const std::string& get_url() const
 	{
@@ -339,12 +316,12 @@ private:
 		// TODO: return false on timeout
 		while ( m_loading_frame < framenum )
 		{
-			log_msg(_("sprite_definition: "
+			log_debug(_("sprite_definition: "
 				"loading of frame " SIZET_FMT " requested "
 				"(we are at " SIZET_FMT "/" SIZET_FMT ")"),
 				framenum, m_loading_frame, m_frame_count);
-			// Could this ever happen ?
-			assert(0);
+			// Could this ever happen ? YES ! See tuner_7_6_0_0_pandora.swf
+			return false;
 		}
 		return true;
 	}
@@ -367,7 +344,7 @@ private:
     // get_bound() is currently only used by generic_character which normally
     // is used only shape character definitions. See character_def.h to learn
     // why it is virtual anyway.
-    assert(0); // should not be called
+    abort(); // should not be called
 		static rect unused;
 		return unused;
   }
