@@ -5,7 +5,7 @@
 
 // Quadratic bezier outline shapes, the basis for most SWF rendering.
 
-/* $Id: shape.h,v 1.23 2007/04/24 10:01:46 strk Exp $ */
+/* $Id: shape.h,v 1.36 2007/12/01 15:40:59 strk Exp $ */
 
 #ifndef GNASH_SHAPE_H
 #define GNASH_SHAPE_H
@@ -13,12 +13,12 @@
 #include "tu_config.h"
 #include "styles.h"
 
+#include <vector> // for path composition
+
 
 // Forward declarations
 namespace gnash {
-	namespace tesselate {
-		class tesselating_shape;
-	}
+	class rect; // for path::expandBounds
 }
 
 namespace gnash {
@@ -29,21 +29,71 @@ namespace gnash {
 	class edge
 	{
 	public:
-		edge();
-		edge(float cx, float cy, float ax, float ay);
-		void	tesselate_curve() const;
-		bool	is_straight() const;
+		edge()
+			:
+			cp(),
+			ap()
+		{
+		}
+
+		edge(float cx, float cy, float ax, float ay)
+			:
+    			cp(cx, cy),
+			ap(ax, ay)
+		{
+		}
+
+		edge(point ncp, point nap)
+			:
+			cp(ncp),
+			ap(nap)
+		{
+		}
+
+		bool isStraight() const
+		{
+			return cp == ap;
+		}
+
+		bool is_straight() const { return isStraight(); }
+		
+		/// Transform the edge according to the given matrix.
+		void transform(const matrix& mat);
 
 		/// Return squared distance between point pt and segment A-B
 		static float squareDistancePtSeg(const point& pt, const point& A, const point& B);
 
 		/// Return distance between point pt and segment A-B
 		static float distancePtSeg(const point& pt, const point& A, const point& B);
+
+		/// Find point of the quadratic curve defined by points A,C,B
+		//
+		/// @param A The first point
+		/// @param C The second point (control point)
+		/// @param B The third point (anchor point)
+		/// @param ret The point to write result into
+		/// @param t the step factor between 0 and 1
+		///
+		static point pointOnCurve(const point& A, const point& C, const point& B, float t);
+
+		/// Return square distance between point pt and the point on curve found by
+		/// applying the T parameter to the quadratic bezier curve function
+		//
+		/// @param A The first point of the bezier curve
+		/// @param C The second point of the bezier curve (control point)
+		/// @param B The third point of the bezier curve (anchor point)
+		/// @param p The point we want to compute distance from 
+		/// @param t the step factor between 0 and 1
+		///
+		static float squareDistancePtCurve(const point& A, const point& C, const point& B, const point& p, float t)
+		{
+			return p.squareDistance( pointOnCurve(A, C, B, t) );
+		}
 		
 	//private:
 		// *quadratic* bezier: point = p0 * t^2 + p1 * 2t(1-t) + p2 * (1-t)^2
-		float	m_cx, m_cy;		// "control" point
-		float	m_ax, m_ay;		// "anchor" point
+		point cp; // "control" point
+		point ap; // "anchor" point
 	};
 
 
@@ -53,7 +103,12 @@ namespace gnash {
 	class path
 	{
 	public:
-		path();
+		/// Default constructor
+		//
+		/// @param newShape
+		///	True if this path starts a new subshape
+		///
+		path(bool newShape=false);
 
 		/// Initialize a path 
 		//
@@ -75,10 +130,13 @@ namespace gnash {
 		///	Line style index for right fill (1-based).
 		///	Zero means NO style.
 		///
+		/// @param newShape
+		///	True if this path starts a new subshape
 		///
-		path(float ax, float ay, int fill0, int fill1, int line);
+		///
+		path(float ax, float ay, int fill0, int fill1, int line, bool newShape=false);
 
-		/// Re-initialize a path 
+		/// Re-initialize a path, maintaining the "new shape" flag untouched
 		//
 		/// @param ax
 		///	X coordinate of path origin in TWIPS
@@ -97,6 +155,12 @@ namespace gnash {
 		///
 		void	reset(float ax, float ay, int fill0, int fill1, int line);
 
+		/// Set this path as the start of a new (sub)shape
+		void setNewShape() { m_new_shape=true; }
+
+		/// Return true if this path starts a new (sub)shape
+		bool getNewShape() const { return m_new_shape; }
+
 		/// Return true if this path contains no edges
 		bool	is_empty() const;
 
@@ -106,25 +170,22 @@ namespace gnash {
 			return is_empty();
 		}
 
-		/// Point-in-shape test. 
+		/// Ray crossing count. (OBSOLETE)
 		//
-		/// Return true if the query point is on the filled
-		/// interior of this shape.
+		/// Update ray crossing for the given query point using
+		/// edges in this path. Was used to detect point in shape,
+    /// but now we're using a different method.
 		///
-		/// Note that boundaries are NOT checked (a point of the boundary
-		/// will return false here).
-		/// Use withinSquareDistance() for boundary checking.
+		/// @param ray_crossings
+		///	Number of crossings, updated by this method.
 		///
-		/// Return false if this path has no associated fill.
+		/// @param x
+		///     X ordinate of the query point, in local coordinate space.
 		///
-		/// WARNING: often a filled shape is composed by multiple
-		///          paths. we probably fail in those cases.
+		/// @param y
+		///     Y ordinate of the query point, in local coordinate space.
 		///
-		///
-		bool	point_test(float x, float y) const;
-
-		/// Push the path into the tesselator.
-		void	tesselate() const;
+		void ray_crossing(int& ray_crossings, float x, float y) const;
 
 		/// @{ Primitives for the Drawing API
 		///
@@ -250,13 +311,34 @@ namespace gnash {
 		/// Close this path with a straight line, if not already closed
 		void close();
 
-		/// return true if the given point is withing the given squared distance
+		/// \brief
+		/// Return true if the given point is withing the given squared distance
+		/// from this path edges.
 		//
 		/// NOTE: if the path is empty, false is returned.
 		///
 		bool withinSquareDistance(const point& p, float dist) const;
 
 
+		/// Expand given rect to include bounds of this path
+		//
+		/// @param r
+		///	The rectangle to expand with our own bounds
+		///
+		/// @param thickness
+		///	The thickess of our lines, half the thickness will
+		///	be added in all directions
+		///
+		void expandBounds(rect& r, unsigned int thickness) const;
+
+		bool isNewShape() const
+		{
+			return m_new_shape;
+		}
+		
+		/// Transform all path coordinates according to the given matrix.
+		void transform(const matrix& mat);
+
 	//private:
 
 		/// Left fill style index (1-based)
@@ -268,78 +350,20 @@ namespace gnash {
 		/// Line style index (1-based)
 	        unsigned m_line;
 
-		/// Path/shape origin
-		float	m_ax, m_ay;
+		/// Path/shape origin 
+		point ap; 
 
 		/// Edges forming the path
 		std::vector<edge> m_edges;
 
-		/// ?
+		/// This flag is set when the path is the first one of a new "sub-shape".
+		/// All paths with a higher index in the list belong to the same 
+		/// shape unless they have m_new_shape==true on their own.
+		/// Sub-shapes affect the order in which outlines and shapes are rendered.
 		bool m_new_shape;
 
 	};
 
-	/// For holding a pre-tesselated shape.
-	class mesh
-	{
-	public:
-		mesh();
-
-		void	set_tri_strip(const point pts[], int count);
-
-
-		void	output_cached_data(tu_file* out);
-		void	input_cached_data(tu_file* in);
-	private:
-	  friend class triangulating_render_handler; 
-		std::vector<int16_t>	m_triangle_strip;
-	};
-
-
-	/// For holding a line-strip (i.e. polyline).
-	class line_strip
-	{
-	public:
-		line_strip();
-		line_strip(int style, const point coords[], int coord_count);
-
-		int	get_style() const { return m_style; }
-		void	output_cached_data(tu_file* out);
-		void	input_cached_data(tu_file* in);
-	private:
-    friend class triangulating_render_handler; 
-		int	m_style;
-		std::vector<int16_t>	m_coords;
-	};
-
-
-	/// A whole shape, tesselated to a certain error tolerance.
-	class DSOEXPORT mesh_set
-	{
-	public:
-		mesh_set();
-		mesh_set(const tesselate::tesselating_shape* sh,
-			 float error_tolerance);
-
-//		int	get_last_frame_rendered() const;
-//		void	set_last_frame_rendered(int frame_counter);
-		float	get_error_tolerance() const { return m_error_tolerance; }
-
-		void	set_tri_strip(int style, const point pts[], int count);
-		void	add_line_strip(int style, const point coords[], int coord_count);
-
-		void	output_cached_data(tu_file* out);
-		void	input_cached_data(tu_file* in);
-
-	private:
-    friend class triangulating_render_handler; 
-//		int	m_last_frame_rendered;	// @@ Hm, we shouldn't spontaneously drop cached data I don't think...
-		float	m_error_tolerance;
-		std::vector<mesh>	m_meshes;	// One mesh per style.
-		std::vector<line_strip>	m_line_strips;
-	};
-
-
 }	// end namespace gnash
 
 
