@@ -1,6 +1,6 @@
 // RemoveObjectTag.cpp: RemoveObject* tag for Gnash.
 //
-//   Copyright (C) 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 
-/* $Id: RemoveObjectTag.cpp,v 1.4 2007/07/01 10:54:35 bjacques Exp $ */
+/* $Id: RemoveObjectTag.cpp,v 1.7 2008/01/21 20:56:02 rsavoye Exp $ */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "RemoveObjectTag.h"
@@ -63,15 +63,24 @@ RemoveObjectTag::loader(stream* in, tag_type tag, movie_definition* m)
     std::auto_ptr<RemoveObjectTag> t ( new RemoveObjectTag );
     t->read(in, tag);
 
+    int depth = t->getDepth();
+
     IF_VERBOSE_PARSE
     (
-	log_parse(_("  remove_object_2(%d)"), t->getDepth());
+	log_parse(_("  remove_object_2(%d)"), depth);
     );
 
-    m->removeTimelineDepth(t->getDepth());
+    if ( depth < 0 && depth >= character::staticDepthOffset )
+    {
+        m->removeTimelineDepth(depth);
+    }
+    else
+    {
+	log_debug("RemoveObjectTag depth %d is out of static depth zone. Won't unregister its TimelineDepth.", depth);
+    }
 
     // Ownership transferred to movie_definition
-    m->add_execute_tag(t.release());
+    m->addControlTag(t.release());
 }
 
 } // namespace gnash::SWF
