@@ -1,5 +1,5 @@
 // 
-//   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+//   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 // 
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "gnashconfig.h"
 #endif
 
 #include "log.h"
@@ -29,6 +29,8 @@
 #include "ActionExec.h" // for operator()
 #include "VM.h" // for storing _global in a local register
 #include "builtin_function.h" // for Function constructor
+#include "Object.h" // for getObjectInterface
+#include "namedStrings.h"
 
 #include <typeinfo>
 #include <iostream>
@@ -38,7 +40,6 @@ using namespace std;
 
 namespace gnash {
 
-
 swf_function::~swf_function()
 {
 #ifndef GNASH_USE_GC
@@ -50,7 +51,7 @@ swf_function::swf_function(const action_buffer* ab,
 			as_environment* env,
 			size_t start, const ScopeStack& scopeStack)
 	:
-	as_function(NULL),
+	as_function(new as_object(getObjectInterface())),
 	//ctor(0),
 	m_action_buffer(ab),
 	m_env(env),
@@ -67,74 +68,6 @@ swf_function::swf_function(const action_buffer* ab,
 	init_member("constructor", as_value(as_function::getFunctionConstructor().get()));
 }
 
-/*private static*/
-boost::intrusive_ptr<as_object>
-swf_function::getSuper(as_object& obj)
-{ 
-//#define GNASH_DEBUG_GETSUPER
-	// Super class prototype is : obj.__proto__.__constructor__.prototype 
-	boost::intrusive_ptr<as_object> proto = obj.get_prototype();
-	if ( ! proto )
-	{
-#ifdef GNASH_DEBUG_GETSUPER
-		log_msg("Object %p doesn't have a __proto__", &obj);
-#endif
-		return NULL;
-	}
-
-	// TODO: add a getConstructor() method to as_object
-	//       returning an as_function ?
-	//
-	as_value ctor;
-	bool ret = proto->get_member("__constructor__", &ctor);
-	if ( ! ret )
-	{
-#ifdef GNASH_DEBUG_GETSUPER
-		log_msg("Object.__proto__ %p doesn't have a __constructor__", (void*)proto.get());
-#endif
-		return NULL;
-	}
-
-	// TODO: if we cast ctor to an as_function and call getPrototype on it,
-	// 	 it is possible that the returned object is NOT the current
-	// 	 'prototype' member, as as_function caches it ?
-	//
-	boost::intrusive_ptr<as_object> ctor_obj = ctor.to_object();
-	if ( ! ctor_obj )
-	{
-#ifdef GNASH_DEBUG_GETSUPER
-		log_msg("Object.__proto__.__constructor__ doesn't cast to an object");
-#endif
-		return NULL;
-	}
-
-#ifdef GNASH_DEBUG_GETSUPER
-	log_msg("ctor_obj is %p", ctor_obj.get());
-#endif
-
-	as_value ctor_proto;
-	ret = ctor_obj->get_member("prototype", &ctor_proto);
-	if ( ! ret )
-	{
-#ifdef GNASH_DEBUG_GETSUPER
-		log_msg("Object.__proto__.constructor %p doesn't have a prototype", ctor_obj.get());
-#endif
-		return NULL;
-	}
-
-	boost::intrusive_ptr<as_object> super = ctor_proto.to_object();
-	if ( ! super )
-	{
-#ifdef GNASH_DEBUG_GETSUPER
-		log_msg("Object.__proto__.constructor.prototype doesn't cast to an object");
-#endif
-		return NULL;
-	}
-
-	return super; // FIXME: return the intrusive_ptr directly !!
-
-}
-
 /*private static*/
 as_array_object* 
 swf_function::getArguments(swf_function& callee, const fn_call& fn)
@@ -152,7 +85,7 @@ swf_function::getArguments(swf_function& callee, const fn_call& fn)
 	{
 		arguments->push(fn.arg(i));
 	}
-	arguments->set_member("callee", &callee);
+	arguments->set_member(NSV::PROP_CALLEE, &callee);
 
 	return arguments;
 
@@ -162,36 +95,51 @@ swf_function::getArguments(swf_function& callee, const fn_call& fn)
 as_value
 swf_function::operator()(const fn_call& fn)
 {
+	// Set up local stack frame, for parameters and locals.
+	as_environment::FrameGuard guard(this);
 
 	as_environment*	our_env = m_env;
 	assert(our_env);
-	if (our_env == NULL)
+	// if(our_env->get_original_target()->isDestroyed())
+    if (our_env == NULL)
 	{
 		our_env = &fn.env();
 	}
+	//our_env = &fn.env(); // I think this should be it...
 	assert(our_env);
 
 #if 0
-	log_msg("swf_function() stack:\n"); fn.env().dump_stack();
-	log_msg("  first_arg_bottom_index: %d\n", fn.first_arg_bottom_index);
+	log_debug("swf_function() stack:\n"); fn.env().dump_stack();
+	log_debug("  first_arg_bottom_index: %d\n", fn.first_arg_bottom_index);
 #endif
-
-	// Set up local stack frame, for parameters and locals.
-	our_env->pushCallFrame(this);
-
-	// Some features are version-dependant
+	// Some features are version-dependant.
 	unsigned swfversion = VM::get().getSWFVersion();
+	assert(fn.this_ptr);
+	as_object *super = NULL;
+	if (swfversion > 5)
+	{
+		super = fn.this_ptr->get_super();
+	}
 
 	if (m_is_function2 == false)
 	{
 		// Conventional function.
 
 		// Push the arguments onto the local frame.
-		int	args_to_pass = imin(fn.nargs, m_args.size());
-		for (int i = 0; i < args_to_pass; i++)
+		for (size_t i=0, n=m_args.size(); i<n; ++i)
 		{
 			assert(m_args[i].m_register == 0);
-			our_env->add_local(m_args[i].m_name, fn.arg(i));
+			if ( i < fn.nargs )
+			{
+				our_env->add_local(m_args[i].m_name, fn.arg(i));
+			}
+			else
+			{
+				// Still declare named arguments, even if
+				// they are not passed from caller
+				// See bug #22203
+				our_env->declare_local(m_args[i].m_name);
+			}
 		}
 
 		// Add 'this'
@@ -201,7 +149,6 @@ swf_function::operator()(const fn_call& fn)
 		// Add 'super' (SWF6+ only)
 		if ( swfversion > 5 )
 		{
-			boost::intrusive_ptr<as_object> super = getSuper(*(fn.this_ptr));
 			our_env->set_local("super", as_value(super));
 		}
 
@@ -216,26 +163,43 @@ swf_function::operator()(const fn_call& fn)
 		our_env->add_local_registers(m_local_register_count);
 
 		// Handle the explicit args.
-		int	args_to_pass = imin(fn.nargs, m_args.size());
-		for (int i = 0; i < args_to_pass; i++)
+		for (size_t i=0, n=m_args.size(); i<n; ++i)
 		{
-			if (m_args[i].m_register == 0)
+			if ( ! m_args[i].m_register ) // not a register, declare as local
 			{
-				// Conventional arg passing: create a local var.
-				our_env->add_local(m_args[i].m_name, fn.arg(i));
+				if ( i < fn.nargs )
+				{
+					// Conventional arg passing: create a local var.
+					our_env->add_local(m_args[i].m_name, fn.arg(i));
+				}
+				else
+				{
+					// Still declare named arguments, even if
+					// they are not passed from caller
+					// See bug #22203
+					our_env->declare_local(m_args[i].m_name);
+				}
 			}
 			else
 			{
-				// Pass argument into a register.
-				int	reg = m_args[i].m_register;
-				our_env->local_register(reg) = fn.arg(i);
+				if ( i < fn.nargs )
+				{
+					// Pass argument into a register.
+					int	reg = m_args[i].m_register;
+					our_env->local_register(reg) = fn.arg(i);
+				}
+				else
+				{
+					// The argument was not passed, no
+					// need to setup a register I guess..
+				}
 			}
 		}
 
 		// Handle the implicit args.
 		// @@ why start at 1 ? Note that starting at 0 makes	
 		// intro.swf movie fail to play correctly.
-		uint8_t current_reg = 1;
+		boost::uint8_t current_reg = 1;
 		if (m_function2_flags & PRELOAD_THIS)
 		{
 			// preload 'this' into a register.
@@ -280,7 +244,7 @@ swf_function::operator()(const fn_call& fn)
 		if ( (m_function2_flags & PRELOAD_SUPER) && swfversion > 5)
 		{
 			// Put 'super' in a register (SWF6+ only).
-			our_env->local_register(current_reg).set_as_object(getSuper(*(fn.this_ptr)));
+			our_env->local_register(current_reg).set_as_object(super);
 			current_reg++;
 		}
 
@@ -291,14 +255,15 @@ swf_function::operator()(const fn_call& fn)
 		else if ( swfversion > 5 )
 		{
 			// Put 'super' in a local var (SWF6+ only)
-			our_env->add_local("super", as_value(getSuper(*(fn.this_ptr))));
+			our_env->add_local("super", as_value(super));
 		}
 
 		if (m_function2_flags & PRELOAD_ROOT) 
 		{
 			// Put '_root' in a register.
+			// NOTE: _lockroot will be hanlded by get_root()
 			our_env->local_register(current_reg).set_as_object(
-				our_env->get_target()->get_root_movie());
+				our_env->get_target()->get_root());
 			current_reg++;
 		}
 
@@ -319,13 +284,33 @@ swf_function::operator()(const fn_call& fn)
 		}
 	}
 
+	as_value result;
+
 	// Execute the actions.
-	//ActionExec exec(*m_action_buffer, *our_env, m_start_pc, m_length, fn.result, m_with_stack, m_is_function2);
-        as_value result;
-	ActionExec exec(*this, *our_env, &result, fn.this_ptr.get());
-	exec();
+	// Do this in a try block to proper drop the pushed call frame 
+	// in case of problems (most interesting action limits)
+	try 
+	{
+		//ActionExec exec(*m_action_buffer, *our_env, m_start_pc, m_length, fn.result, m_with_stack, m_is_function2);
+		ActionExec exec(*this, *our_env, &result, fn.this_ptr.get());
+		exec();
+	}
+	catch (ActionLimitException& ale) // expected and sane 
+	{
+		//log_debug("ActionLimitException got from swf_function execution: %s", ale.what());
+		throw;
+	}
+	catch (std::exception& ex) // unexpected but we can tell what it is
+	{
+		log_debug("Unexpected exception from swf_function execution: %s", ex.what());
+		throw;
+	}
+	catch (...) // unexpected, unknown, but why not cleaning up...
+	{
+		log_debug("Unknown exception got from swf_function execution");
+		throw;
+	}
 
-	our_env->popCallFrame();
         return result;
 }
 
