@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,11 +36,16 @@
 #include "render.h"
 #include "render_handler.h"
 #include "render_handler_agg.h"
+#include "SystemClock.h"
+#ifdef RENDERER_CAIRO
+#include "render_handler_cairo.h"
+#endif
 
 #include <cstdio>
 #include <string>
 #include <memory> // for auto_ptr
 #include <cmath> // for ceil and exp2
+#include <iostream>
 
 #define SHOW_INVALIDATED_BOUNDS_ON_ADVANCE 1
 
@@ -49,6 +54,9 @@
 #endif
 
 
+using std::cout;
+using std::endl;
+
 namespace gnash {
 
 MovieTester::MovieTester(const std::string& url)
@@ -59,6 +67,10 @@ MovieTester::MovieTester(const std::string& url)
 	// Initialize gnash code lib
 	gnashInit();
 
+	// Set base url *before* calling create_movie
+	// TODO: use PWD if url == '-'
+	set_base_url(url);
+
 	if ( url == "-" )
 	{
 		std::auto_ptr<tu_file> in (
@@ -68,13 +80,25 @@ MovieTester::MovieTester(const std::string& url)
 	}
 	else
 	{
+		URL urlObj(url);
+		if ( urlObj.protocol() == "file" )
+		{
+			RcInitFile& rcfile = RcInitFile::getDefaultInstance();
+			const std::string& path = urlObj.path();
+#if 1 // add the *directory* the movie was loaded from to the local sandbox path
+			size_t lastSlash = path.find_last_of('/');
+			std::string dir = path.substr(0, lastSlash+1);
+			rcfile.addLocalSandboxPath(dir);
+			log_debug(_("%s appended to local sandboxes"), dir.c_str());
+#else // add the *file* to be loaded to the local sandbox path
+			rcfile.addLocalSandboxPath(path);
+			log_debug(_("%s appended to local sandboxes"), path.c_str());
+#endif
+		}
 		// _url should be always set at this point...
-		_movie_def = gnash::create_library_movie(URL(url), NULL, false);
+		_movie_def = gnash::create_library_movie(urlObj, NULL, false);
 	}
 
-	// TODO: use PWD if url == '-'
-	set_base_url(url);
-
 	if ( ! _movie_def )
 	{
 		throw GnashException("Could not load movie from "+url);
@@ -83,7 +107,7 @@ MovieTester::MovieTester(const std::string& url)
 	// Initialize the sound handler(s)
 	initTestingSoundHandlers();
 
-	_movie_root = &(VM::init(*_movie_def).getRoot());
+	_movie_root = &(VM::init(*_movie_def, _clock).getRoot());
 
 	// Initialize viewport size with the one advertised in the header
 	_width = unsigned(_movie_def->get_width_pixels());
@@ -96,22 +120,29 @@ MovieTester::MovieTester(const std::string& url)
 	_movie_def->completeLoad();
 	_movie_def->ensure_frame_loaded(_movie_def->get_frame_count());
 
-	_movie = _movie_root->get_root_movie();
-	assert(_movie);
-
 	// Activate verbosity so that self-contained testcases are
 	// also used 
 	gnash::LogFile& dbglogfile = gnash::LogFile::getDefaultInstance();
 	dbglogfile.setVerbosity(1);
 
+	
+	std::auto_ptr<movie_instance> mi ( _movie_def->create_movie_instance() );
 
-	// Now place the root movie on the stage
-	advance();
+	// Set _movie before calling ::render
+	_movie = mi.get();
+
+	// Finally, place the root movie on the stage ...
+        _movie_root->setRootMovie( mi.release() );
+
+	// ... and render it
+	render();
 }
 
 void
 MovieTester::render(render_handler& h, InvalidatedRanges& invalidated_regions) 
 {
+	assert(_movie);
+
 	set_render_handler(&h);
 
 	h.set_invalidated_regions(invalidated_regions);
@@ -150,7 +181,7 @@ MovieTester::render()
 	_movie_root->add_invalidated_bounds(_invalidatedBounds, false);
 
 #ifdef SHOW_INVALIDATED_BOUNDS_ON_ADVANCE
-	cout << "frame " << _movie->get_current_frame() << ") Invalidated bounds " << _invalidatedBounds << endl;
+	std::cout << "frame " << _movie->get_current_frame() << ") Invalidated bounds " << _invalidatedBounds << std::endl;
 #endif
 
 	// Force full redraw by using a WORLD invalidated ranges
@@ -190,9 +221,23 @@ MovieTester::render()
 }
 
 void
-MovieTester::advance() 
+MovieTester::advanceClock(unsigned long ms)
 {
-	_movie_root->advance(1.0);
+	_clock.advance(ms);
+}
+
+void
+MovieTester::advance(bool updateClock)
+{
+	if ( updateClock )
+	{
+		// TODO: cache 'clockAdvance' 
+		float fps = _movie_def->get_frame_rate();
+		unsigned long clockAdvance = long(1000/fps);
+		advanceClock(clockAdvance);
+	}
+
+	_movie_root->advance();
 
 	render();
 
@@ -230,14 +275,14 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 	{
 		std::stringstream ss;
 	        ss << "exp:" << color.toShortString() << " ";
-		log_msg("UNTESTED: NORENDERER: pix:%d,%d %s %s", x,  y, ss.str().c_str(), label.c_str());
+		cout << "UNTESTED: NORENDERER: pix:" << x << "," << y << " exp:" << color.toShortString() << " " << label << endl;
 	}
 
 	FuzzyPixel exp(color, tolerance);
 	const char* X="";
 	if ( expectFailure ) X="X";
 
-	//cout <<"chekPixel(" << color << ") called" << endl;
+	//std::cout <<"chekPixel(" << color << ") called" << std::endl;
 
 	for (TRenderers::const_iterator it=_testingRenderers.begin(), itE=_testingRenderers.end();
 				it != itE; ++it)
@@ -255,16 +300,13 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 	        if ( ! handler.getAveragePixel(obt_col, x, y, radius) )
 		{
 			ss << " is out of rendering buffer";
-			log_msg("%sFAILED: %s (%s)", X,
-					ss.str().c_str(),
-					label.c_str()
-					);
+			cout << X << "FAILED: " << ss.str() << " (" << label << ")" << endl;
 			continue;
 		}
 
 		// Find minimum tolerance as a function of BPP
 
-		unsigned short minRendererTolerance = tolerance;
+		unsigned short minRendererTolerance = 1;
 		unsigned int bpp = handler.getBitsPerPixel();
 		if ( bpp ) 
 		{
@@ -272,7 +314,8 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 			minRendererTolerance = int(ceil(exp2(8 - bpp/3)));
 		}
 
-		unsigned short tol = std::max(tolerance, minRendererTolerance);
+		//unsigned short tol = std::max(tolerance, minRendererTolerance);
+		unsigned short tol = tolerance*minRendererTolerance; 
 
 	        ss << "exp:" << color.toShortString() << " ";
 	        ss << "obt:" << obt_col.toShortString() << " ";
@@ -281,17 +324,11 @@ MovieTester::checkPixel(int x, int y, unsigned radius, const rgba& color,
 		FuzzyPixel obt(obt_col, tol);
 		if (exp ==  obt) // equality operator would use tolerance of most tolerating FuzzyPixel
 		{
-			log_msg("%sPASSED: %s %s", X,
-					ss.str().c_str(),
-					label.c_str()
-					);
+			cout << X << "PASSED: " << ss.str() << " (" << label << ")" << endl;
 		}
 		else
 		{
-			log_msg("%sFAILED: %s %s", X,
-					ss.str().c_str(),
-					label.c_str()
-					);
+			cout << X << "FAILED: " << ss.str() << " (" << label << ")" << endl;
 		}
 	}
 }
@@ -391,34 +428,40 @@ MovieTester::initTestingRenderers()
 	// Initialize AGG
 	static const char* aggPixelFormats[] = {
 		"RGB555", "RGB565", "RGBA16",
-		"RGB24", "BGR24", "RGBA32", "BGRA32"
+		"RGB24", "BGR24", "RGBA32", "BGRA32",
+		"ARGB32", "ABGR32"
 	};
 
 	for (unsigned i=0; i<sizeof(aggPixelFormats)/sizeof(*aggPixelFormats); ++i)
 	{
 		const char* pixelFormat = aggPixelFormats[i];
-		std::string name = "AGG_"+string(pixelFormat);
+		std::string name = "AGG_" + std::string(pixelFormat);
 
 		handler.reset( create_render_handler_agg(pixelFormat) );
 		if ( handler.get() )
 		{
-			//log_msg("Renderer %s initialized", name.c_str());
-			cout << "Renderer " << name << " initialized" << endl;
+			//log_debug("Renderer %s initialized", name.c_str());
+			std::cout << "Renderer " << name << " initialized" << std::endl;
 			addTestingRenderer(handler, name); 
 		}
 		else
 		{
-			cout << "Renderer " << name << " not supported" << endl;
+			std::cout << "Renderer " << name << " not supported" << std::endl;
 		}
 	}
 #endif // RENDERER_AGG
 
 #ifdef RENDERER_CAIRO
 	// Initialize Cairo
+	handler.reset(renderer::cairo::create_handler());
+
+        addTestingRenderer(handler, "Cairo");
 #endif
 
 #ifdef RENDERER_OPENGL
 	// Initialize opengl renderer
+	handler.reset(create_render_handler_ogl(false));
+	addTestingRenderer(handler, "OpenGL");
 #endif
 }
 
@@ -427,21 +470,21 @@ MovieTester::addTestingRenderer(std::auto_ptr<render_handler> h, const std::stri
 {
 	if ( ! h->initTestBuffer(_width, _height) )
 	{
-		cout << "UNTESTED: render handler " << name
+		std::cout << "UNTESTED: render handler " << name
 			<< " doesn't support in-memory rendering "
-			<< endl;
+			<< std::endl;
 		return;
 	}
 
 	// TODO: make the core lib support this
 	if ( ! _testingRenderers.empty() )
 	{
-		cout << "UNTESTED: can't test render handler " << name
+		std::cout << "UNTESTED: can't test render handler " << name
 			<< " because gnash core lib is unable to support testing of "
 			<< "multiple renderers from a single process "
 			<< "and we're already testing render handler "
 			<< _testingRenderers.front()->getName()
-			<< endl;
+			<< std::endl;
 		return;
 	}
 
@@ -470,13 +513,13 @@ MovieTester::initTestingSoundHandlers()
 {
 
 #ifdef SOUND_SDL
-	cout << "Creating SDL sound handler" << endl;
-        _sound_handler.reset( gnash::create_sound_handler_sdl() );
+	std::cout << "Creating SDL sound handler" << std::endl;
+        _sound_handler.reset( gnash::media::create_sound_handler_sdl() );
 #elif defined(SOUND_GST)
-	cout << "Creating GST sound handler" << endl;
-        _sound_handler.reset( gnash::create_sound_handler_gst() );
+	std::cout << "Creating GST sound handler" << std::endl;
+        _sound_handler.reset( gnash::media::create_sound_handler_gst() );
 #else
-	cerr << "Neigher SOUND_SDL nor SOUND_GST defined" << endl;
+	std::cerr << "Neigher SOUND_SDL nor SOUND_GST defined" << std::endl;
 	return;
 	//exit(1);
 #endif
@@ -484,4 +527,15 @@ MovieTester::initTestingSoundHandlers()
 	gnash::set_sound_handler(_sound_handler.get());
 }
 
+void
+MovieTester::restart() 
+{
+	_movie_root->clear(); // restart();
+	_movie = _movie_def->create_movie_instance();
+	_movie_root->setRootMovie(_movie);
+
+	// Set _movie before calling ::render
+	render();
+}
+
 } // namespace gnash
