@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,10 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
 #include "URL.h"
+#include "VM.h"
+#include "string_table.h"
 
 #include "check.h"
 #include <string>
@@ -45,7 +46,7 @@ getCoverArt()
 	character* coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
 	sprite_instance* coverart = coverartch->to_movie();
 
-	//log_msg("Coverart is %p, displaylist is:", coverart);
+	//log_debug("Coverart is %p, displaylist is:", coverart);
 	//coverart->getDisplayList().dump();
 
 	return coverart;
@@ -87,15 +88,19 @@ main(int /*argc*/, char** /*argv*/)
 	URL offspringURL("offspring.swf", mediaURL);
 	std::string url;
 
+	gnash::RcInitFile& rc = gnash::RcInitFile::getDefaultInstance();
+	rc.addLocalSandboxPath(MEDIADIR);
+
+
 
 	root = tester->getRootMovie();
 	assert(root.get());
 
-	check_equals(root->get_frame_count(), 1);
+	check_equals(root->get_frame_count(), 2);
 	check_equals(root->get_current_frame(), 0);
 
 	tester->advance();
-	check_equals(root->get_current_frame(), 0);
+	check_equals(root->get_current_frame(), 1);
 
 	// Verify that 'coverart' exists and is empty
 	character* coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
@@ -111,7 +116,8 @@ main(int /*argc*/, char** /*argv*/)
 	tester->movePointerTo(80, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->pressMouseButton();
-	sleep(1); // give it some time...
+	tester->advance(); // loads (should) happen on next advance
+	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
 	coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
 	check(coverart != coverartch->to_movie());
 	coverart = coverartch->to_movie();
@@ -121,11 +127,16 @@ main(int /*argc*/, char** /*argv*/)
 	// Check scribbling on the lynch
 	checkScribbling();
 
+	// Run 'coverart' tests..
+	tester->movePointerTo(640,180);
+	tester->click(); tester->advance();
+
 	// Click on the second (green)
 	tester->movePointerTo(280, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->click();
-	sleep(1); // give it some time...
+	tester->advance(); // loads (should) happen on next advance
+	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
 	coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
 	coverart = coverartch->to_movie();
 	check_equals(coverart->get_movie_definition()->get_url(), greenURL.str());
@@ -135,16 +146,43 @@ main(int /*argc*/, char** /*argv*/)
 	// Check scribbling on the jpeg
 	checkScribbling();
 
+	// Run 'coverart' tests..
+	tester->movePointerTo(640,180);
+	tester->click(); tester->advance();
+
 	// Click on the third (offspring)
 	tester->movePointerTo(480, 80);
 	check(tester->isMouseOverMouseEntity());
 	tester->click();
-	sleep(1); // give it some time to load
+	tester->advance(); // loads (should) happen on next advance
+	usleep(500); // give it some time... TODO: drop this test and use a self-containment instead
 	coverartch = const_cast<character*>(tester->findDisplayItemByName(*root, "coverart"));
 	coverart = coverartch->to_movie();
 	check_equals(coverart->get_movie_definition()->get_url(), offspringURL.str());
 
 	// Check scribbling on the offspring
 	checkScribbling();
+
+	// Run 'coverart' tests..
+	tester->movePointerTo(640,180);
+	tester->click(); tester->advance();
+
+	// Get summary ...
+	tester->pressKey(key::SHIFT);
+	tester->click(); tester->advance();
+	tester->releaseKey(key::SHIFT);
+
+	// Consistency checking
+	string_table& st = root->getVM().getStringTable();
+	as_value eot;
+	// It's an swf6, so lowercase 'END_OF_TEST'
+	bool endOfTestFound = root->get_member(st.find("end_of_test"), &eot);
+	check(endOfTestFound);
+	if ( endOfTestFound )
+	{
+		//cerr << eot.to_debug_string() << endl;
+		check_equals(eot.to_bool(), true);
+	}
+
 }
 
