@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,6 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
 #include "GnashException.h"
 
@@ -126,7 +125,7 @@ main(int /*argc*/, char** /*argv*/)
 			// visually, it seems so, but loop-back is too complex
 			// to be sure (ie: xtrace window text might be reset or something)
 			// I checked that it's not resetDisplayList invalidating it...
-			check( tester.getInvalidatedRanges().isNull() );
+			xcheck( tester.getInvalidatedRanges().isNull() );
 		}
 		else // we did nothing here...
 		{
