@@ -3,7 +3,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -34,13 +34,18 @@ add_xtrace_function_clip(SWFMovieClip mc, SWFBlock font, int depth, int x, int y
 {
 	SWFTextField tf;
 	SWFDisplayItem it;
+	int flags;
 
 	tf = newSWFTextField();
 
 	SWFTextField_setFont(tf, font);
 
 	/* setting flags seem unneeded */
-	/*SWFTextField_setFlags(tf, SWFTEXTFIELD_USEFONT|SWFTEXTFIELD_NOEDIT);*/
+	flags = 0;
+	//flags |= SWFTEXTFIELD_USEFONT;
+	flags |= SWFTEXTFIELD_WORDWRAP;
+	//flags |= SWFTEXTFIELD_NOEDIT;
+	SWFTextField_setFlags(tf, flags);
 
 	/* Add all ascii chars */
 	SWFTextField_addChars(tf, asciichars);
@@ -161,10 +166,7 @@ make_fill_square(int x, int y, int width, int height, byte or, byte og, byte ob,
 static SWFAction
 get_dejagnu_actions()
 {
-	static const size_t BUFLEN = 4096;
-
-	char buf[BUFLEN];
-	snprintf(buf, BUFLEN,
+	static const char *buf = 
 		"TestState = function() {\n"
 		" this.passed = 0;\n"
 		" this.failed = 0;\n"
@@ -173,6 +175,10 @@ get_dejagnu_actions()
 		" this.untest = 0;\n"
 		" this.unresolve = 0;\n"
 		"};\n"
+		"TestState.prototype.note = function (msg) {\n"
+		" _root.xtrace(msg);\n"
+		" trace (msg); "
+		"};\n"
 		"TestState.prototype.fail = function (why) {\n"
 		" this.failed++;\n"
 		" var msg = 'FAILED: '+why;\n"
@@ -200,19 +206,39 @@ get_dejagnu_actions()
 		// " _root.xtrace(msg);\n"
 		" trace (msg); "
 		"};\n"
-		"TestState.prototype.totals = function() {\n"
-		" _root.xtrace('#passed: '+ this.passed);\n"
-		" trace('#passed: '+ this.passed);\n"
-		" _root.xtrace('#failed: '+ this.failed);\n"
-		" trace('#failed: '+ this.failed);\n"
+		"TestState.prototype.printtotals = function() {\n"
+		" this.note('#passed: '+ this.passed);\n"
+		" this.note('#failed: '+ this.failed);\n"
 		" if ( this.xpassed ) {\n"
-		"   _root.xtrace('#unexpected successes: '+ this.xpassed);\n"
-		"   trace('#unexpected successes: '+ this.xpassed);\n"
+		"   this.note('#unexpected successes: '+ this.xpassed);\n"
 		" }\n"
 		" if ( this.xfailed ) {\n"
-		"   _root.xtrace('#expected failures: '+ this.xfailed);\n"
-		"   trace('#expected failures: '+ this.xfailed);\n"
+		"   this.note('#expected failures: '+ this.xfailed);\n"
 		" }\n"
+		" this.note('#total tests run: '+ this.testcount());\n"
+		"};\n"
+		"TestState.prototype.totals = function(exp, msg) {\n"
+		" var obt = this.testcount(); "
+		" if ( exp != undefined && obt != exp ) { "
+		"   this.fail('Tests run '+obt+' (expected '+exp+') ['+msg+']'); "
+		" }"
+		" this.printtotals();"
+		"};\n"
+		"TestState.prototype.xtotals = function(exp, msg) {\n"
+		" var obt = this.testcount(); "
+		" if ( exp != undefined && obt != exp ) { "
+		"   this.xfail('Tests run '+obt+' (expected '+exp+') ['+msg+']'); "
+		" } else {"
+		"   this.xpass('Tests run: '+obt+' ['+msg+']'); "
+		" }"
+		" this.printtotals();"
+		"};\n"
+		"TestState.prototype.testcount = function() {\n"
+		" c=this.passed;\n"
+		" c+=this.failed;\n"
+		" if ( this.xpassed ) c+=this.xpassed;\n"
+		" if ( this.xfailed ) c+=this.xfailed;\n"
+		" return c;"
 		"};\n"
 		"_root.runtest = new TestState();\n"
 		"_root.check_equals = function(obt, exp) {\n"
@@ -247,11 +273,13 @@ get_dejagnu_actions()
 		" _root.xtrace(msg);\n"
 		" trace(msg);\n"
 		"};\n"
-		"_root.totals = function() {\n"
-		" _root.runtest.totals();\n"
+		"_root.totals = function(exp, info) {\n"
+		" _root.runtest.totals(exp, info);\n"
 		"};\n"
-		"_root.dejagnu_module_initialized = 1;\n"
-	);
+		"_root.xtotals = function(exp, info) {\n"
+		" _root.runtest.xtotals(exp, info);\n"
+		"};\n"
+		"_root.dejagnu_module_initialized = 1;\n";
 
 	return compileSWFActionCode(buf);
 }
