@@ -1,5 +1,5 @@
 /* 
- *   Copyright (C) 2005, 2006, 2007 Free Software Foundation, Inc.
+ *   Copyright (C) 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,8 +24,8 @@
 #include "sprite_instance.h"
 #include "character.h"
 #include "dlist.h"
-#include "container.h"
 #include "log.h"
+#include "VM.h"
 
 #include "check.h"
 #include <string>
@@ -65,9 +65,12 @@ main(int /*argc*/, char** /*argv*/)
 	check(loaded);
 	check_equals(loaded->get_parent(), root);
 
+	string_table& st = VM::get().getStringTable();
+	string_table::key xscale = st.find("_xscale");
+	string_table::key yscale = st.find("_yscale");
 	// we need a const_cast as get_member *might* eventually
 	// change the character (fetching _x shouldn't change it though)
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(tmp, as_value(50));
 
 	check_equals(loaded->get_height(), 2056);
@@ -86,25 +89,25 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(string(text->get_text_value()), "50");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "48");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 48);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 48);
 	tester.pressMouseButton();
 	check_equals(string(text->get_text_value()), "48");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "46");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 46);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 46);
 	tester.pressMouseButton();
 	check_equals(string(text->get_text_value()), "46");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "44");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 44);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 44);
 
 	// click some on the "larger" button
@@ -114,33 +117,33 @@ main(int /*argc*/, char** /*argv*/)
 	check_equals(string(text->get_text_value()), "44");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "46");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 46);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 46);
 	tester.pressMouseButton();
 	check_equals(string(text->get_text_value()), "46");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "48");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 48);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 48);
 	tester.pressMouseButton();
 	check_equals(string(text->get_text_value()), "48");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "50");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 50);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 50);
 	tester.pressMouseButton();
 	check_equals(string(text->get_text_value()), "50");
 	tester.depressMouseButton();
 	check_equals(string(text->get_text_value()), "52");
-	check(const_cast<character*>(loaded)->get_member("_xscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(xscale, &tmp));
 	check_equals(round(tmp.to_number()), 52);
-	check(const_cast<character*>(loaded)->get_member("_yscale", &tmp));
+	check(const_cast<character*>(loaded)->get_member(yscale, &tmp));
 	check_equals(round(tmp.to_number()), 52);
 
 
